#!/usr/bin/perl

# gedail_pack.pm -- Jean-Franois BAQUET -- Aot 2000
# gedail@baquet.org - http://www.baquet.org

# thanks to Christian GENNERAT for his contribution
# to the 1.40 version and linux support -- Mai 2002

# realized on a GedcomToHTML.pl - 1.45 basis  
# g2h.pl 1.45 -- Dan Pidcock -- November 1998
# D.Pidcock@bath.ac.uk - http://www.bath.ac.uk/~enpdp

# generates a Web genealogical site from a Gedcom file
# based on the gedcom 5.5 standard, doesn't handle all 
# gedcom elements.

# The geneanet index generation module use parts of
# GedcomToGeneanet.pl - version 6.00 -- 2000/01/11
# Copyright Geneanet 1997-1998 http://www.geneanet.org

# Copyright (C) 2002

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 2 of the License.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 

# Feel free to use this program, but please keep the authorship the
# same.  If you have any changes you have made, I would be happy to
# incorporate them and put your name on the credits. - J.F.Baquet

package gedail_pack;
use Exporter;
@ISA = qw(Exporter);
@EXPORT = qw(gedail_main gedail_vers process_accents jpegsize bincopy);

eval { require Compress::Zlib };
$has_Compress_Zlib = $@ ? 0 : 1;
if ($gzip and $has_Compress_Zlib) { use IO::Zlib; } # for gzip
else { $gzip = 0; }

sub gedail_vers {
	require 'gediopt.pm';
	return wantarray ? ('2.10a',$msg_lang) : '2.10a'; # 13/07/2006

}
sub gedail_main {

################################################################################
#                             Gedail default options                           #
################################################################################

### 0 Basic options
#
$file_pref = 'Gedi'; # Prefix for file names (1 to 4 Alphanumerics)
					 # this is useful if you have several genealogies to process
$tpl_frames = 1;	# Templates with frames (0:without)
$sosa_only = 0;		# process root ancestors only
$desc_only = 0;		# process descendants only
$est_date = 1;		# date estimation (1:yes)
$conv_date = 1;		# French revolutionnar dates conversion
$est_place = 1;		# place estimation (1:yes)
$cd = 1;			# Include files for CD autoload

### 1 Individuals files processing options
#
$ind_opt = 1;		# Individuals charts option
$print_notes = 0;	# Print individual's notes level 1
$print_notes_2 = 0;	# Print individual's notes level 2
$print_refn = 0;	# Print individual's reference
$print_sosa = 1;	# Print Sosa-Stradonitz number
$print_desc = 0;	# Print d'Aboville number
$print_events = 1;	# Print events
$print_sources = 0;	# Print sources
$full_place = 1;	# Publish full place, or only city in events
$full_date = 1;		# Publish full date, or only year
$hide_XX = 1;		# Make 20th century dates privates
$hide_XX_pics = 0;	# Make 20th century pictures private
$check_images = 1;	# Check for images 

### 2 Names processing options
#
$nam_opt = 1;		# Surnames output option
$nam_by_letter = 0;	# Letter by letter Index for Names 
$name_asis = 0;		# No name formatting

### 3 Places processing options
#
$loc_opt = 1;		# Locations output option
$loc_by_letter = 0;	# Letter by letter Index for Places
$place_links = 1;	# Places links files in Gedail directory
$place_format = "";	# Places format (default beginning by city)  
$place_index = "2";	# Places index zones  (in descending order)
#################################################################################################### 
# zones codification : 
# 0 : ignored
# 1 : hamlet or parrish
# 2 : city
# 3 : zipcode
# 4 : county / dpartement
# 5 : state	/ region
# 6 : country
# [] : optionnal
# space : blank space
#################################################################################################### 
# Samples :  
#$place_format = "2";			# city
#$place_format = "2[,0]";		# city[, ignored] 
#$place_format = "2[,1],6";		# city[,hamlet],country
#$place_format = "1,2,5";		# hamlet, city, state
#$place_format = "1,4,5";		# city, county, state
#$place_format = "1,2,4,3,5,6";		# hamlet or parrish, city, county, zipcode, state, country
#$place_format = "1 2 3 4 5 6";		# hamlet or parrish city code county state country  
#$place_format = "1,2,3,4[,5],6";	# hamlet or parrish,city,code,departement[,region],country
#$place_format = "2,3,4,5,6,1";		# city,zipcode,county,state,country,subdivision
#$place_format = "2[,3],4,5,6";		# city[,zipcode],county,state,country
#$place_format = "1;2;3;4;5;6";		# hamlet;city;zipcode;county;state;country
#$place_format = "2[, 1][ (3)]";	# city[, parrish][ (zipcode)]
#$place_format = "2[ (6/5)]";		# city (country/state)
#$place_index = "2";			# city  
#$place_index = "32";			# zipcode city
#$place_index = "652";			# country state city
#$place_index = "65432";		# country state county zipcode city
#################################################################################################### 

### 4 Ascending tree processing options
#
$gen_opt = 1;		# Ascendants processing option (0:none) 
$gen_number = 30;	# Generations output number (0:none)
$name_root = '';	# sosa root name 'forenames /surname/' ( first occurence is taken )
$indi_root = '';	# sosa root identifier in INDI field  (ex : 'I27')
			# may be read in the .ged file with a text editor like wordpad

### 5 Descending tree processing options
#
$dsc_opt = 1;		# Descendants processing option (0:none) 
$desc_number_min = 0;	# Minimum descendants number required (0:any)
$desc_name_min = 50;	# Minimum name owners required for default processing (0:any)
$desc_sosas_only = 0;	# Descendants of sosas only
$desc_by_name = 1;	# Name by name Index for Descendants
@desc_names  = ();		# Surnames list separated by commas (default root surname, ('*'):all)

### 6 Flash list file processing options 
##  this .csv file may be read with excel or sent by mail in text format to other genealogists  
#
$fli_opt = 1;		# Create flash list .csv file
$fli_geneanet = 1;	# Geneanet format, with place;region;subregion;country
$fli_fq_only = 1;	# full qualified (surname and place) only
$fli_sosa_only = 0;	# sosas surnames only

### 7 Marriage list file processing options 
##  this .csv file may be read with excel or sent by mail in text format to other genealogists  
#
$mli_opt = 0;		# Create marriages list .csv file
$mli_jftools = 1;	# Jftools format, with place;region;subregion;country
$mli_fq_only = 1;	# full qualified (defined place) only
$mli_sosa_only = 0;	# sosas surnames only


### 8 Main page options
#
$main_page  = 1;	# Gedail return page is $pref_ind.html (0:$home_page)
$main_header  = '';	# Header text in index page
$main_footer  = '';	# Footer text in index page
$main_left  = '';	# Left text in index page
$main_right  = '';	# Right text in index page
$base_title = '';	# base identification (no identification if omitted)
$banner_pic = ''; # banner image
$banner_url = ''; # banner URL
$aut_pic  = '';	    # base author picture
$aut_name  = '';	# base author name (from SUBM NAME if omitted)
$aut_addr1 = '';	# base author address 1st line (from SUBM ADDR if omitted)
$aut_addr2 = '';	# base author address 2nd line (from SUBM CONT if omitted)
$aut_addr3 = '';	# base author address 3rd line (from SUBM CONT if omitted)
$aut_addr4 = '';	# base author address 4th line (from SUBM CONT if omitted)
$aut_ctry  = '';	# base author country (from SUBM CTRY if omitted)
$aut_phone = '';	# base author phone number (from SUBM PHON if omitted)
$aut_email = '';	# base author email address (from SUBM EMAIL if omitted)
$aut_file  = 0;	# base author gedcom file download (no download if omitted)
$aut_opt  = 0;	# base author option file download (no download if omitted)
$counter = 1;		# compteur de visites de la page index
$counter_init = 1;	# forcage de la valeur initiale du compteur de visites de la page index
$home_page = '';	# User exit page (default none)
 
### Generation Language options
## You can write your own templates LNNN-xx, and your gedigen_xx.opt for an xx language 
#  
$gen_lang = 'en';		# Generation language; must have template files for this language, may be a list.

### 9 Advanced options 
## find a very good reason to change it
#
$php = 0;				# Send compressed pages using uz.php
$place_out = '';		# Output places format  (ex : $place_out = '2, 6"; => city, country)
@priv_refs = (PRV,PRIV);	# Make information private for these REFN
$priv_name = '';		# Replacement name for privates individuals
$priv_note   = '#';		# Private note prefix char
$priv_string = '(XX)';		# Date string	for XX century individuals
$sosa_link = '<sup>#</sup>';	# sosa indicator character
$line_link = '<sup>*</sup>';	# lineage indicator character
$desc_indent = '- ';		# Descendants indentation
$css_style = 'gedi';		# CSS file name
$tpl_dir_frames = 'B001';	# Directory name (or prefix) where the HTML templates are located
$tpl_dir_noframes = 'B000';	# Directory name (or prefix) where the HTML templates are located
				# the naming convention is LNNN-ll
				# LNNN : Template number (unique identifier)
				# -ll  : Language (de, en, fr, nl .. you can create your own translation)
$indf_size = 15;		# Number of individuals by file
$namf_size = 100;		# Number of individuals by people file
$namx_size = 500;		# Number of items by name index page (0:one letter by page)
$locf_size = 100;		# Number of individuals by location file
$locx_size = 500;		# Number of items by location index page (0:one letter by page)
$dscf_size = 200;		# Number of individuals by descendants file
$dscx_size = 500;		# Number of items by descendants index page (0:one letter by page)
$genf_size = 64;		# Number of individuals by generation file
$sour_size = 250;		# Number of lines by source file

### 10 Gedpics Options
#
$photo_max_size = '640x480';
$photo_thumb_size = '56x42';
$photo_gallery = '10x5';

### Expert
#
$out_dir = '';				# Directory where the HTML files will be stored
$photo_dir = '';			# Directory where the individual's picture files are
$dsc_max_index = 30;		# Maximum number of names for alpha index
$refn_root = 'root';		# sosa root identifier in REFN field (ex : '1' or 'root')
$gnetdef_country = 'FRA';	# Geneanet default country
$gnetdef_region = 'NPC';	# Geneanet default region
$gnetdef_subregion = '';	# Geneanet default subregion
$charset = 'ANSI';			# ANSI, IBMPC, MAC, ANSEL, UFT-8 translation in ANSI
$gzip = '0';				# Compression

### File naming options : suffixes to be addended at $file_pref
#
$sour_path = 'sources';		# Path to the sources files on the Web server
$pages_path = 'pages';		# Path to the web files on the Web server
$photos_path = 'photos';	# Path to the ancestors pictures on the Web server
$pics_path = 'pics';		# Path to the web graphisms on the Web server
$pics_dir = 'Pics';			# Directory where the PICS used by gedail are
$indx_suff = '_ind';    # home index file
$indf_suff = '0000';    # individuals files
$locx_suff = 'lx00';    # places index files
$locf_suff = 'l000';    # places files
$namx_suff = 'nx00';    # names index files
$namf_suff = 'n000';    # names files
$genx_suff = 'gx00';    # sosas index file
$genf_suff = 'g000';    # sosas files
$desx_suff = 'dx00';    # descendants index file
$desf_suff = 'd000';    # descendants files
$flif_suff = 'flst';    # flash list .csv file
$mlif_suff = 'marr';    # marriages .csv file
$sour_suff = 's000';    # sources .txt files

# Don't touch anything else below unless you know what's happenin'
 ($version,$msg_lang) = gedail_vers(); $website = 'www.baquet.org'; $save_max = 5; print "** gedail version $version - (c) J-F Baquet 2000-2004 **\n"; $msg_lang	= 'en' unless ( -f "gedimsg_$msg_lang.pm" ); require "gedimsg_$msg_lang.pm";
 ($file_ged,$file_opt) = @_; if ($^O eq 'MSWin32') { $msg_sys = 'DOS'; } if ( !$file_ged ) { print "\n****************************************************\n"; &mess_print('m_ged_undef'); &mess_print('m_auto_scan'); &mess_print('m_to_stop'); &mess_print('m_sel_y_n');
 print "****************************************************\n"; $i_save = 0; if (open( SAV_FILE, "gedisave.txt" )) { while (<SAV_FILE>) { chomp; $file_save[$i_save] = $_;  $i_save++; } close SAV_FILE ; } for ($i=0;$i<$i_save;$i++) { $gedfile[$i] = $file_save[$i]; &mess_print( 'm_ged_file' );
 chomp($kb_parms = <STDIN>); if ( $kb_parms =~ /^(o|j|y).*/i ) { $file_ged = $file_save[$i]; $file_ged =~ /(.*)\.(ged|GED)$/; if ( !$file_opt ) { $file_opt = $1 . ".opt"; } $i = $i_save; } elsif ( $kb_parms !~ /^n.*/i ) {  die "\n*********************** stop ***********************\n"; } }
 if ( !$file_ged ) { &mess_print('m_continue'); chomp($kb_parms = <STDIN>); if ( $kb_parms !~ /^(o|j|y).*/i ) {  die "\n*********************** stop ***********************\n"; } $retcode = 1; $disk = "C"; while ( ( $retcode >= 0 ) and ( !$file_ged ) ) { $retcode = scan_ged($disk.":");
 if ( $retcode > 0 ) { $i = 0; while ( ( !$file_ged ) and ( $i < $retcode ) ) { &mess_print( 'm_ged_file' ); $kb_parms = <STDIN>; chomp($kb_parms); if ( !$kb_parms ) { die "\n*********************** stop ***********************\n"; } elsif ( $kb_parms =~ /^(o|j|y).*/i ) { $file_ged = $gedfile[$i];
 $file_ged =~ /(.*)\.(ged|GED)$/; if ( !$file_opt ) { $file_opt = $1 . ".opt"; } } elsif ( $kb_parms =~ /^(n|N).*/ ) { $i++; } } } $disk++; if ( $disk > "Z" ) { $retcode = -1; } } } if ( !$file_ged ) { die "\n************* file not found *****************\n"; } }
 $file_ged =~ /^(.*?)((\w|~)*)\.(ged|GED)$/; $ged_dir = $1; $gedcom_id = $2; ( $sec, $min, $heure, $jour, $mois, $an, $sjour, $ajour, $isdst ) = localtime(time); $mois++; $an += 1900; $hmin=sprintf("%dh%02d",$heure,$min); $curr_year = $an; $curr_date = "$an-$mois-$jour"; $tpl_dir = '';
 if ( open( OPTIONS, "$file_opt" ) ) { while (<OPTIONS>) { eval $_ } close OPTIONS ;						  $option_id = $gedcom_id; } elsif ( open( OPTIONS, "default.opt" ) ) { &mess_print('m_q_def_opt'); while (<OPTIONS>) { eval $_ } close OPTIONS ; $file_opt = "default.opt"; $option_id = "default"; }
 else { &mess_print('e_n_def_opt'); $file_opt = ""; $aut_opt = 0; } if (!$desc_number_min and $desc_number) { $desc_number_min = $desc_number; } elsif (!$desc_number_min) { $desc_number_min = 0; } if (!$gen_number) { $gen_opt = 0; } if (!$gen_opt) { $gen_number = 0; } if ($nam_by_letter) { $namx_size = 0; }
 if ($loc_by_letter) { $locx_size = 0; } if ($by_letter) { $locx_size = 0; $namx_size = 0; } if ($sosa_only) { $mli_sosa_only = 1; $fli_sosa_only = 1; } &set_place_mask; if ( $file_pref !~ /^[A-Za-z]{1,4}$/ ) { die "invalid file name prefix, must be 1 to 4 alphabetics\n"; }
 if ($aut_home and !$main_page) { $home_page = $aut_home; } if (!$main_page and $home_page) { $main_img = "home"; $main_page = $home_page; } else { $main_page = $file_pref . "_ind.html"; $main_img = "accueil"; } if ($home_page) { $home_img = "home"; } else { $home_img = "null"; } $indx_name = $file_pref . $indx_suff;
 $indf_name = $file_pref . $indf_suff; $locx_name = $file_pref . $locx_suff; $locf_name = $file_pref . $locf_suff; $namx_name = $file_pref . $namx_suff; $namf_name = $file_pref . $namf_suff; $genx_name = $file_pref . $genx_suff; $genf_name = $file_pref . $genf_suff;
 $dscx_name = $file_pref . $desx_suff; $dscf_name = $file_pref . $desf_suff; $flif_name = $file_pref . $flif_suff;					    $mlif_name = $file_pref . $mlif_suff; $sour_name = $file_pref . $sour_suff; if ( !$out_dir ) { if ( $ind_opt or $loc_opt or $nam_opt or $gen_opt ) { $out_dir = $ged_dir . "$file_pref"; }
 else { $out_dir = $ged_dir;  $out_dir =~ s/^(.*?)[\/\\]$/$1/; } } elsif ( $out_dir =~ /^(\.\/|\.\\|\/|\\)(.*?)[\/\\]+$/ ) { $out_dir = $ged_dir.$2; } elsif ( $out_dir =~ /^(\.\/|\.\\|\/|\\)(.*?[^\/\\])$/ ) { $out_dir = $ged_dir.$2; } elsif ( $out_dir =~ /^([^\/\\]*)$/ ) { $out_dir = $ged_dir . $1; } elsif ( $out_dir =~ /^(\.\.\/|\.\.\\)(.*)$/ ) { die "Couldn't create $out_dir directory\n"; } if ( !-e $out_dir ) { unless ( mkdir( $out_dir, 0755 ) ) { die "Couldn't create $out_dir directory\n"; } }
 unless ( open( LOG_FILE, ">$out_dir/gedail.log" ) ) { die "\nCouldn't open log file $out_dir/gedail.log\n"; } if ($check_images) { if ( !-e "$out_dir/$photos_path" ) { unless ( mkdir( "$out_dir/$photos_path", 0755 ) ) { die "Couldn't create $out_dir/$photos_path directory\n"; } }
 $gallery = $file_pref.'px00'; if ( -e "$out_dir/$photos_path/$gallery.html" and $photo_gallery and ($photo_gallery ne '0x0') ) { $pics_exist = 1 }	 else { $pics_exist = 0 }			 if ( $photo_max_size =~ /(\d+)x(\d+)/i) { $photo_max_width = $1; $photo_max_height = $2; } else {  $photo_max_width = 640;
 $photo_max_height = 480; } if ( $photo_thumb_size =~ /(\d+)x(\d+)/i) { $photo_thumb_width = $1; $photo_thumb_height = $2; } else {  $photo_thumb_width = 56; $photo_thumb_height = 42; } $obj_num = 0; } if ((!$gen_lang) and $lang) {  $gen_lang = $lang; } if (!$gen_lang) {  $gen_lang = $msg_lang; } @tablang = split(/,/,$gen_lang);
 if ( open( OPTIONS, "gedigen_$tablang[0].opt" ) ) { while (<OPTIONS>) { eval $_ } close OPTIONS ;						    } if ($print_events) { foreach $event ( keys %str_events ) {  if (!$events_list) { $events_list = $event; } else { $events_list .= '|'.$event; } if ($event eq 'OCCU') {	$event_occu = 1; } elsif ($event eq 'TITL') {	$event_titl = 1; } elsif ($event eq 'BIRT') {	$event_birt = 1; } elsif ($event eq 'DEAT') {	$event_deat = 1; } elsif ($event eq 'MARR') {	$event_marr = 1; } } }
 &log_text("-------------------- Gedail $vers started $jour/$mois/$an $msg_lang"); $start_time = time(); if (!$tpl_dir) { if (!$tpl_frames) {	$tpl_dir = $tpl_dir_noframes;	} else {	$tpl_dir = $tpl_dir_frames;	} } &mess_print('e_templ_dir'); &log_text("Templates directory : $tpl_dir"); &mess_print('m_w_opt_fil');
 &log_text("Processing option file $file_opt");		 if ( open( OPTIONS, "$file_opt" ) ) { while (<OPTIONS>) { print LOG_FILE $_ } close OPTIONS ; print LOG_FILE "\n";  } $file_outnum = 0; if ( $ind_opt or $loc_opt or $nam_opt or $gen_number or $dsc_opt or $asc_opt  ) { if ( open( CSS_IN, "$css_style.css" )) { $css = 1;
 $out_opt = 2; &open_new("$out_dir/$css_style.css"); while (<CSS_IN>) { $line = $_; $line =~ s/Pics\//$pics_path\//g; &cond_print("$line"); } &cond_out("$out_dir/$css_style.css"); close CSS_IN  ; } if ( $pics_path !~ /^\.\.(.*)$/ ) {	 		 opendir PICS, "$pics_dir"
 or die "\nCouldn't open pics directory $pics_dir : $!\n"; @pic_files = grep /.*?\..*/, readdir PICS; closedir PICS; if ( !$pics_path or $pics_path =~ /^\.$/ ) { $pics_path = ''; $out_path = $out_dir;	} elsif ( $pics_path =~ /^\.[\/\\](.*?)$/ ) { $pics_path = $1; $out_path = $out_dir.'/'.$pics_path; } else { $out_path = $out_dir.'/'.$pics_path; } if ( !-e $out_path ) { unless ( mkdir( $out_path, 0755 ) ) { die "Couldn't create $out_path directory\n"; } }
 foreach $pic_file (@pic_files) { $rc = &bincopy( "$pics_dir/$pic_file", "$out_dir/$pics_path/$pic_file" ); if ($rc==1) { &log_text("$out_dir/$pics_path/$pic_file copied $rc"); $file_outnum++; } } undef(@pic_files); } if (!$pics_path) { $pics_path = '..'; } else { $pics_path = '../'.$pics_path; } if ($gzip) { $php = 1; } if ( $php and (-e "uz.php" ))  { $rc = &bincopy( "uz.php", "$out_dir/uz.php" );
 if ($rc==1) { &log_text("$out_dir/uz.php copied"); $file_outnum++; } } else { $php = 0; }															 if ($cd) { $rc = bincopy( "autorun.inf", "$out_dir/autorun.inf" ); if ($rc==1) { &log_text("$out_dir/autorun.inf copied"); $file_outnum++; } $rc = bincopy( "gedail.ico", "$out_dir/gedail.ico" );
 if ($rc==1) { &log_text("$out_dir/gedail.ico copied"); $file_outnum++; } $rc = bincopy( "loader.exe", "$out_dir/loader.exe" ); if ($rc==1) { &log_text("$out_dir/loader.exe copied"); $file_outnum++; } $rc = bincopy( "loader.ini", "$out_dir/loader.ini" ); if ($rc==1) { &log_text("$out_dir/loader.ini copied"); $file_outnum++; } }
 if($aut_pic) { $rc = bincopy( "$aut_pic", "$out_dir/$aut_pic" ); if ($rc==1) { &log_text("$out_dir/$aut_pic copied"); $file_outnum++; } } if($banner_pic) { $rc = bincopy( "$banner_pic", "$out_dir/$banner_pic" );
 if ($rc==1) { &log_text("$out_dir/$banner_pic copied"); $file_outnum++; } } $rc = bincopy( "favicon.ico", "$out_dir/favicon.ico" ); if ($rc==1) { &log_text("$out_dir/favicon.ico copied"); $file_outnum++; } } $in0 = 0; $in1 = 0; $save_nl = $/; $ged_file_nl = $/;
 open( INPUT, "< $file_ged" ) or die "\nCouldn't open gedcom file $file_ged\n"; binmode INPUT or die "($!,$^E)"; defined( $r = sysread( INPUT, $head, 20 ) ) or die "\nCouldn't read gedcom file $file_ged\n"; close INPUT; if ($head =~ /^0 HEAD\x20*\r\d/is) { $ged_file_nl = "\r"; } $/ = $ged_file_nl; 
 unless ( open( GED_FILE, "$file_ged" ) ) { die "\nCouldn't open gedcom file $file_ged\n"; } &mess_print('m_w_ged_fil'); &log_text("Processing gedcom file $file_ged");		 $|            = 1; $num_indivs   = 0; $num_families = 0; $fam_cnt = 0; $num_chil = 0; $num_famc = 0; 
 %name_cnt = (); %place_count = (); %placx_count = (); @sosa_cnt = (); @gen_count = (); %loc_k=();%nam_k = (); %fam_k=(); %ind_k=(); %note=(); %note_indiv_link=(); %nam_type=(); %obj_list=(); %obj_title=(); %obj_ref=(); %fam_husb=(); %fam_wife=(); %fam_chil=(); %fam_marr=();  %fam_marr_year=(); %fam_marr_date=(); %fam_marr_plac=(); %fam_marr_city=();
 %indiv_link=(); %indiv_famc=(); %indiv_fams=(); %indiv_sosa=(); %indiv_file = (); %indiv_dabo = (); %indiv_branch = (); %indiv_name=(); %indiv_surname=(); %indiv_forname=(); %indiv_givname=(); %indiv_has_child=(); %indiv_famn=(); %indiv_note=(); %indiv_source=(); %indiv_events=(); %indiv_occu=(); %indiv_refn=(); %indiv_titl=(); 
 %indiv_place=(); %indiv_city=(); %indiv_orig=();  %indiv_birt_date=(); %indiv_birt_year=(); %indiv_birt_plac=(); %indiv_birt_city=(); %indiv_deat=();%indiv_deat_date=(); %indiv_deat_year=(); %indiv_deat_plac=(); %indiv_deat_city=(); %branch_id =(); %branch_ref =(); %descload =();%indirref=();
 %names_alpha =();%places_alpha=(); $irefn = 0; if ($print_sources and $ind_opt) { $sour_exist = 0; $sour_lines = 0; $sour_seq = '0001'; $repo_ini = 0; } while (<GED_FILE>) { if ( $charset ne "ANSI" ) { $_ = &process_accents($charset,$_); } if (/^\s*0\s*(.*)/) {								 $level = 0;
 $rol = $1; $in2 = 0; if ($in1 == 13) { &print_source; } if ($in1 == 14) { $repo{$repo_id} = $repo_item{'NAME'}.''.$repo_item{'ADDR'}.''.$repo_item{'PHON'}.''.$repo_item{'NOTE'}; %repo_item = (); } $in1 = 0; if    ($rol =~ /^HEAD/)        { $in0 = 1; } elsif ($rol =~ /^@(.*)@\sSUBM/) { $in0 = 2; } elsif ($rol =~ /^@(.*)@\sFAM/)  { $in0 = 3;
 if ($indiv_events{$fam_husb{$fam_id}}) { $indiv_events{$fam_husb{$fam_id}} = events_clear($indiv_events{$fam_husb{$fam_id}}); } if ($indiv_events{$fam_wife{$fam_id}}) { $indiv_events{$fam_wife{$fam_id}} = events_clear($indiv_events{$fam_wife{$fam_id}}); } $fam_id = $1; $fam_k{$fam_id} = $num_families; $famc_cnt = 1; $num_families++; &mess_print('m_progress'); }
 elsif ($rol =~ /^@(.*)@\sINDI/) { $in0 = 4; if ($indiv_events{$indiv_id} and !$indiv_fams{$indiv_id}) { $indiv_events{$indiv_id} = events_clear($indiv_events{$indiv_id}); } $indiv_id = $1; $ind_k{$indiv_id} = indinum($indiv_id); $num_indivs++; $num_fams = 0; &mess_print( 'm_progress' ); }
 elsif ($rol =~ /^@(.*)@\sNOTE\s*(.*)$/) { if ($print_notes) { $note_id = $1; $note{$note_id} .= $2; $in0 = 5; } else { $in0 = 0; } } elsif ($rol =~ /^@(.*)@\sOBJE/) { if ($check_images ) { $obje_id = $1; $in0 = 6; } else { $in0 = 0; } } elsif ($rol =~ /^@(.*)@\sSOUR/) { if ($print_sources) { $sour_id = $1;
 $in0 = 7; } else { $in0 = 0; } } elsif ($rol =~ /^@(.*)@\sREPO/) { if ($print_sources) { $repo_id = $1; $in0 = 8; } else { $in0 = 0; } } elsif ($rol =~ /^TRLR/) { $in0 = 0; 
 if ($indiv_events{$indiv_id} and !$indiv_fams{$indiv_id}) { $indiv_events{$indiv_id} = events_clear($indiv_events{$indiv_id}); } if ($indiv_events{$fam_husb{$fam_id}}) { $indiv_events{$fam_husb{$fam_id}} = events_clear($indiv_events{$fam_husb{$fam_id}}); } if ($indiv_events{$fam_wife{$fam_id}}) { $indiv_events{$fam_wife{$fam_id}} = events_clear($indiv_events{$fam_wife{$fam_id}}); } }
 else { &err_print( 'e_unkno_rec' ); } } elsif (/^\s*1\s(.*)/) {							 $level = 1; $rol = $1; if ($in1 == 13 and $in0 != 7) { &print_source; } if ( $in0 == 1 ) { if ( $rol =~ /^CHAR\s*(.*)$/ ) { $charset = uc($1); $in1 = 0; } if ( $rol =~ /^PLAC\s*(.*)$/ ) { $in1 = 12; } }
 elsif ( $in0 == 2 ) { if ( ( !$aut_name ) && ( $rol =~ /^NAME\s*(.*)$/ ) ) { $aut_name = $1; } if ( ( !$aut_addr1 ) && ( $rol =~ /^ADDR\s*(.*)$/ ) ) { $aut_addr1 = $1; $addr_cont = 1; } else { $addr_cont = 0; } if ( ( !$aut_ctry ) && ( $rol =~ /^CTRY\s*(.*)$/ ) ) { $aut_ctry = $1; }
 if ( ( !$aut_phone ) && ( $rol =~ /^PHON\s*(.*)$/ ) ) { $aut_phone = $1; } if ( ( !$aut_email ) && ( $rol =~ /^_?EMAIL\s*(.*)$/ ) ) { $aut_email = $1; $aut_email =~ s/@/&#64;/g; } $in1 = 99; } elsif ( $in0 == 3 ) { if ( $rol =~ /^HUSB\s@(.*)@/ ) { $fam_husb{$fam_id} = $1; $in1 = 0;} elsif ( $rol =~ /^WIFE\s@(.*)@/ ) { $fam_wife{$fam_id} = $1; $in1 = 0;} elsif ( $rol =~ /^CHIL\s@(.*)@/ ) { if ( $famc_cnt == 1 ) { $fam_chil{$fam_id} = $1; }
 else { $fam_chil{$fam_id} .= "" . $1; } $indiv_link{ $fam_wife{$fam_id} } = $line_link; $indiv_link{ $fam_husb{$fam_id} } = $line_link; $famc_cnt++; $num_chil++; $in1 = 0; } elsif ( $rol =~ /^MARR.*$/  and !$event_marr ) { $in1 = 9; } elsif ( $rol =~ /^NOTE\s@(.*)@/ ) { $curr_note = $1;
 if ($fam_husb{$fam_id}) { $note_indiv_link{$curr_note} = $fam_husb{$fam_id}; if ($note{$curr_note} !~ /^@/) { $note{$curr_note} = "<A NAME='N".$curr_note."'>@".$curr_note.'@</A> '.$note{$curr_note}; }
 $curr_note = '<A CLASS="main1" HREF="#indiv_file('.$fam_husb{$fam_id}.').html#N'.$curr_note.'">@'.$curr_note.'@</A>'; $tag_note = 'x'.$fam_marr_year{$fam_id}; $indiv_note{$fam_wife{$fam_id}} = note_add($indiv_note{$fam_wife{$fam_id}},$tag_note,$curr_note); } else { $note_indiv_link{$curr_note} = $fam_wife{$fam_id}; } $in1 = 11; } elsif ( $rol =~ /^NOTE\s(.*)/ ) { $curr_note = $1;
 $tag_note = 'x'.$fam_marr_year{$fam_id}; $indiv_note{$fam_husb{$fam_id}} = note_add($indiv_note{$fam_husb{$fam_id}},$tag_note,$curr_note); $indiv_note{$fam_wife{$fam_id}} = note_add($indiv_note{$fam_wife{$fam_id}},$tag_note,$curr_note); $in1 = 11; }
 elsif ( ($rol =~ /^OBJE\s*@(.*?)@/) and $check_images ) { if (defined $fam_wife{$fam_id}) { if ( $obj_list{$fam_wife{$fam_id}} ) { $obj_list{$fam_wife{$fam_id}} .= "".$1;	} else { $obj_list{$fam_wife{$fam_id}} = $1; } $obj_ref{$1} = $fam_wife{$fam_id};   } if (defined $fam_husb{$fam_id}) { if ( $obj_list{$fam_husb{$fam_id}} ) { $obj_list{$fam_husb{$fam_id}} .= "".$1;  } else { $obj_list{$fam_husb{$fam_id}} = $1; } $obj_ref{$1} = $fam_husb{$fam_id};	  }
 $in1 = 0;  } elsif ( ($rol =~ /^OBJE\s*$/) and $check_images ) { $in1 = 15; $obj_num++; } elsif ( $rol =~ /^SOUR\s@(.*)@/ ) { if ($print_sources) { $indiv_source{$fam_husb{$fam_id}} .= ''.$1; $indiv_source{$fam_wife{$fam_id}} .= ''.$1; } $in1 = 0; }
 elsif ( $rol =~ /^SOUR\s(.*)/ ) {  if ($print_sources) { $sour_type = 'TITL'; $sour_item{'TITL'} = $1; $sour_id = '#'.$sour_seq; $indiv_source{$fam_husb{$fam_id}} .= ''.$sour_id; $indiv_source{$fam_wife{$fam_id}} .= ''.$sour_id; $sour_seq++; $sour_lines++; $in1 = 13; } else { $in1 = 0; } }
 elsif ($print_events) { $event = ''; eval "if (\$rol =~ /^($events_list)\\s*(.*)\$/) { \$event = \$1; \$event_txt = \$2;}"; if ($event) { if ($event ne 'NCHI' or !$nchi_flag) { $indiv_events{$fam_husb{$fam_id}} = event_add($indiv_events{$fam_husb{$fam_id}},$event,$event_txt); $indiv_events{$fam_wife{$fam_id}} = event_add($indiv_events{$fam_wife{$fam_id}},$event,$event_txt); $in1 = 10; }
 else { $in1 = 0; } } } else { $in1 = 0; }		} elsif ( $in0 == 4 ) { if ( $rol =~ /^NAME\s+(.*)$/ ) { $name = $1; if ( ( !$indi_root ) && ( $name_root ) && ( $1 eq $name_root ) ) { $indi_root = $indiv_id; } $name =~ /(.*?)\s*\/(.*)\/\s*(.*)/;
 if ($name_asis) { $indiv_surname{$indiv_id} = $2; } else { $indiv_surname{$indiv_id} = majuscule($2); $indiv_surname{$indiv_id} =~ s/\([^\)]*\)//g; }	 if (($1) and ($3)) { $indiv_forname{$indiv_id} = $1." ".$3; }  else { $indiv_forname{$indiv_id} = $1.$3; } if ($indiv_forname{$indiv_id} =~ /^[\s_]*$/) { $indiv_forname{$indiv_id} = "?"; } if ($indiv_surname{$indiv_id} =~ /^[\s_]*$/) { $indiv_surname{$indiv_id} = "?"; } $_ = $name;
 $indiv_name{$indiv_id} = "<i>$indiv_forname{$indiv_id}</i> $indiv_surname{$indiv_id}"; if (!defined $nam_type{$indiv_surname{$indiv_id}}) { $nam_type{$indiv_surname{$indiv_id}} = 0; } $in1 = 7; $name_cnt{ $indiv_surname{$indiv_id} }++; } elsif ( $rol =~ /^SEX\s(.)/ ) { $indiv_sex{$indiv_id} = $1; $in1 = 0; } elsif ( $rol =~ /^BIRT.*$/  and !$event_birt ) { $in1 = 1; } elsif ( $rol =~ /^DEAT\sY$/i  and !$event_deat ) { $in1 = 2; $indiv_deat_date{$indiv_id} = ' ';} elsif ( $rol =~ /^DEAT.*$/  and !$event_deat ) { $in1 = 2; } elsif ( ($rol =~ /^OCCU\s(.*)$/) and !$event_occu ) {  $event_txt = $1;
 $indiv_occu{$indiv_id} = event_add($indiv_occu{$indiv_id},'OCCU',$event_txt); $in1 = 3; } elsif ( $rol =~ /^REFN\s(.*)$/ ) { $indiv_refn{$indiv_id} = $1; $irefn = 1; if ( ( !$indi_root ) && ( $refn_root ) && ( $1 eq $refn_root ) ) { $indi_root = $indiv_id; } $in1 = 0;  } elsif ( $rol =~ /^NOTE\s@(.*)@/ ) { if ($print_notes) { $note_indiv_link{$1} = $indiv_id; $in1 = 4; }
 else { $in1 = 0; } } elsif ( $rol =~ /^NOTE\s(.*)/ ) { if ($print_notes) { $indiv_note{$indiv_id} = note_add($indiv_note{$indiv_id},'',$1); $in1 = 4; } else { $in1 = 0; } } elsif ( $rol =~ /^SOUR\s@(.*)@/ ) { if ($print_sources) { $indiv_source{$indiv_id} .= ''.$1; }
 $in1 = 0;  } elsif ( $rol =~ /^SOUR\s(.*)/ ) { if ($print_sources) { $sour_type = 'TITL'; $sour_item{'TITL'} = $1; $sour_id = '#'.$sour_seq; $indiv_source{$indiv_id} .= ''.$sour_id; $sour_seq++; $sour_lines++; $in1 = 13; } else { $in1 = 0; } }
 elsif ( ($rol =~ /^TITL\s(.*)$/) and !$event_titl  ) { $event_txt = $1; $indiv_titl{$indiv_id} = event_add($indiv_titl{$indiv_id},'TITL',$event_txt); $in1 = 6; } elsif ( $rol =~ /^FAMC\s@(.*)@/ ) { if ($indiv_famc{$indiv_id}) { &mess_print('e_too_par'); } else { $indiv_famc{$indiv_id} = $1; } $num_famc++;
 $in1 = 0;  } elsif ( $rol =~ /^FAMS\s@(.*)@/ ) { if ( $num_fams > 0 ) { $indiv_fams{$indiv_id} .= "" . $1; } else { $indiv_fams{$indiv_id} = $1; } $num_fams++; $in1 = 0;  } elsif ( ($rol =~ /^OBJE\s*@(.*?)@/) and $check_images ) { if ( $obj_list{$indiv_id} ) { $obj_list{$indiv_id} .= "".$1; } else { $obj_list{$indiv_id} = $1; } $obj_ref{$1} = $indiv_id;		 $in1 = 0;  }
 elsif ( ($rol =~ /^OBJE\s*$/) and $check_images ) { $in1 = 8; $obj_num++; } elsif ($print_events) { $event = ''; eval "if (\$rol =~ /^($events_list)\\s*(.*)\$/) { \$event = \$1; \$event_txt = \$2;}"; if ($event) { $indiv_events{$indiv_id} = event_add($indiv_events{$indiv_id},$event,$event_txt); $in1 = 5;
 if ($event eq 'NCHI') { $nchi_flag = 1; }  if ($event eq 'DEAT' and $event_txt eq 'Y') { $indiv_deat_date{$indiv_id} = ' '; }  } else { $in1 = 0; } } else { $in1 = 0; } } elsif ( $in0 == 5 ) { if    ( $rol =~ /^CONC\s?(.*)$/ ) { $note{$note_id} .= "$1"; } elsif ( $rol =~ /^CONT\s?(.*)$/ ) { $note{$note_id} .= "<br>$1"; } $in1 = 0;  }
 elsif ( $in0 == 6 ) { if ( $rol =~ /^FORM\s+(.*)$/ ) { $obje_form = $1; if ( $obje_form =~ /(GIF|JPG|BMP|PNG|JPEG)/i ) { $obj_type{$obje_id} = 'img'; } elsif ( $obje_form =~ /(HTM|HTML|URL)/i ) { $obj_type{$obje_id} = 'web'; } else { $in0 = 0 } }
 if ( $rol =~ /^FILE\s+(.*)$/ ) { &image_in($obje_id,$1); } if ( ($rol =~ /^TITL\s+(.*)$/)  and !$obj_title{$obje_id}) { $obj_title{$obje_id} = $1; } $in1 = 0;  } elsif ( $in0 == 7 ) { if    ( $rol =~ /^(AUTH|TITL|PUBL|TEXT|REPO|REFN|ABBR|CALN|NOTE)\s?(.*)$/ ) {  $sour_type = $1; 
 $sour_item{$sour_type} = $2; $sour_lines++; $in1 = 13; } } elsif ( $in0 == 8 ) { if    ( $rol =~ /^(NAME|ADDR|PHON|NOTE)\s?(.*)$/ ) { $repo_type = $1;  $repo_item{$repo_type} = $2; $in1 = 14; } } } elsif (/^\s*2\s(.*)/) {							 $level = 2; $rol = $1; $in2 = 0;
 if ( $rol =~ /^SOUR\s@(.*)@/ and $print_sources ) { if ($in0 == 4) { $indiv_source{$indiv_id} .= ''.$1; } if ($in0 == 3) {  $indiv_source{$fam_husb{$fam_id}} .= ''.$1; $indiv_source{$fam_wife{$fam_id}} .= ''.$1; } } elsif ( $in1 == 99) { if ( ($addr_cont) && ( $rol =~ /^CONT\s*(.*)$/ ) ) { if    ( $addr_cont == 1 ) { $aut_addr2 = $1; $addr_cont = 2; } elsif ( $addr_cont == 2 ) {	$aut_addr3 = $1; $addr_cont = 3; } elsif ( $addr_cont == 3 ) { $aut_addr4 = $1; $addr_cont = 0; } } }
 elsif ( $in1 == 1 ) { if ( $rol =~ /^DATE\s(.*)$/ ) { $indiv_birt_date{$indiv_id} = $1; $indiv_birt_year{$indiv_id} = Ged_year($1); $indiv_birt{$indiv_id} = 1; } elsif ( $rol =~ /^PLAC\s(.*)$/ ) { ($indiv_birt_plac{$indiv_id},$indiv_birt_city{$indiv_id},$indiv_orig{$indiv_id}) = Ged_place($1); $indiv_place{$indiv_id} = $indiv_birt_plac{$indiv_id}; $indiv_city {$indiv_id} = $indiv_birt_city{$indiv_id}; }
 elsif ( $rol =~ /^NOTE\s(.*)/ and $print_notes_2 ) { $indiv_note{$indiv_id} = note_add($indiv_note{$indiv_id},'',$1); $in2 = 1;} } elsif ( $in1 == 2 ) { if ( $rol =~ /^DATE\s(.*)$/ ) { $indiv_deat_date{$indiv_id} = $1; $indiv_deat_year{$indiv_id} = Ged_year($1); $indiv_deat{$indiv_id}      = 1; } elsif ( $rol =~ /^PLAC\s(.*)$/ ) { ($indiv_deat_plac{$indiv_id},$indiv_deat_city{$indiv_id},$dummy) = Ged_place($1); }
 elsif ( $rol =~ /^NOTE\s(.*)/ and $print_notes_2 ) { $indiv_note{$indiv_id} = note_add($indiv_note{$indiv_id},'+',$1); $in2 = 1; } } elsif ( $in1 == 3 ) { if ( $rol =~ /^(CONT|CONC)\s(.*)$/ ) { $kkk = '_'.$1; $indiv_occu{$indiv_id} = event_add($indiv_occu{$indiv_id},$kkk,$2); } if ( $rol =~ /^DATE\s(.*)$/ ) { $indiv_occu{$indiv_id} = event_add($indiv_occu{$indiv_id},'DATE',$1); }
 elsif ( $rol =~ /^PLAC\s(.*)$/ ) { $event_place = $1; $indiv_occu{$indiv_id} = event_add($indiv_occu{$indiv_id},'PLAC',$event_place); } } elsif ( $in1 == 4 ) { if    ( $rol =~ /^(CONC|CONT)\s?(.*)$/ ) { $indiv_note{$indiv_id} = note_add($indiv_note{$indiv_id},$1,$2); } }
 elsif ( $in1 == 5 ) { if ( $rol =~ /^(CONT|CONC)\s(.*)$/ ) { $kkk = '_'.$1; $indiv_events{$indiv_id} = event_add($indiv_events{$indiv_id},$kkk,$2); } if ( $rol =~ /^DATE\s(.*)$/ ) { $event_date = $1; $event_year = Ged_year($1); if ( $event eq 'BIRT' ) { $indiv_birt_date{$indiv_id} = $event_date; $indiv_birt_year{$indiv_id} = $event_year; $indiv_birt{$indiv_id} = 1; }
 if ( $event eq 'DEAT' ) { $indiv_deat_date{$indiv_id} = $event_date; $indiv_deat_year{$indiv_id} = $event_year; $indiv_deat{$indiv_id} = 1; } $indiv_events{$indiv_id} = event_add($indiv_events{$indiv_id},'DATE',$event_date); if ( $est_date and ($event =~ /^(BAPM|CHR)\s*$/) and !$indiv_birt_date{$indiv_id}) { $indiv_birt_year{$indiv_id}	= $event_year; $indiv_birt_date{$indiv_id}	= '~'.$event_date; }
 elsif ( $est_date and ($event =~ /^(BURI|CREM|FNRL|INTE)\s*$/) and ($indiv_deat_date{$indiv_id} !~ /^/s*$/)) { $indiv_deat_year{$indiv_id}	= $event_year; $indiv_deat_date{$indiv_id}	= '~'.$event_date; } } elsif ( $rol =~ /^PLAC\s(.*)$/ ) { $event_place = $1; if ( $event eq 'BIRT' ) { ($indiv_birt_plac{$indiv_id},$indiv_birt_city{$indiv_id},$indiv_orig{$indiv_id}) = Ged_place($event_place); $indiv_place{$indiv_id} = $indiv_birt_plac{$indiv_id}; $indiv_city {$indiv_id} = $indiv_birt_city{$indiv_id}; }
 if ( $event eq 'DEAT' ) { ($indiv_deat_plac{$indiv_id},$indiv_deat_city{$indiv_id},$dummy) = Ged_place($event_place); } $indiv_events{$indiv_id} = event_add($indiv_events{$indiv_id},'PLAC',$event_place); if ( $est_place and ($event =~ /^(BAPM|CHR)\s*$/) and !$indiv_birt_plac{$indiv_id}) { if ( $event_place !~ /^(.*?)\?$/ ) { $event_place .= '?'; } ($indiv_birt_plac{$indiv_id},$indiv_birt_city{$indiv_id},$indiv_orig{$indiv_id}) = Ged_place($event_place); }
 elsif ( $est_place and ($event =~ /^(BURI|CREM|FNRL|INTE)\s*$/) and !$indiv_deat_plac{$indiv_id}) { if ( $event_place !~ /^(.*?)\?$/ ) { $event_place .= '?'; } ($indiv_deat_plac{$indiv_id},$indiv_deat_city{$indiv_id},$dummy) = Ged_place($event_place); } } elsif ( $rol =~ /^(NOTE|SOUR)\s(.*)$/  and $print_notes_2) { $indiv_events{$indiv_id} = event_add($indiv_events{$indiv_id},'NOTE',$2); $in2 = 1; }
 elsif ( $rol =~ /^(ASSO|TYPE)\s(.*)$/ ) { $indiv_events{$indiv_id} = event_add($indiv_events{$indiv_id},$1,$2); $in2 = 1; } } elsif ( $in1 == 6 ) { if ( $rol =~ /^(CONT|CONC)\s(.*)$/ ) { $kkk = '_'.$1; $indiv_titl{$indiv_id} = event_add($indiv_titl{$indiv_id},$kkk,$2); }
 if ( $rol =~ /^DATE\s(.*)$/ ) { $indiv_titl{$indiv_id} = event_add($indiv_titl{$indiv_id},'DATE',$1); } elsif ( $rol =~ /^PLAC\s(.*)$/ ) { $event_place = $1; $indiv_titl{$indiv_id} = event_add($indiv_titl{$indiv_id},'PLAC',$event_place); } } elsif ( $in1 == 7 ) { if ( $rol =~ /^GIVN\s+(.*)$/ ) { $indiv_givname{$indiv_id} = "$1"; if (!defined($indiv_forname{$indiv_id})) {  $indiv_forname{$indiv_id} = $1;  $indiv_name{$indiv_id} = "<i>$indiv_forname{$indiv_id}</i> $indiv_surname{$indiv_id}"; } }
 elsif ( $rol =~ /^SURN\s+(.*)$/ ) { if (!defined($indiv_surname{$indiv_id})) {  $indiv_surname{$indiv_id} = $1; $indiv_name{$indiv_id} = "<i>$indiv_forname{$indiv_id}</i> $indiv_surname{$indiv_id}"; } } } elsif ( $in1 == 8 ) { if ( $rol =~ /^FORM\s+(.*)$/ ) { $img_id = $indiv_id."#".$obj_num;
 $obje_form = $1; if ( $obje_form =~ /(GIF|JPG|BMP|PNG|JPEG)/i ) { $obj_type{$img_id} = 'img'; } elsif ( $obje_form =~ /(HTM|HTML|URL)/i ) { $obj_type{$img_id} = 'web'; } else { $in1 = 0 } } elsif ( $rol =~ /^FILE\s+(.*)$/ ) { $img_id = $indiv_id."#".$obj_num;
 &image_in($img_id,$1); if ( $obj_list{$indiv_id} ) { $obj_list{$indiv_id} .= "".$img_id; } else { $obj_list{$indiv_id} = $img_id; } $obj_ref{$indiv_id} = $indiv_id;	 } elsif ( $rol =~ /^TITL\s+(.*)$/ ) { $img_id = $indiv_id."#".$obj_num; $obj_title{$img_id} = $1; } }
 elsif ( $in1 == 9 ) { if ( $rol =~ /^DATE\s(.*)$/ ) { $fam_marr_date{$fam_id} = $1; $fam_marr_year{$fam_id} = Ged_year($1); $fam_marr{$fam_id}      = 1; } elsif ( $rol =~ /^PLAC\s(.*)$/ ) { ($fam_marr_plac{$fam_id},$fam_marr_city{$fam_id},$dummy) = Ged_place($1); $fam_marr{$fam_id} = 1; }
 elsif ($rol =~ /^NOTE\s(.*)/ and $print_notes_2 ) { $curr_note = $1; $tag_note = 'x'.$fam_marr_year{$fam_id}; $indiv_note{$fam_husb{$fam_id}} = note_add($indiv_note{$fam_husb{$fam_id}},$tag_note,$curr_note); $indiv_note{$fam_wife{$fam_id}} = note_add($indiv_note{$fam_wife{$fam_id}},$tag_note,$curr_note); $in2 = 1; } }
 elsif ( $in1 == 10 ) { if ( $rol =~ /^DATE\s(.*)$/ ) { $event_date = $1; $event_year = Ged_year($1); if ( $event eq 'MARR' ) { $fam_marr_date{$fam_id} = $event_date; $fam_marr_year{$fam_id} = $event_year; $fam_marr{$fam_id} = 1; }
 $indiv_events{$fam_husb{$fam_id}} = event_add($indiv_events{$fam_husb{$fam_id}},'DATE',$event_date); $indiv_events{$fam_wife{$fam_id}} = event_add($indiv_events{$fam_wife{$fam_id}},'DATE',$event_date); if ( $est_date and ($event =~ /^(ENGA|MARB|MARC|MARL|MARS|MBON)\s*$/) and !$fam_marr_date{$fam_id}) { $fam_marr_year{$fam_id}	= $event_year; $fam_marr_date{$fam_id}	= '~'.$event_year; } }
 elsif ( $rol =~ /^PLAC\s(.*)$/ ) { $event_place = $1; if ( $event eq 'MARR' ) { ($fam_marr_plac{$fam_id},$fam_marr_city{$fam_id},$dummy) = Ged_place($event_place); } $indiv_events{$fam_husb{$fam_id}} = event_add($indiv_events{$fam_husb{$fam_id}},'PLAC',$event_place); $indiv_events{$fam_wife{$fam_id}} = event_add($indiv_events{$fam_wife{$fam_id}},'PLAC',$event_place); if ( $est_place and ($event =~ /^(ENGA|MARB|MARL|MARS|MBON)\s*$/) and !$fam_marr_plac{$fam_id}) { if ( $event_place !~ /^(.*?)\?$/ ) { $event_place .= '?'; } ($fam_marr_plac{$fam_id},$fam_marr_city{$fam_id},$dummy) = Ged_place($event_place); } }
 elsif ( $rol =~ /^(NOTE|SOUR)\s(.*)$/  and $print_notes_2) { $indiv_events{$fam_husb{$fam_id}} = event_add($indiv_events{$fam_husb{$fam_id}},'NOTE',$2); $indiv_events{$fam_wife{$fam_id}} = event_add($indiv_events{$fam_wife{$fam_id}},'NOTE',$2); $in2 = 1; } elsif ( $rol =~ /^(ASSO|TYPE)\s(.*)$/ ) { $event_1 = $1;
 $event_2 = $2; $indiv_events{$fam_husb{$fam_id}} = event_add($indiv_events{$fam_husb{$fam_id}},$event_1,$event_2); $indiv_events{$fam_wife{$fam_id}} = event_add($indiv_events{$fam_wife{$fam_id}},$event_1,$event_2); $in2 = 1; } } elsif ( $in1 == 11 ) { if ( $rol =~ /^(CONC|CONT)\s?(.*)$/ ) { $pref_note = $1;
 $curr_note = $2; $indiv_note{$fam_husb{$fam_id}} = note_add($indiv_note{$fam_husb{$fam_id}},$pref_note,$curr_note);  $indiv_note{$fam_wife{$fam_id}} = note_add($indiv_note{$fam_wife{$fam_id}},$pref_note,$curr_note);  } } elsif ( $in1 == 12 ) { if ( $rol =~ /^FORM\s(.*)$/ and !$place_format) { $place_form = $1;
 $place_form =~ s/Subdivision/1/i; $place_form =~ s/Town/2/i; $place_form =~ s/Area code/3/i; $place_form =~ s/County/4/i; $place_form =~ s/Region/5/i; $place_form =~ s/Country/6/i; $place_form =~ s/\s//g;
 if ($place_form =~ /^[123456\,\;]*$/) { $place_format = $place_form; &set_place_mask; } } } elsif ( $in1 == 13 ) { if    ( $rol =~ /^CONC\s?(.*)$/ ) { $sour_item{$sour_type} .= $1; } if    ( $rol =~ /^CONT\s?(.*)$/ ) { $sour_item{$sour_type} .= '<br>'.$1; } if    ( $rol =~ /^(CALN|MEDI)\s?(.*)$/ ) { $sour_item{$sour_type} .= ' - '.$2; } if    ( $rol =~ /^TEXT\s?(.*)$/ ) { $sour_type = 'TEXT'; $sour_item{$sour_type} = $1; $in2 = 1; } $sour_lines++; }
 elsif ( $in1 == 14 ) { if    ( $rol =~ /^CONC\s?(.*)$/ ) { $repo_item{$repo_type} .= $1; } if    ( $rol =~ /^(CONT|ADR1|ADR2|STAE|POST|CTRY)\s?(.*)$/ ) { $repo_item{$repo_type} .= '<br>'.$2; } } elsif ( $in1 == 15 ) { if (defined $fam_wife{$fam_id}) {  $spouses_id[1] = $fam_wife{$fam_id};  $obj_ref{$fam_id} = $fam_wife{$fam_id};  }
 else { $spouses_id[1] = ''; } if (defined $fam_husb{$fam_id}) {  $spouses_id[0] = $fam_husb{$fam_id};  $obj_ref{$fam_id} = $fam_husb{$fam_id};  } else { $spouses_id[0] = ''; } foreach $indiv_id (@spouses_id) { if ($indiv_id) { if ( $rol =~ /^FORM\s+(.*)$/ ) { $img_id = $indiv_id."#".$obj_num;
 $obje_form = $1; if ( $obje_form =~ /(GIF|JPG|BMP|PNG|JPEG)/i ) { $obj_type{$img_id} = 'img'; } elsif ( $obje_form =~ /(HTM|HTML|URL)/i ) { $obj_type{$img_id} = 'web'; } else { $in1 = 0 } } elsif ( $rol =~ /^FILE\s+(.*)$/ ) { $img_id = $indiv_id."#".$obj_num;
 &image_in($img_id,$1); if ( $obj_list{$indiv_id} ) { $obj_list{$indiv_id} .= "".$img_id; } else { $obj_list{$indiv_id} = $img_id; } } elsif ( $rol =~ /^TITL\s+(.*)$/ ) { $img_id = $indiv_id."#".$obj_num; $obj_title{$img_id} = $1; } } } }	 }
 elsif (/^\s*3\s(.*)/ and $in2) {							 $level = 3; $rol = $1; if ( $in1 == 13 ) { if    ( $rol =~ /^CONT\s?(.*)$/ ) { $sour_item{$sour_type} .= "<br>".$1; } if    ( $rol =~ /^CONC\s?(.*)$/ ) { $sour_item{$sour_type} .= $1; } $sour_lines++; } elsif ( $in1 == 1 or $in1 == 2 ) { if ( $rol =~ /^(CONC|CONT)\s?(.*)$/  and $print_notes_2) { $indiv_note{$indiv_id} = note_add($indiv_note{$indiv_id},$1,$2); } }
 elsif ( $in1 == 9 ) { if ( $rol =~ /^(CONC|CONT)\s?(.*)$/ and $print_notes_2 ) { $pref_note = $1; $curr_note = $2; $indiv_note{$fam_husb{$fam_id}} = note_add($indiv_note{$fam_husb{$fam_id}},$pref_note,$curr_note);  $indiv_note{$fam_wife{$fam_id}} = note_add($indiv_note{$fam_wife{$fam_id}},$pref_note,$curr_note);  } }
 elsif ( $in1 == 5 ) { if ( $rol =~ /^(CONT|CONC)\s(.*)$/ ) { $indiv_events{$indiv_id} = event_add($indiv_events{$indiv_id},$1,$2); } } elsif ( $in1 == 10 ) { if ( $rol =~ /^(CONT|CONC)\s(.*)$/ ) { $event_1 = $1; $event_2 = $2;
 $indiv_events{$fam_husb{$fam_id}} = event_add($indiv_events{$fam_husb{$fam_id}},$event_1,$event_2); $indiv_events{$fam_wife{$fam_id}} = event_add($indiv_events{$fam_wife{$fam_id}},$event_1,$event_2); } } } } close GED_FILE ; $/ = $save_nl; if ($print_sources and $ind_opt and $sour_exist) { $sour_count = $sour_size; &print_source;
 &print_skip($sour_dir,$next_file,$first_file);  if (%repo) { &print_repo; } } if ($print_notes) { foreach $note_id ( keys %note ) { $indiv_note{$note_indiv_link{$note_id}} = note_add($indiv_note{$note_indiv_link{$note_id}},'',$note{$note_id}); } undef(@note); } if ($num_famc != $num_chil) { $num_errs = $num_famc - $num_chil;
 &mess_print('e_ged_struc'); } if ($gen_opt) { &mess_print( "m_sosas" ); if ( $print_sosa or $gen_number or $fli_sosa_only or $mli_sosa_only ) { %indiv_row = ''; %sosa_gen = ''; %sosa_fam = ''; @sosa_level = ''; @ancestors = ''; if ( !$indi_root ) { &err_print('e_no_root');
 $max_row = 0; $nb_loop = 0; SOSA:	foreach $indiv_id ( keys %ind_k ) { if ( !$indiv_famc{$indiv_id} ) { if ($indiv_fams{$indiv_id}) { @fams = split ( //, $indiv_fams{$indiv_id} ); $fams_len = @fams; for ( $i = 0 ; $i < $fams_len ; $i++ ) { if (($indiv_famc{$fam_wife{$fams[$i]}}) or ($indiv_famc{$fam_husb{$fams[$i]}})) { next SOSA; } } }
 $indiv_row{$indiv_id} = 1; $nb_anc = 1; $anc = $indiv_id; while ($nb_anc) { $i_scan++; if ($i_scan >= 100000) { print "."; $i_scan=0; } foreach $family ( split(//,$indiv_fams{$anc}) ) { $ischild = 0; foreach $child ( split(//,$fam_chil{$family}) ) { $isanc = 0;
 if (!$ischild) { $isanc = 1; $ischild = 1;} else {						  foreach $fams1 ( split(//,$indiv_fams{$child}) ) { if ($fam_chil{$fams1}) { $isanc = 1; }								 } } if ($isanc) {  if ($child eq $anc) { &err_print('e_is_loop');	 $indi_root = ''; last; } push ( @ancestors, $child );
 $indiv_row{ $child }++; $indiv_row{ $child } +=	$indiv_row{$anc}; if ( $indiv_row{ $child } > $max_row ) { $indi_root = $child; $max_row   = $indiv_row{ $child }; $nb_loop++; if ($nb_loop > 2*$num_indivs) { &err_print('e_too_many');	 $gen_stop = 1; last SOSA; } } } }
 $indiv_row{$anc} = 0; } $nb_anc = @ancestors; $anc = pop (@ancestors); } } } } undef(@ancestors); print "\n"; if ($indi_root) { &err_print('e_is_root'); $root_name = html($indiv_name{$indi_root}); $sosa = 1; $sosa_cnt[1] = 1; $gen_count[1] = 1; $g = 1; $indiv_id = $indi_root;
 $indiv_sosa{$indi_root} = 1; $sosa_fam{1} = "root";	 $sosa_gen{1} = 1; $fam_husb{"root"} = $indi_root; push ( @sosa_level, 0 ); $nb_loop = 0; while ($g) { LOOP:			while ( $indiv_famc{$indiv_id} ) { if ($gen_stop) { $sosa_flag{$indiv_id} = 1; } push ( @sosa_level, $sosa );
 $sosa = sosa_x2($sosa); $sosa_cnt[$g+1]++;  $gen_count[$g+1]++; $sosa_fam{$sosa} = $indiv_famc{$indiv_id}; $sosa_gen{$sosa} = $g+1; if ( $fam_husb{$indiv_famc{$indiv_id}} ) { if ( $fam_wife{$indiv_famc{$indiv_id}} ) { $sosaw = sosa_inc($sosa);  $sosa_gen{$sosaw} = $g+1; $gen_count[$g+1]++;
 if ($print_sosa) { if ( $indiv_sosa{$fam_wife{$indiv_famc{$indiv_id}}} ) { if (!$gen_stop and $print_sosa == 1) {	$indiv_sosa{$fam_wife{$indiv_famc{$indiv_id}}} .= ", ".$sosaw; } elsif (!$gen_stop and $print_sosa == 2) {	$indiv_sosa{$fam_wife{$indiv_famc{$indiv_id}}} = "..., ".$sosaw; } elsif ($gen_stop and $indiv_sosa{$fam_wife{$indiv_famc{$indiv_id}}} !~ /,/) { $indiv_sosa{$fam_wife{$indiv_famc{$indiv_id}}} .= ", ...";  }	 }
 else { $indiv_sosa{$fam_wife{$indiv_famc{$indiv_id}}} = $sosaw; } } else { $indiv_sosa{$fam_wife{$indiv_famc{$indiv_id}}} = 1; } } $indiv_id = $fam_husb{$indiv_famc{$indiv_id}}; } elsif ( $fam_wife{$indiv_famc{$indiv_id}} ) { $indiv_id = $fam_wife{$indiv_famc{$indiv_id}}; $sosa = sosa_inc($sosa); }
 else { &err_print('e_no_parents');	 last LOOP; } if ($print_sosa) { if ( $indiv_sosa{$indiv_id} ) { if ($print_sosa == 1) {	$indiv_sosa{$indiv_id} .= ", ".$sosa; } elsif ($print_sosa == 2) {	$indiv_sosa{$indiv_id} = "..., ".$sosa; } elsif ($indiv_sosa{$indiv_id} !~ /,/) { $indiv_sosa{$indiv_id} .= ", ...";  }	 }
 else {  $indiv_sosa{$indiv_id} = $sosa; $nam_type{$indiv_surname{$indiv_id}} = 1; } } else { $indiv_sosa{$indiv_id} = 1; } $g++; if (($gen_stop) and ($sosa_flag{$indiv_id})) { last LOOP; } $nb_loop++; if ($nb_loop > 2*$num_indivs) { &err_print('e_too_many');	
 $gen_stop = 1; last LOOP; 				} } $indiv_id = 0; until ( $indiv_id or ( $g == 0 ) ) { while ( ($sosa =~ /[13579]$/) && $g ) { $sosa = pop (@sosa_level); $g--; } if ( $fam_wife{ $sosa_fam{$sosa} } ) { $indiv_id = $fam_wife{ $sosa_fam{$sosa} }; } $sosa = sosa_inc($sosa); } } }
 else { &err_print('e_de_cujus'); $print_sosa=0; $gen_number=0; $fli_sosa_only=0; $mli_sosa_only=0; } } undef(@sosa_flag); } if ($dsc_opt and !@desc_names) { if ($indi_root) { $desc_names[0] = $indiv_surname{$indi_root}; } foreach $nam_id ( sort { $name_cnt{$b} <=> $name_cnt{$a} } keys %name_cnt ) { if ($name_cnt{$nam_id} < $desc_name_min) { last; }        if ($nam_id ne $indiv_surname{$indi_root}) { push ( @desc_names, $nam_id); } }              }
 if ($dsc_opt and ($desc_names[0] eq '*')) { for ( $i = 1 ; $i < @desc_names ; $i++ ) { $nam_omit{$desc_names[$i]} = 1; } @desc_names = (); foreach $nam_id ( sort { $a cmp $b } keys %nam_type ) { if (($name_cnt{$nam_id} >= $desc_name_min) and $nam_id !~ /^X*\?*$/i and !$nam_omit{$nam_id} and (!$desc_sosas_only or ($desc_sosas_only and $nam_type{$nam_id}))) { push ( @desc_names, $nam_id); } } }
 if (!$name_asis) { $iii = 0; foreach $desc_name ( @desc_names ) { $desc_name = majuscule($desc_name); $desc_names[$iii] = majuscule($desc_names[$iii]); $iii++; } } if ($dsc_opt and @desc_names) { &log_print( "l_dsc_proc" );
 @dabo_id = (1,2,3,4,5,6,7,8,9,A,B,C,D,E,F,G,H,I,J,K,L,M,N,O,P,Q,R,S,T,U,V,W,X,Y,Z); $bnum = 0; $branch_max = 0; $branch = 0; $names_sel = 0; %branch_count=(); %branch_nnum=(); %branch_head=(); %branch_orig=(); %desc_nnum=(); @name_is=(); @desc_select = (); @name_select = (); foreach $desc_name ( @desc_names ) { $bnum++;  
 $desc_nnum{$desc_name} = $bnum; $branch_number = 0; foreach $indiv_id ( sort { $a cmp $b }  keys %ind_k ) { if ( (!$indiv_famc{$indiv_id}) and ($indiv_surname{$indiv_id} =~ /^($desc_name)\s*$/i ) ) { if (($sosa_only or $desc_sosas_only) and !$indiv_sosa{$indiv_id}) { next; } if ($desc_sosas_only and $indiv_sex{$indiv_id} eq 'F') { next; } if (!$name_is[$bnum]) { $name_is[$bnum]	=1; $names_sel++; } $branch++;
 $indiv_branch{$indiv_id} = $branch; $branch_head{$branch} = $indiv_id; $branch_nnum{$branch} = $bnum-1; $indiv_dabo{$indiv_id} = 0; $nb_desc = 1; $branch_count{$branch} = 0; $desc = $indiv_id; while ($nb_desc) { if (!$branch_orig{$branch}) { if ($indiv_deat_plac{$desc}) { $branch_orig{$branch} = $indiv_deat_city{$desc}; } if ($indiv_birt_plac{$desc}) { $branch_orig{$branch} = $indiv_birt_city{$desc}; } }
 if (!$desc_only or ($indiv_surname{$desc} =~ /^($desc_name)\s*$/i )) { $child_num = 0; $fam_num = 0; $old_marr_year = ''; foreach $family ( sort { marr_year($a)<=>marr_year($b) } split(//,$indiv_fams{$desc}) ) { if (!$branch_orig{$branch} and $fam_marr_plac{$family}) { $branch_orig{$branch} = $fam_marr_city{$family};  }
 $desc_ok = 1; if ($indiv_sex{$desc} eq 'F') {  foreach $desc_nam1 ( @desc_names ) { if ($indiv_surname{$fam_husb{$family}} =~ /^($desc_nam1)\s*$/i ) { $desc_ok = 0; $indiv_has_child{$desc} = -1; } } } if ($desc_ok) {  $old_birt_year = '';
 if ($fam_chil{$family})	{ $indiv_has_child{$desc}++; } foreach $child ( sort { birt_year($a) <=> birt_year($b) } split(//,$fam_chil{$family}) ) { if (!$indiv_dabo{$desc}) { $indiv_dabo{$child} = $dabo_id[$child_num];  }	 else { $indiv_dabo{$child} = $indiv_dabo{$desc}.$dabo_id[$child_num];	 }	
 $indiv_branch{$child} = $branch; if ($fam_num) { $indiv_famn{$child} = $fam_num+1;  }  if ( ($branch_count{$branch} == $desc_number_min) or (!$desc_sosas_only and $indiv_sosa{$branch_head{$branch}}) ) { $branch_number++; } $branch_count{$branch}++; push ( @descendants, $child );
 $child_num++; } } $fam_num++; } } $nb_desc = @descendants; $desc = shift (@descendants); } } } if ($branch_number > 26) { $id = 'AA' } else { $id = 'A'; } foreach $branch ( sort { $branch_head{$a} cmp $branch_head{$b} } keys %branch_count ) { if ( ($branch_count{$branch} < $desc_number_min) and ($desc_sosas_only or (!$desc_sosas_only and !$indiv_sosa{$branch_head{$branch}})) ) { next; } if ( $branch_nnum{$branch} ne $bnum-1) { next; } if ($desc_sosas_only) { $indiv_sosa{$branch_head{$branch}} =~ /^([^,]*)(,.*)?$/; $branch_id{$branch} = $1;  }
 elsif ($names_sel > 1) { $branch_id{$branch} = $bnum-1; $branch_id{$branch} .= $id; } else { $branch_id{$branch} = $id;} if (!$name_select[$bnum]) { $name_select[$bnum]	=1; push @desc_select,$desc_names[$bnum-1]; } $id++; $branch_max ++; $desc_count[$branch_nnum{$branch}]++; } }
 $desc_name ='                                                                        '; &mess_print( "m_dabo" ); undef(@descendants); &log_print( "l_dsc_found" ); print "\n"; } else { $dsc_opt = 0; } if (!@desc_select) { $dsc_opt = 0; } $file_id    = $indf_name; $file_first = 0;
 INDI0 : foreach $indiv_id ( sort { $ind_k{$a} <=> $ind_k{$b} } keys %ind_k ) { if(!$indiv_id) { next INDI0; } if (!$file_first) { $file_first = indinum($indiv_id); } if (($sosa_only) and (!$indiv_sosa{$indiv_id})) { next INDI0; } if ($desc_only and (!$branch_id{$indiv_branch{$indiv_id}})) { next INDI0; } $indiv_file{$indiv_id} = $file_id; if (!defined $indiv_surname{$indiv_id}) { $indiv_surname{$indiv_id} = '?'; } if (!defined $nam_k{$indiv_surname{$indiv_id}}) { $nam_k{$indiv_surname{$indiv_id}} = $indiv_surname{$indiv_id}; } $file_curr = indinum($indiv_id);
 if ( $file_curr > $file_first + $indf_size - 1 ) { $file_id++; $file_first = $file_curr; } foreach $priv_ref (@priv_refs) { if ( $indiv_refn{$indiv_id}  =~ /^($priv_ref)\s*$/   ) { $indiv_birt_date{$indiv_id} = ""; $indiv_birt_plac{$indiv_id} = ""; $indiv_deat_date{$indiv_id} = ""; $indiv_deat_plac{$indiv_id} = ""; $indiv_note{$indiv_id}      = ""; $indiv_titl{$indiv_id}      = ""; $indiv_occu{$indiv_id}      = ""; $indiv_events{$indiv_id}    = ""; $indiv_refn{$indiv_id}      = "prv"; if ($priv_name) {  $indiv_name{$indiv_id} = $priv_name; $indiv_surname{$indiv_id} = $priv_name; $indiv_forname{$indiv_id} = ''; } } }
 if ( $indiv_birt_year{$indiv_id} > $curr_year ) { &err_print('e_born_aft'); } } if ($check_images) { if ( !$photo_dir ) { $photo_dir = "$out_dir/$photos_path"; } if ( opendir PHOTOS, "$photo_dir" ) { @photo_files = grep /^(I\d+|\d+|\d+I)\.(GIF|JPG|BMP|PNG|JPEG)$/i, readdir PHOTOS; closedir PHOTOS;
 foreach $photo_file (@photo_files) { $photo_file =~ /.*?(I{0,1}\d+I{0,1})\.(GIF|JPG|BMP|PNG|JPEG)/i; $indiv_id = $1; if ( $indiv_name{$indiv_id} and !$img_path{$indiv_id} ) { if ( !-e "$out_dir/$photos_path/$indiv_id.jpg" ) { &rc = bincopy("$photo_dir/$indiv_id.jpg","$out_dir/$photos_path/$indiv_id.jpg");
 if ($rc==1) { &log_text("$photo_dir/$indiv_id.jpg copied $rc");	$file_outnum++; } } ( $img_width{$indiv_id}, $img_height{$indiv_id}, $img_path{$indiv_id} ) = &img_prop("$photo_file"); if ( -e "$photo_dir/tn_$indiv_id.jpg" ) { $img_thumb{$indiv_id} = "../$photos_path/tn_$indiv_id.jpg"; } } } } else {  &err_print('e_pictu_dir'); }
 if ($pics_exist) { $jvs_file =  $out_dir.'/'.$photos_path.'/pics_links.jvs'; $out_opt = 2; &open_new("$jvs_file"); &cond_print("function pics_links(id) {\n"); &cond_print(" var v = '';\n"); &cond_print(" var vf = '';\n"); &cond_print(" switch(id) {\n"); foreach $obj_id ( sort { $a cmp $b } keys %obj_ref ) { &cond_print(" case '$obj_id':v='$obj_ref{$obj_id}';vf='$indiv_file{$obj_ref{$obj_id}}';break;\n"); }
 &cond_print(" default : return;\n");   &cond_print(" }\n");   if ($php) { &cond_print(" window.location.href = '../$tablang[0]/uz.php?p=' + vf + '.html#' + v; return;\n"); } else { &cond_print(" window.location.href = '../$tablang[0]/' + vf + '.html#' + v; return;\n"); } &cond_print("}\n");
 &cond_out("$jvs_file"); } }																			   if (@tablang <= 1) { $flags = '&nbsp;'	} else {  $flags = ''; $first_file = $indx_name;	 if ($php) {	$first_file = "uz.php?p=".$first_file; } foreach $lang_id ( @tablang ) { $flags .= "<a href=\"../$lang_id/$first_file.html\" title='$lang_id'><img src=\"$pics_path/$lang_id.gif\" border=0></a>&nbsp;";  } }
 $plinks = &place_link_init; $nbrow_span =1; $img_sosa ="<img src=\"$pics_path/i_a.gif\" class=va border=0>"; $img_dabo ="<img src=\"$pics_path/i_d.gif\" class=va border=0>"; $iindir = 0; print "\n"; foreach $lang_id ( @tablang ) { print "$lang_id";
 %place_count = ''; %placx_count = '';  $ind_ok = 0; if ( open( OPTIONS, "gedigen_$lang_id.opt" ) ) { while (<OPTIONS>) { eval $_ } close OPTIONS ;						    } $lang_dir = $out_dir.'/'.$lang_id; if ( !-e $lang_dir ) { unless ( mkdir( $lang_dir, 0755 ) ) { die "Couldn't create $lang_dir directory\n"; } }
 if ($plinks) { $rc = &bincopy( "cassini.jvs", "$lang_dir/cassini.jvs" ); if ($rc==1) { &log_text("$lang_dir/cassini.jvs created $rc");	$file_outnum++; } } @tab_mois=split(/,/,$str_mois);
 @GED_mois=('JAN','FEB','MAR','APR','MAY','JUN','JUL','AUG','SEP','OCT','NOV','DEC'); if ($php)  { $rc = &bincopy( "uz.php", "$lang_dir/uz.php" ); if ($rc==1) { &log_text("$lang_dir/uz.php copied"); $file_outnum++; } } if ( $tpl_dir =~ /^([A-Za-z]\d{3})(-[A-Za-z]{2})?$/ ) { $tpl_dir = $1.'-'.$lang_id;		 } if ($ind_opt) { $out_opt = 1;
 &get_template("ind-tpl",'#ind_givname|#ind_forname|#ind_surname',''); &log_print('l_cre_indiv'); } $file_id   = ""; $num_names = 0; $ind_cnt = 0; $i_scan=0;  INDI1 : foreach $indiv_id ( sort { $ind_k{$a} <=> $ind_k{$b} } keys %ind_k ) { if(!$indiv_id) { next; } $i_scan++; if ($i_scan >= 500) { print "."; $i_scan=0; } if (($sosa_only) and (!$indiv_sosa{$indiv_id})) { next INDI1; } if (($desc_only) and (!$branch_id{$indiv_branch{$indiv_id}})) { next INDI1; } $ind_cnt++;
 if ( $indiv_file{$indiv_id} ne $file_id ) { if ($num_names) { &cond_out("$lang_dir/$file_id.html");	} $file_id = $indiv_file{$indiv_id}; &open_new("$lang_dir/$file_id.html"); &print_header; } $num_names++; if ($ind_opt) { $father    = ""; $mother    = "";
 $father_id = $fam_husb{ $indiv_famc{$indiv_id} }; $mother_id = $fam_wife{ $indiv_famc{$indiv_id} }; $pgfather = ""; $pgmother = ""; $mgfather = ""; $mgmother = ""; if ($father_id) { if ($desc_only and (!$branch_id{$indiv_branch{$father_id}})) { $father = $indiv_name{$father_id}; $indiv_rel{$father_id} = $indiv_id; }
 else { $father = "<a href=\"$indiv_file{$father_id}.html#$father_id\">$indiv_name{$father_id}</a>"; } $pgfather_id = $fam_husb{ $indiv_famc{$father_id} }; if ($pgfather_id) { if ($desc_only and (!$branch_id{$indiv_branch{$pgfather_id}})) { $pgfather = $indiv_name{$pgfather_id}; $indiv_rel{$pgfather_id} = $indiv_id; }
 else { $pgfather = "<a href=\"$indiv_file{$pgfather_id}.html#$pgfather_id\">$indiv_name{$pgfather_id}</a>"; } } $pgmother_id = $fam_wife{ $indiv_famc{$father_id} }; if ($pgmother_id) { if ($desc_only and (!$branch_id{$indiv_branch{$pgmother_id}})) { $pgmother = $indiv_name{$pgmother_id}; $indiv_rel{$pgmother_id} = $indiv_id; }
 else { $pgmother = "<a href=\"$indiv_file{$pgmother_id}.html#$pgmother_id\">$indiv_name{$pgmother_id}</a>"; } } } if ($mother_id) { if ($desc_only and (!$branch_id{$indiv_branch{$mother_id}})) { $mother = $indiv_name{$mother_id}; $indiv_rel{$mother_id} = $indiv_id; }
 else { $mother = "<a href=\"$indiv_file{$mother_id}.html#$mother_id\">$indiv_name{$mother_id}</a>"; } $mgfather_id = $fam_husb{ $indiv_famc{$mother_id} }; if ($mgfather_id) { if ($desc_only and (!$branch_id{$indiv_branch{$mgfather_id}})) { $mgfather = $indiv_name{$mgfather_id}; $indiv_rel{$mgfather_id} = $indiv_id; }
 else { $mgfather = "<a href=\"$indiv_file{$mgfather_id}.html#$mgfather_id\">$indiv_name{$mgfather_id}</a>"; } } $mgmother_id = $fam_wife{ $indiv_famc{$mother_id} }; if ($mgmother_id) { if ($desc_only and (!$branch_id{$indiv_branch{$mgmother_id}})) { $mgmother = $indiv_name{$mgmother_id}; $indiv_rel{$mgmother_id} = $indiv_id; }
 else { $mgmother = "<a href=\"$indiv_file{$mgmother_id}.html#$mgmother_id\">$indiv_name{$mgmother_id}</a>"; } } } $parents_chart = ""; if ( ( $father ne "" ) or ( $mother ne "" ) ) { $parents_chart .= 	"<td class=Ig width=150>$pgfather</td><td width=150>$pgmother</td>"
 ."<td class=Ig width=150>$mgfather</td><td width=150>$mgmother</td></tr><tr>"; if ( ( $pgfather ne "" ) or ( $pgmother ne "" ) ) { $parents_chart .= "<td class=t300 colspan=2 width=300 height=30></td>"; } else { $parents_chart .= "<td colspan=2></td>"; } if ( ( $mgfather ne "" ) or ( $mgmother ne "" ) ) { $parents_chart .= "<td class=t300 colspan=2 width=300 height=30></td>"; }
 else { $parents_chart .= "<td colspan=2></td>"; } $parents_chart .= "</tr><tr align=center><td colspan=2 class=Ip>$father</td>" ."<td colspan=2 class=Ip>$mother</td></tr><tr>" ."<td class=t600 colspan=4 width=600 height=30></td>"; }
 if ($parents_chart) { $parents_chart = "<tr align=center>".$parents_chart."</tr>"; } $ind_photo = ""; if (($indiv_refn{$indiv_id} eq "prv") or (($indiv_birt_date{$indiv_id} eq "$priv_string") and ($hide_XX_pics))) { $img_path{$indiv_id} = ""; $obj_list{$indiv_id} = ""; }
 elsif ( ($check_images) && ( $img_path{$indiv_id} ne "" ) && ( $obj_list{$indiv_id} eq "" ) ) { $img_title{$indiv_id} = "$indiv_forname{$indiv_id} $indiv_surname{$indiv_id}";  $img_title{$indiv_id} =~ s/\'/ /g; if (!$img_thumb{$indiv_id}) { $img_thumb{$indiv_id} = "$pics_path/photo.gif"; } $ind_photo = "<a HREF=\"javascript:photo('$img_path{$indiv_id}'" .", '$img_title{$indiv_id}', '$img_width{$indiv_id}', '$img_height{$indiv_id}')\">" ."<img src=\"$img_thumb{$indiv_id}\" border=0></a>-&nbsp;"; }
 elsif ( ($check_images) && ( $obj_list{$indiv_id} ne "" ) ) { @objects = split ( //, $obj_list{$indiv_id} ); $obj_len = @objects; for ( $i = 0 ; $i < $obj_len ; $i++ ) { if ($obj_type{$objects[$i]} eq 'web' and $obj_path{$objects[$i]}) { $ind_photo .= "<a HREF=\"$obj_path{$objects[$i]}\" title=\"$obj_title{$objects[$i]}\" target=_blank>" ."<img src=\"$pics_path/web.gif\" STYLE=\"vertical-align:middle\" border=0></a>&nbsp;"; }
 elsif ($obj_path{$objects[$i]}){ if (!$obj_title{$objects[$i]}) {  $obj_path{$objects[$i]} =~ /^(.*)(\\|\/)(gi_)?([^\\\/]+?)\.([^\.]+?)$/; $obj_title{$objects[$i]} = $4;  } $obj_title{$objects[$i]} =~ s/\'/ /g; if (!$obj_thumb{$objects[$i]}) { $obj_thumb{$objects[$i]} = "$pics_path/photo.gif"; } $ind_photo .= "<a HREF=\"javascript:photo('$obj_path{$objects[$i]}'" .", '$obj_title{$objects[$i]}', '$obj_width{$objects[$i]}', '$obj_height{$objects[$i]}')\" title=\"$obj_title{$objects[$i]}\">" ."<img src=\"$obj_thumb{$objects[$i]}\" border=0></a>&nbsp;"; } } }
 if ( $hide_XX and !$indiv_deat{$indiv_id}) { if ( $indiv_birt_year{$indiv_id} + 100 > $curr_year ) { $indiv_birt_date{$indiv_id} = "$priv_string"; $indiv_birt_plac{$indiv_id} = ""; $indiv_birt_city{$indiv_id} = ""; $indiv_note{$indiv_id}      = ""; $indiv_titl{$indiv_id}      = ""; $indiv_occu{$indiv_id}      = ""; $indiv_events{$indiv_id}    = ""; } }
 for ( $i = $block_beg ; $i < $block_beg + $block_cnt ; $i++ ) { if (!$vartemp[$i]) { &cond_print("$template[$i]"); next; } $line = $template[$i]; if ($indiv_has_child{$indiv_id} > 0) { $branch_link = "javascript:descLoad('".$branch_id{$indiv_branch{$indiv_id}}."','".$indiv_branch{$indiv_id}.$indiv_dabo{$indiv_id}."')";  }
 else { if (length($indiv_dabo{$indiv_id}) == 1) { $branch_link = '0'; } else { $branch_link = substr($indiv_dabo{$indiv_id},0,length($indiv_dabo{$indiv_id})-1); } $branch_link = "javascript:descLoad('".$branch_id{$indiv_branch{$indiv_id}}."','".$indiv_branch{$indiv_id}.$branch_link."')";  }
 if ( $print_desc && $branch_id{$indiv_branch{$indiv_id}} ) { $ind_dabo = "<a class=main1 href=\"".$branch_link."\">".$branch_id{$indiv_branch{$indiv_id}}." ".$img_dabo."<\/a> ".$indiv_dabo{$indiv_id}; $line =~ s/<!#ind_dabo\s?(.*?)\>/$1/g; $line =~ s/#ind_dabo/$ind_dabo/g; }
 elsif ( $branch_id{$indiv_branch{$indiv_id}} ) { $ind_dabo = "&nbsp;<a class=main1 href=\"".$branch_link."\">$img_dabo<\/a>"; $line =~ s/<!#ind_dabo.*?>//g; $line =~ s/#ind_dabo/$ind_dabo/g; } else {  $line =~ s/<!#ind_dabo.*?>//g;  $line =~ s/^.*?#ind_dabo\s*(<br>)?.*?$//g;  }
 if ( $irefn && $print_refn && $indiv_refn{$indiv_id} ) { $line =~ s/<!#ind_refn\s?(.*?)\>/$1/g; $line =~ s/#ind_refn/$indiv_refn{$indiv_id}/g; } elsif ( !$irefn && $print_refn ) { $line =~ s/<!#ind_refn\s?(.*?)\>/$1/g; $line =~ s/#ind_refn/$indiv_id/g; } else {  $line =~ s/<!#ind_refn.*?>//g; 
 $line =~ s/^.*?#ind_refn\s*(<br>)?.*?$//g;  } if ( $print_sosa and $indi_root and $indiv_sosa{$indiv_id} ) { @ind_sosa = split(',',$indiv_sosa{$indiv_id}); if ( $sosa_gen{$indiv_sosa{$indiv_id}} > $gen_number ) { $ind_sosa[0] = $indiv_sosa{$indiv_id}; } else { $branch_link = "javascript:sosaLoad('".$ind_sosa[0]."')";
 $ind_sosa[0] = "<a class=main1 href=\"".$branch_link."\">".$img_sosa." ".$indiv_sosa{$indiv_id}."<\/a>"; } $line =~ s/<!#ind_sosa\s?(.*?)\>/$1/g; $line =~ s/#ind_sosa/$ind_sosa[0]/g; } elsif ( $indi_root and $indiv_sosa{$indiv_id} and ($sosa_gen{$indiv_sosa{$indiv_id}} <= $gen_number) ) { @ind_sosa = split(',',$indiv_sosa{$indiv_id}); $branch_link = "javascript:sosaLoad('".$ind_sosa[0]."')";
 $ind_sosa[0] = "&nbsp;<a class=main1 href=\"".$branch_link."\">$img_sosa<\/a>"; $line =~ s/<!#ind_sosa.*?>//g;  $line =~ s/#ind_sosa/$ind_sosa[0]/g; } else {  $line =~ s/<!#ind_sosa.*?>//g;  $line =~ s/^.*?#ind_sosa\s*(<br>)?.*?$//g;  } $line =~ s/#ind_id/$indiv_id/g; $line =~ s/#ind_name/$indiv_name{$indiv_id}/g; if ( $extra_vars{"#ind_givname"} ) { $line =~ s/#ind_givname/$indiv_givname{$indiv_id}/g; } if ( $extra_vars{"#ind_forname"} ) { $line =~ s/#ind_forname/$indiv_forname{$indiv_id}/g; }  if ( $extra_vars{"#ind_surname"} ) { $line =~ s/#ind_surname/$indiv_surname{$indiv_id}/g; }  $line =~ s/#ind_photo/$ind_photo/g;
 $line =~ s/#parents_chart/$parents_chart/g; $ind_birt_date = Ged_date($indiv_birt_date{$indiv_id}); $ind_deat_date = Ged_date($indiv_deat_date{$indiv_id}); $line =~ s/#ind_birt_date/$ind_birt_date/g;
 if (!$ind_deat_date and !$indiv_deat_plac{$indiv_id} and !$indiv_deat_city{$indiv_id}) { $line =~ s/#Idate/Ioff/g; } else { $line =~ s/#Idate/Idate/g; } $line =~ s/#ind_deat_date/$ind_deat_date/g; if ($full_place) { $line =~ s/#ind_birt_place/$indiv_birt_plac{$indiv_id}/g; $line =~ s/#ind_deat_place/$indiv_deat_plac{$indiv_id}/g; } else { $line =~ s/#ind_birt_place/$indiv_birt_city{$indiv_id}/g; $line =~ s/#ind_deat_place/$indiv_deat_city{$indiv_id}/g; }
 &cond_print("$line"); } } $old_marr_year = ''; @fams     = sort { marr_year($a)<=>marr_year($b) } split ( //, $indiv_fams{$indiv_id} ); $fams_len = @fams; if ( $fams_len != 0 ) { for ( $fams_num = 0 ; $fams_num < $fams_len ; $fams_num++ ) { if ( $indiv_sex{$indiv_id} eq "M" ) { $spouse_id = $fam_wife{ $fams[$fams_num] }; }
 elsif ( $indiv_sex{$indiv_id} eq "F" ) { $spouse_id = $fam_husb{ $fams[$fams_num] }; } else { print STDERR "\nPACS $fams_num $fams_len / $fams[$fams_num] / $indiv_id\n"; } $spouse = $indiv_name{$spouse_id}; if ($est_place and !$indiv_place{$indiv_id}) { if ($fam_marr_plac{$fams[$fams_num]}) { $indiv_place{$indiv_id} = $fam_marr_plac{$fams[$fams_num]}; $indiv_city {$indiv_id} = $fam_marr_city{$fams[$fams_num]}; $indiv_orig{$indiv_id} = "?"; if ($place_index) { $indiv_placx{$indiv_id} = Ged_placx($indiv_place{$indiv_id}); } } }
 if ($est_date) { if ( !$indiv_birt_year{$indiv_id} and $fam_marr_year{$fams[$fams_num]}) { $indiv_birt_year{$indiv_id} = $fam_marr_year{$fams[$fams_num]}-30; $indiv_birt_date{$indiv_id} = '~'.$indiv_birt_year{$indiv_id}.'?'; } } if ($spouse) { if (($sosa_only and (!$indiv_sosa{$spouse_id})) or ($desc_only and (!$branch_id{$indiv_branch{$spouse_id}}))) { $spouse_ref = "$spouse";
 $indiv_rel{$spouse_id} = $indiv_id; } else { $spouse_ref = "<a href=\"$indiv_file{$spouse_id}.html#$spouse_id\">$spouse</a>"; } if ($est_date) { if ( !$indiv_birt_year{$indiv_id} and $indiv_birt_year{$spouse_id}) { $indiv_birt_year{$indiv_id} = $indiv_birt_year{$spouse_id}; $indiv_birt_date{$indiv_id} = '~'.$indiv_birt_year{$indiv_id}.'?'; } } }
 else { $spouse_ref = ""; } if ( $fams_len > 1 ) { $marr_num = $fams_num + 1; $marr_num = '<sup>' . $marr_num . '</sup>'; } else { $marr_num = ""; } if ( $fam_marr_plac{ $fams[$fams_num] } ) { $marr_place = $fam_marr_plac{ $fams[$fams_num] }; $marr_city = $fam_marr_city{ $fams[$fams_num] }; }
 else { $marr_city = ""; $marr_place = "";} if ( $hide_XX && ($fam_marr_year{$fams[$fams_num]} + 80 > $curr_year) ) { $fam_marr_date{$fams[$fams_num]} = "$priv_string";  $fam_marr_year{$fams[$fams_num]} = "";  $fam_marr_plac{$fams[$fams_num]} = "";  $fam_marr_city{$fams[$fams_num]} = "";  } 
 if ($ind_opt) { $old_birt_year = ''; @child = sort { birt_year($a) <=> birt_year($b) } split ( //, $fam_chil{ $fams[$fams_num] } ); $num_children   = @child; $children_chart = ""; if ( $num_children > 0 ) { @c_span = ( 0, 0, 3, 3, 2, 2, 1, 1, 1, 1 );
 @c_midd = ( 0, 9, 1, 1, 1, 1, 1, 1, 1, 1 ); @c_left = ( 0, 0, 1, 1, 0, 0, 1, 1, 0, 0 ); if ($css) {$c_fontd = " class=Ic>"; $c_fontf = "";}  else {$c_fontd = "><font size=-1>"; $c_fontf = "</font>";} if ( ( $num_children > 2 ) && ( $num_children != 9 ) ) { $children_chart .= "<tr>";
 for ( $i = 0 ; $i < 4 ; $i++ ) { $children_chart .= "<td class=trans width=67 height=1></td>"; } $children_chart .= "<td class=trans width=64 height=1></td>"; for ( $i = 0 ; $i < 4 ; $i++ ) { $children_chart .= "<td class=trans width=67 height=1></td>"; } $children_chart .= "</tr>"; }
 for ( $i = 0 ; $i < $num_children ; $i++ ) { if ($sosa_only and (!$indiv_sosa{$child[$i]})) { $child_ref[$i] = "$indiv_name{$child[$i]}"; $indiv_rel{$child[$i]} = $indiv_id; } else { if ($indiv_sosa{$child[$i]}) { $indiv_link{$child[$i]} = $sosa_link; } $child_ref[$i] = "<a href=\"$indiv_file{$child[$i]}.html#$child[$i]\">				$indiv_name{$child[$i]}</a>$indiv_link{$child[$i]}"; }
 $child_ref[$i] =~ s/\-/ /g; if ($est_date) { if ( !$indiv_birt_year{$indiv_id} and $indiv_birt_year{$child[$i]}) { $indiv_birt_year{$indiv_id} = $indiv_birt_year{$child[$i]}-31; $indiv_birt_date{$indiv_id} = '~'.$indiv_birt_year{$indiv_id}.'?'; } } } $start_child = 0;
 while ( ( $num_children - $start_child ) > 9 ) { $children_chart .= "<tr align=center><td class=t_c9 colspan=9 width=600 height=30></td></tr>" ."<tr align=center>"; for ($i=$start_child;   $i<$start_child+4; $i++) { $children_chart .=  "<TD".$c_fontd."$child_ref[$i]&nbsp;".$c_fontf."</td>"; }
 $children_chart .= "<td class=t_l width=64 height=60></td>"; for ($i=$start_child+4; $i<$start_child+8; $i++) { $children_chart .= "<TD".$c_fontd ."$child_ref[$i]&nbsp;".$c_fontf."</td>"; } $children_chart .= "</tr>"; $start_child += 8; } $num_left       = $num_children - $start_child;
 $children_chart .= "<tr align=center><td class=t_c$num_left colspan=9 width=600 height=30>" ."</td></tr><tr align=center>"; if ( $c_left[$num_left] ) { $children_chart .= "<td colspan=$c_left[$num_left]></td>"; } $c_num = int( $num_left / 2 ); for ( $i=$start_child; $i<$start_child+$c_num; $i++ ) { $children_chart .= "<td colspan=$c_span[$num_left]"
 .$c_fontd."$child_ref[$i]&nbsp;".$c_fontf."</td>"; } if ( $num_left % 2 ) { $children_chart .= "<td colspan=$c_midd[$num_left]" .$c_fontd."$child_ref[$start_child+$c_num]&nbsp;".$c_fontf."</td>"; $c_num++; } else { $children_chart .= "<td colspan=$c_midd[$num_left]>&nbsp;</td>"; }
 for ( $i=$start_child+$c_num; $i<$start_child+$num_left; $i++ ) { $children_chart .= "<td colspan=$c_span[$num_left]" .$c_fontd."$child_ref[$i]&nbsp;".$c_fontf."</td>"; } if ( $c_left[$num_left] ) { $children_chart .= "<td colspan=$c_left[$num_left]>&nbsp;</td>"; } $children_chart .= "</tr>"; }
 for ( $i = $item_beg ; $i < $item_beg + $item_cnt ; $i++ ) { if (!$vartemp[$i]) { &cond_print("$template[$i]"); next; } $line = $template[$i]; $line =~ s/#marr_num/$marr_num/g; $marr_date  = Ged_date($fam_marr_date{$fams[$fams_num]}); $line =~ s/#marr_date/$marr_date/g;
 if ($full_place) { $line =~ s/#marr_place/$marr_place/g; } else { $line =~ s/#marr_place/$marr_city/g; } $line =~ s/#marr_city/$marr_city/g;  $line =~ s/#spouse_ref/$spouse_ref/g; $line =~ s/#children_chart/$children_chart/g; &cond_print("$line"); } } } } if ($ind_opt) { if ( $indiv_events{$indiv_id} ) { $ind_events = events_edit($indiv_events{$indiv_id},'events'); } if ( $indiv_occu{$indiv_id} ) { $ind_occu = events_edit($indiv_occu{$indiv_id},'occu'); } if ( $indiv_titl{$indiv_id} ) { $ind_titl = events_edit($indiv_titl{$indiv_id},'titl'); } for ( $i = $fblock_beg ; $i < $fblock_beg + $fblock_cnt ; $i++ ) { if (!$vartemp[$i]) { &cond_print("$template[$i]"); next; } $line = $template[$i];
 if ( $print_notes && $indiv_note{$indiv_id} ) {  $line =~ s/<!#ind_note\s?(.*?)\>/$1/g; $indiv_note{$indiv_id} =~ s/#indiv_file\((.*?)\)/$indiv_file{$1}/g; $indiv_note{$indiv_id} =~ s/([^'])(http:\/\/[^\s<]*)/$1<a class='main1' href='$2' target=_blank>$2<\/a>/gi;  if ($indiv_note{$indiv_id} !~ /<LI>/) {  $line =~ s/#ind_note/$indiv_note{$indiv_id}/ ; }
 else {  $line =~ s/#ind_note(<br>)?/<UL class=note>$indiv_note{$indiv_id}<\/UL>/ ;  } } else {  $line =~ s/<!#ind_note.*?>//g;  $line =~ s/^.*?#ind_note\s*(<br>)?.*?$//g;  } if ( $print_sources && $indiv_source{$indiv_id} ) { $line =~ s/<!#ind_sour\s?(.*?)\>/$1/g;
 @sources = split ( //, $indiv_source{$indiv_id} ); $sour_len = @sources; $ind_sour = ''; %ind_sour_id = (); for ($j=1;$j<$sour_len;$j++) { $ind_sour_id{$sources[$j]} = 1; } foreach $sour_id ( sort { $a cmp $b } keys %ind_sour_id ) { $ind_sour .= '<a href="../'.$sour_path.'/'.$sour_file{$sour_id}.'.html#'.$sour_id.'" target="_blank">'.$sour_id.'</a> '; }
 $line =~ s/#ind_sour/$ind_sour/ ; } else {  $line =~ s/<!#ind_sour.*?>//g;  $line =~ s/^.*?#ind_sour\s*(<br>)?.*?$//g;  } if ( $indiv_titl{$indiv_id} ) { $line =~ s/<!#ind_titl\s?(.*?)\>/$1/g; $line =~ s/#ind_titl(<br>)?/$ind_titl/ ;  } else {  $line =~ s/<!#ind_titl.*?>//g; 
 $line =~ s/^.*?#ind_titl\s*(<br>)?.*?$//g;  } if ( $indiv_events{$indiv_id} and $ind_events ) { $line =~ s/<!#ind_events\s?(.*?)\>/$1/g; $line =~ s/#ind_events(<br>)?/$ind_events/ ;  } else {  $line =~ s/<!#ind_events.*?>//g;  $line =~ s/^.*?#ind_events\s*(<br>)?.*?$//g;  }
 if ( $indiv_occu{$indiv_id} ) { $line =~ s/<!#ind_occu\s?(.*?)\>/$1/g; $line =~ s/#ind_occu(<br>)?/$ind_occu/ ;  } else {  $line =~ s/<!#ind_occu.*?>//g;  $line =~ s/^.*?#ind_occu\s*(<br>)?.*?$//g;  } &cond_print("$line"); } } if ($est_place and !$indiv_place{$indiv_id}) { if ( $indiv_deat_plac{$indiv_id} ) { $indiv_place{$indiv_id} = $indiv_deat_plac{$indiv_id}; $indiv_city {$indiv_id} = $indiv_deat_city{$indiv_id}; $indiv_orig{$indiv_id} = "?"; }
 elsif ( $fam_marr_plac{ $indiv_famc{$indiv_id} } ) { $indiv_place{$indiv_id} = $fam_marr_plac{ $indiv_famc{$indiv_id} }; $indiv_city {$indiv_id} = $fam_marr_city{ $indiv_famc{$indiv_id} }; $indiv_orig{$indiv_id} = "?"; } elsif ($indiv_birt_plac{$fam_husb{$indiv_famc{$indiv_id}}}) {  $indiv_place{$indiv_id} = $indiv_birt_plac{$fam_husb{$indiv_famc{$indiv_id}}}; $indiv_city {$indiv_id} = $indiv_birt_city{$fam_husb{$indiv_famc{$indiv_id}}}; $indiv_orig{$indiv_id} = "?"; }
 elsif ($indiv_birt_plac{$fam_wife{$indiv_famc{$indiv_id}}}) {  $indiv_place{$indiv_id} = $indiv_birt_plac{$fam_wife{$indiv_famc{$indiv_id}}}; $indiv_city {$indiv_id} = $indiv_birt_city{$fam_wife{$indiv_famc{$indiv_id}}}; $indiv_orig{$indiv_id} = "?"; } }
 if (($indiv_refn{$indiv_id} eq "prv") or ($indiv_birt_date{$indiv_id} eq "$priv_string")) { $indiv_place{$indiv_id} = ""; $indiv_city{$indiv_id} = ""; $indiv_orig{$indiv_id} = ""; } if ( $indiv_place{$indiv_id} =~ /^(.*?)\s*\?+$/ ) {  $indiv_place{$indiv_id} = $1;  $indiv_orig{$indiv_id} = "?"; } if ( $indiv_place{$indiv_id} ) { $place_count{ $indiv_place{$indiv_id} }++; if ($place_index) { $indiv_placx{$indiv_id} = Ged_placx($indiv_place{$indiv_id}); } if ( $place_index ) { $placx_count{ $indiv_placx{$indiv_id} }++; if ( $placx_count{ $indiv_placx{$indiv_id} } == 1 ) { $loc_k{ $indiv_placx {$indiv_id} } = $indiv_placx{$indiv_id};  } }
 else { if ( $place_count{ $indiv_place{$indiv_id} } == 1 ) { $loc_k{ $indiv_place{$indiv_id} } = $indiv_place{$indiv_id}; } } } if ($est_date) { if ( !$indiv_birt_year{$indiv_id} ) { if ( $fam_marr_year{$indiv_famc{$indiv_id}} and $fam_marr_year{$indiv_famc{$indiv_id}} < $curr_year-3 ) { $indiv_birt_year{$indiv_id} = $fam_marr_year{$indiv_famc{$indiv_id}}+3; $indiv_birt_date{$indiv_id} = '~'.$indiv_birt_year{$indiv_id}.'?'; }
 elsif ($indiv_birt_year{$fam_wife{$indiv_famc{$indiv_id}}} and $indiv_birt_year{$fam_wife{$indiv_famc{$indiv_id}}} < $curr_year-30 ) {  $indiv_birt_year{$indiv_id} = $indiv_birt_year{$fam_wife{$indiv_famc{$indiv_id}}}+30; $indiv_birt_date{$indiv_id} = '~'.$indiv_birt_year{$indiv_id}.'?'; }
 elsif ($indiv_birt_year{$fam_husb{$indiv_famc{$indiv_id}}} and $indiv_birt_year{$fam_husb{$indiv_famc{$indiv_id}}} < $curr_year-30 ) {  $indiv_birt_year{$indiv_id} = $indiv_birt_year{$fam_husb{$indiv_famc{$indiv_id}}}+30; $indiv_birt_date{$indiv_id} = '~'.$indiv_birt_year{$indiv_id}.'?'; } } } if ( $hide_XX and !$indiv_deat{$indiv_id}) { if ( $indiv_birt_year{$indiv_id} + 100 > $curr_year ) { $indiv_birt_date{$indiv_id} = "$priv_string"; $indiv_birt_plac{$indiv_id} = ""; $indiv_note{$indiv_id}      = ""; $indiv_titl{$indiv_id}      = ""; $indiv_occu{$indiv_id}      = ""; $indiv_events{$indiv_id}    = ""; } }
 if (($indiv_refn{$indiv_id} eq "prv") or ($indiv_birt_date{$indiv_id} eq "$priv_string")) { $indiv_birt_year{$indiv_id} = ""; } } &cond_out("$lang_dir/$file_id.html"); $ind_ok = 1; if ($nam_opt) { $nbrow_span++; $out_opt = 1; &get_template("nam-tpl",'#ind_occu',''); &log_print( 'l_cre_perso'); $free_cnt    = 0; $file_id     = $namf_name;
 $first_file  = $namf_name; $prev_file   = $namf_name; $next_file   = $file_id; $file_span   = 0; $num_names   = 0; $new_file    = 1; $old_surname = "-o-o-"; $old_letter  = "-"; INDI2 : foreach $indiv_id ( sort by_surname keys %ind_k ) { if(!$indiv_id) { next; } if (($sosa_only) and (!$indiv_sosa{$indiv_id}) and (!$indiv_rel{$indiv_id})) { next INDI2; } if (($desc_only) and (!$branch_id{$indiv_branch{$indiv_id}}) and (!$indiv_rel{$indiv_id})) { next INDI2; } if ( $file_span && ( $free_cnt <= 0 ) ) { $new_file = 1; } if ( noaccnosp($indiv_surname{$indiv_id}) ne $old_surname ) { $file_span = 0;
 if ( $free_cnt <= 0 ) { $new_file = 1; } elsif ( $name_cnt{ $indiv_surname{$indiv_id} } > $free_cnt ) { if ( $name_cnt{ $indiv_surname{$indiv_id} } <= $namf_size ) { $new_file = 1; } else { $file_span = 1; } } } if ($new_file) { if ($num_names) { &cond_out("$lang_dir/$file_id.html");
 $prev_file = $file_id; $file_id++; } &open_new("$lang_dir/$file_id.html"); $next_file++; &print_header; $first_block = 1; if ($file_span) { for ( $i = $block_beg ; $i < $block_beg + $block_cnt ; $i++ ) { if (!$vartemp[$i]) { &cond_print("$template[$i]"); next; } $line = $template[$i];
 $line =~ s/#rupture/$old_letter/g; &cond_print("$line"); } $first_block = 0; } $free_cnt = $namf_size; $new_file = 0; } if ( noaccnosp($indiv_surname{$indiv_id}) ne $old_surname ) { $new_letter = substr( noaccent( $indiv_surname{$indiv_id} ), 0, 1 ); if ( $new_letter ne $old_letter ) { if (!$first_block) { for ( $i = $fblock_beg ; $i < $fblock_beg + $fblock_cnt ; $i++ ) { &cond_print("$template[$i]"); } }
 $names_alpha{$new_letter} = "$file_id.html#$new_letter"; for ( $i = $block_beg ; $i < $block_beg + $block_cnt ; $i++ ) { if (!$vartemp[$i]) { &cond_print("$template[$i]"); next; } $line = $template[$i]; $line =~ s/#block_ref/$new_letter/g; $line =~ s/#rupture/$new_letter/g; &cond_print("$line"); }
 $old_letter = $new_letter; $first_block = 0; } $num_names++; $item_ref = '<a name="' . $num_names . '">&nbsp;</a>'; $old_surname = noaccnosp($indiv_surname{$indiv_id}); $name_ref{$old_surname} = "$file_id.html#$num_names"; } else { $item_ref = ""; } if ( $extra_vars{"#ind_occu"} and $indiv_occu{$indiv_id} ) { $ind_occu = events_edit($indiv_occu{$indiv_id},'occu'); } else  { $ind_occu = ""; } $ind_birt_date = Ged_date($indiv_birt_date{$indiv_id}); $ind_deat_date = Ged_date($indiv_deat_date{$indiv_id}); for ( $i = $item_beg ; $i < $item_beg + $item_cnt ; $i++ ) { if (!$vartemp[$i]) { &cond_print("$template[$i]"); next; } $line = $template[$i];
 $line =~ s/<!#item_ref>/$item_ref/g; if ($indiv_file{$indiv_id}) { $line =~ s/#ind_filename/$indiv_file{$indiv_id}.html#$indiv_id/g; } else {						   $line =~ s/#ind_filename/$indiv_file{$indiv_rel{$indiv_id}}.html#$indiv_rel{$indiv_id}/g; }
 $line =~ s/#ind_surname/$indiv_surname{$indiv_id}/g;   if ($indiv_sosa{$indiv_id}) {  $line =~ s/#ind_forname/$indiv_forname{$indiv_id}/g; $line =~ s/#iclass/sosa/g; } else {  $line =~ s/#ind_forname/$indiv_forname{$indiv_id}/g;	 } if ($print_desc && $branch_id{$indiv_branch{$indiv_id}}) {  $ind_dabo = $branch_id{$indiv_branch{$indiv_id}}.'-'.$indiv_dabo{$indiv_id}; }
 else {$ind_dabo = '&nbsp;';	} $line =~ s/#ind_dabo/$ind_dabo/g; $line =~ s/#ind_birt_date/$ind_birt_date/g; $line =~ s/#ind_deat_date/$ind_deat_date/g; if ($full_place) { $line =~ s/#ind_place/$indiv_place{$indiv_id}$indiv_orig{$indiv_id}/g; } else { $line =~ s/#ind_place/$indiv_city{$indiv_id}$indiv_orig{$indiv_id}/g; }
 $line =~ s/#ind_birt_year/$indiv_birt_year{$indiv_id}/g; $line =~ s/#ind_deat_year/$indiv_deat_year{$indiv_id}/g; if ( $extra_vars{"#ind_occu"} ) { $line =~ s/#ind_occu/$ind_occu/g; } $line =~ s/#ind_sosa/$indiv_sosa{$indiv_id}/g; &cond_print("$line"); } $free_cnt--; } &cond_out("$lang_dir/$file_id.html");
 &print_skip($lang_dir,$next_file,$first_file); $out_opt = 1; &get_template("nami-tpl",'',('/#ind_cnt/$ind_cnt/g','/#fam_cnt/$num_names/g')); &log_print('l_cre_nam_x'); $free_cnt   = 0; $file_id    = $namx_name; $first_file = $namx_name; $prev_file  = $namx_name;
 if ( $num_names > $namx_size ) { $next_file = $file_id; } else { $next_file = $namx_name; } $old_letter = "-"; foreach $nam_id ( sort { nameconv($a) cmp nameconv($b) } keys %nam_k ) { $free_cnt--; if ( substr( noaccent($nam_id), 0, 1 ) ne $old_letter ) { if ( $free_cnt <= 0 ) { if ( $next_file ne $first_file ) { &cond_out("$lang_dir/$file_id.html");
 $prev_file = $file_id; $file_id++; } &open_new("$lang_dir/$file_id.html"); $next_file++; &print_header; $first_block = 1; $free_cnt = $namx_size; } if (!$first_block) { for ( $i = $fblock_beg ; $i < $fblock_beg + $fblock_cnt ; $i++ ) { &cond_print("$template[$i]"); } }
 $old_letter = substr( noaccent($nam_id), 0, 1 ); $namx_file{$old_letter} = "$file_id.html"; for ( $i = $block_beg ; $i < $block_beg + $block_cnt ; $i++ ) { if (!$vartemp[$i]) { &cond_print("$template[$i]"); next; } $line = $template[$i]; $line =~ s/#rupture/$old_letter/g; &cond_print("$line"); }
 $first_block = 0; } $next_ref = ""; if ( $name_cnt{$nam_id} > $namf_size) { $name_ref{noaccnosp($nam_id)} =~ /^(.*?)\.html.*/; $next_page = $1; $next_ref = "<sup>"; $next_link = 1; for ( $k = $name_cnt{$nam_id} ; $k > $namf_size ; $k-=$namf_size ) { $next_page++;
 $next_ref .=  "<a class=index href=\"$next_page.html\" target=Main>$next_link</a>"; $next_link++; } $next_ref .= "</sup>"; } $surname_id = $nam_id; for ( $i = $item_beg ; $i < $item_beg + $item_cnt ; $i++ ) { if (!$vartemp[$i]) { &cond_print("$template[$i]"); next; } $line = $template[$i];
 $line =~ s/#surname_ref/$name_ref{noaccnosp($nam_id)}/g; $line =~ s/#surname_id/$surname_id/g;  if ( $name_cnt{$nam_id} > 1 ) { $line =~ s/<!#surname_count>/\($name_cnt{$nam_id}\)$next_ref/g; } else { $line =~ s/<!#surname_count>//g; } &cond_print("$line"); } } &cond_out("$lang_dir/$file_id.html");
 &print_skip($lang_dir,$next_file,$first_file); } if ($loc_opt) { $nbrow_span++; $out_opt = 1; &get_template("loc-tpl",'#ind_occu|#ind_dabo',''); &log_print('l_cre_place'); $free_cnt   = 0; $file_id    = $locf_name; $first_file = $locf_name; $prev_file  = $locf_name; $next_file  = $file_id;
 $file_span  = 0; $num_places = 0; $new_file   = 1; $old_place = "-o-o-"; $old_placx = "-o-o-"; $old_letter = "-"; INDI3 : foreach $indiv_id ( sort by_place keys %ind_k ) { if (($sosa_only) and (!$indiv_sosa{$indiv_id})) { next INDI3; } if (($desc_only) and (!$branch_id{$indiv_branch{$indiv_id}})) { next INDI3; } if ($indiv_fams{$indiv_id}) { $old_marr_year = '';
 @fams = sort { marr_year($a)<=>marr_year($b) } split ( //, $indiv_fams{$indiv_id} );  $ind_marr1_year = $fam_marr_year{$fams[0]};  $ind_marr1_city = $fam_marr_city{$fams[0]}; $ind_marr1_plac = $fam_marr_plac{$fams[0]}; } else { $ind_marr1_year = ""; $ind_marr1_plac = ""; $ind_marr1_city = ""; }  if ( $indiv_place{$indiv_id} ) { if ( $file_span && ( $free_cnt <= 0 ) ) { $new_file = 1; } if ( $indiv_place{$indiv_id} ne $old_place ) { $file_span = 0;
 if ( $free_cnt <= 0 ) { $new_file = 1; } elsif ( $place_count{ $indiv_place{$indiv_id} } > $free_cnt ) { if ( $place_count{ $indiv_place{$indiv_id} } <= $locf_size ) { $new_file = 1; } else { $file_span = 1; } } } if ($new_file) { if ($num_places) { &cond_out("$lang_dir/$file_id.html");
 $prev_file = $file_id; $file_id++; } &open_new("$lang_dir/$file_id.html"); $next_file++; &print_header; $first_block = 1; if ($file_span) { for ( $i=$block_beg; $i<$block_beg+$block_cnt; $i++ ) { if (!$vartemp[$i]) { &cond_print("$template[$i]"); next; } $line = $template[$i];
 $line =~ s/#block_ref//g; if ($indiv_place{$indiv_id} !~ /^\s*$/) {  $line =~ s/#rupture/$indiv_place{$indiv_id}/g; } else {  $line =~ s/#rupture/?/g; } $line =~ s/#placelink//; &cond_print("$line"); } $first_block = 0; } $free_cnt = $locf_size; $new_file = 0; }
 $new_place = $indiv_place{$indiv_id}; $new_placx = $indiv_placx{$indiv_id};  if ( $new_place ne $old_place ) { if (!$first_block) { for ( $i = $fblock_beg ; $i < $fblock_beg + $fblock_cnt ; $i++ ) { &cond_print("$template[$i]"); } } $first_block = 0; $placelink = ''; if ($plinks) { $full_place = lc($indiv_place{$indiv_id}); $full_place =~ s/-/ /g;
 $new_city = lc($indiv_city{$indiv_id}); $new_city = noaccent($new_city); $new_city =~ s/-/ /g; $new_city =~ s/^(.*?)\?$/$1/g; if ($place_link{$full_place}) { $placelink = '<A HREF="'.$place_link{$full_place}.'"><IMG ALIGN=middle SRC="'.$pics_path.'/oeil.gif" BORDER=0></A>'; }
 elsif ($place_link{$new_city}) { $placelink = '<A HREF="'.$place_link{$new_city}.'"><IMG ALIGN=middle SRC="'.$pics_path.'/oeil.gif" BORDER=0></A>'; } elsif ($full_place =~ /[\s\(,](\d{2})/) { $new_city .= ' '.$1; if ($place_link{$new_city}) { $placelink = '<A HREF="'.$place_link{$new_city}.'"><IMG ALIGN=middle SRC="'.$pics_path.'/oeil.gif" BORDER=0></A>'; }	 } }
 for ( $i=$block_beg; $i<$block_beg+$block_cnt; $i++ ) { if (!$vartemp[$i]) { &cond_print("$template[$i]"); next; } $line = $template[$i]; $line =~ s/#block_ref/$num_places/g; if ($new_place !~ /^\s*$/) {  $line =~ s/#rupture/$new_place/g; } else {  $line =~ s/#rupture/?/g; } $line =~ s/#placelink/$placelink/;
 &cond_print("$line"); } $old_place = $new_place; if ($place_index and ($new_placx ne $old_placx)) { $place_ref{ $indiv_placx{$indiv_id} } = "$file_id.html#$num_places"; $old_placx = $new_placx ; } else { $place_ref{ $indiv_place{$indiv_id} } = "$file_id.html#$num_places"; } if ( $indiv_placx{$indiv_id} ) { $loc_id = $indiv_placx{$indiv_id} } else { $loc_id = $indiv_place{$indiv_id} } if ( $loc_id =~ /^\d{2}/) { $new_letter = substr( $loc_id, 0, 2 ); }
 else { $new_letter = substr( noaccent($loc_id), 0, 1 ); } if ( $new_letter ne $old_letter ) { $old_letter = $new_letter; $places_alpha{$old_letter} = "$file_id.html#$num_places"; } $num_places++; }
 if ( $extra_vars{"#ind_occu"} and $indiv_occu{$indiv_id} ) { $ind_occu = events_edit($indiv_occu{$indiv_id},'occu'); } else  { $ind_occu = ""; } if ($print_desc and $extra_vars{"#ind_dabo"} and  $branch_id{$indiv_branch{$indiv_id}}) {  $ind_dabo = $branch_id{$indiv_branch{$indiv_id}}.'-'.$indiv_dabo{$indiv_id}; } else {$ind_dabo = '&nbsp;';	} for ( $i = $item_beg ; $i < $item_beg + $item_cnt ; $i++ ) { if (!$vartemp[$i]) { &cond_print("$template[$i]"); next; } $line = $template[$i];
 if ($indiv_sosa{$indiv_id}) { $line =~ s/#iclass/sosa/g; } $line =~ s/#ind_filename/$indiv_file{$indiv_id}.html#$indiv_id/g; $line =~ s/#ind_surname/$indiv_surname{$indiv_id}/g;   if ((!$sosa_only) and ($indiv_sosa{$indiv_id})) {  $line =~ s/#ind_forname/$indiv_forname{$indiv_id}$sosa_link/g; }
 else { $line =~ s/#ind_forname/$indiv_forname{$indiv_id}/g;	} $line =~ s/#ind_birt_year/$indiv_birt_year{$indiv_id}/g; $line =~ s/#ind_birt_plac/$indiv_birt_city{$indiv_id}/g; $line =~ s/#ind_deat_year/$indiv_deat_year{$indiv_id}/g; $line =~ s/#ind_deat_plac/$indiv_deat_city{$indiv_id}/g; $line =~ s/#ind_marr1_year/$ind_marr1_year/g;
 $line =~ s/#ind_marr1_plac/$ind_marr1_city/g; if ( $extra_vars{"#ind_occu"} ) { $line =~ s/#ind_occu/$ind_occu/g; } if ( $extra_vars{"#ind_dabo"} ) { $line =~ s/#ind_dabo/$ind_dabo/g; } $line =~ s/#ind_sosa/$indiv_sosa{$indiv_id}/g; &cond_print("$line"); } $free_cnt--; } }
 &cond_out("$lang_dir/$file_id.html"); &print_skip($lang_dir,$next_file,$first_file); $out_opt = 1; &get_template("loci-tpl",'',('/#ind_cnt/$ind_cnt/g','/#loc_cnt/$num_places/g')); &log_print('l_cre_pla_x'); $free_cnt   = 0; $file_id    = $locx_name; $first_file = $locx_name;
 $prev_file  = $locx_name; $next_file = $locx_name; $old_letter = "-"; foreach $loc_id ( sort { placeconv($a) cmp placeconv($b) } keys %loc_k ) { $free_cnt--; $place_id = $loc_k{$loc_id};  if ( $loc_id =~ /^\d{2}/) {	$new_letter = substr( $loc_id, 0, 2 ); } else { $new_letter = majuscule(substr( $loc_id, 0, 1 )); } if ( $new_letter ne $old_letter ) { if ( $free_cnt <= 0 ) { if ( $next_file ne $first_file ) { &cond_out("$lang_dir/$file_id.html");
 $prev_file = $file_id; $file_id++; } &open_new("$lang_dir/$file_id.html"); $next_file++; &print_header; $first_block = 1; $free_cnt = $locx_size; } if (!$first_block) { for ( $i = $fblock_beg ; $i < $fblock_beg + $fblock_cnt ; $i++ ) { &cond_print("$template[$i]"); } }
 $first_block = 0; $old_letter = $new_letter; $locx_file{$old_letter} = "$file_id.html"; for ( $i = $block_beg ; $i < $block_beg + $block_cnt ; $i++ ) { if (!$vartemp[$i]) { &cond_print("$template[$i]"); next; } $line = $template[$i]; $line =~ s/#rupture/$old_letter/g;
 &cond_print("$line"); } } @items = split (//, $place_id); $place_id =~ s// /g; $place_id =~ s/,/ /g; $place_id =~ s/^(.*)\s*$/$1/; $place_id =~ s/^\s*$/?/; $next_ref = ""; if ($place_index) { $loc_count = $placx_count{$loc_id}; } else { $loc_count =	$place_count{$loc_id}; } if ( $loc_count > $locf_size) { $place_ref{$loc_k{$loc_id}} =~ /^(.*?)\.html.*/; $next_page = $1; 
 $next_ref = "<sup>"; $next_link = 1; for ( $k = $loc_count ; $k > $locf_size ; $k-=$locf_size ) { $next_page++; $next_ref .=  "<a class=index href=\"$next_page.html\" target=Main>$next_link</a>"; $next_link++; } $next_ref .= "</sup>"; }
 for ( $i = $item_beg ; $i < $item_beg + $item_cnt ; $i++ ) { if (!$vartemp[$i]) { &cond_print("$template[$i]"); next; } $line = $template[$i]; $line =~ s/#place_ref/$place_ref{$loc_k{$loc_id}}/g; for ( $j = 1 ; $j < 10 ; $j++ )	{ if ($pxx[$j]) {	 $line =~ s/#place_id$pxx[$j]/$items[$j-1]/g; } }
 $line =~ s/#place_id/$place_id/g; if ( $loc_count > 1 ) { $line =~ s/<!#place_count>/\($loc_count\)$next_ref/g; } else { $line =~ s/<!#place_count>//g; } &cond_print("$line"); } } &cond_out("$lang_dir/$file_id.html"); &print_skip($lang_dir,$next_file,$first_file); }
 if ( ($gen_number) && ($indi_root) ) { $nbrow_span++; $out_opt = 1; &get_template("gen-tpl",'#husb_occu|#wife_occu|#husb_years|#wife_years',''); &log_print('l_cre_ascen' ); $free_cnt   = 0; $file_id    = $genf_name; $first_file = $genf_name; $prev_file  = $genf_name; $next_file  = $file_id;
 $file_span  = 0; $num_sosas  = 0; $new_file   = 1; $new_block = 1; $first_block = 1; $gen_old    = 0; $gen_curr   = 1;			  foreach $sosa ( sort by_gen_sosa keys %sosa_fam ) { if(!$sosa) { next; } $fam_id   = $sosa_fam{$sosa}; if ( $file_span && ( $free_cnt <= 0 ) ) { $new_file = 1; } if ( $sosa_gen{$sosa} > $gen_old ) { $new_block =1;
 if ( $gen_curr > $gen_number ) { last } $file_span = 0; if ( $free_cnt <= 0 ) { $new_file = 1; } elsif ( $sosa_cnt[ $gen_curr ] > $free_cnt ) { if ( $sosa_cnt[ $gen_curr ] <= $genf_size ) { $new_file = 1; } else { $file_span = 1; } } } if ($new_file) { if ($num_sosas) { &cond_out("$lang_dir/$file_id.html");
 $sosaload{$sosa_max} = "$file_id.html"; $prev_file = $file_id; $file_id++; } &open_new("$lang_dir/$file_id.html"); $next_file++; &print_header; $first_block = 1; if ($file_span) { for ( $i = $block_beg ; $i < $block_beg + $block_cnt ; $i++ ) { if (!$vartemp[$i]) { &cond_print("$template[$i]"); next; } $line = $template[$i];
 $line =~ s/#rupture/$gen_old - suite/g; &cond_print("$line"); } $first_block = 0; } $free_cnt = $genf_size; $new_file = 0; } if ( $new_block ) { if (!$first_block) { for ( $i = $fblock_beg ; $i < $fblock_beg + $fblock_cnt ; $i++ ) { &cond_print("$template[$i]"); } }
 $gen_ref[$gen_curr] = $file_id; for ( $i = $block_beg ; $i < $block_beg + $block_cnt ; $i++ ) { if (!$vartemp[$i]) { &cond_print("$template[$i]"); next; } $line = $template[$i]; $line =~ s/#rupture/$gen_curr/g; &cond_print("$line"); } $first_block = 0; $new_block = 0; $gen_old++;
 $gen_curr++; } $husb_ref = ""; if ( $fam_husb{$fam_id} ) { $num_sosas++; $husb_ref = '<a href="' . $indiv_file{ $fam_husb{$fam_id} } . '.html#' . $fam_husb{$fam_id} . '">'; } $wife_ref = ""; if ( $fam_wife{$fam_id} ) { $num_sosas++;
 $wife_ref = '<a href="' . $indiv_file{ $fam_wife{$fam_id} } . '.html#' . $fam_wife{$fam_id} . '">'; } if ($fam_wife{$fam_id}) { $sosaw = sosa_inc($sosa); $sosa_max = $sosaw; } else { $sosaw = ""; $sosa_max = $sosa;} if ($extra_vars{"#husb_occu"} and $indiv_occu{$fam_husb{$fam_id}}) { $husb_occu = events_edit($indiv_occu{$fam_husb{$fam_id}},'occu'); } else { $husb_occu = ""; } if ($extra_vars{"#wife_occu"} and $indiv_occu{$fam_wife{$fam_id}}) { $wife_occu = events_edit($indiv_occu{$fam_wife{$fam_id}},'occu'); } else { $wife_occu = ""; } if ($extra_vars{"#husb_years"} and $indiv_birt_year{$fam_husb{$fam_id}} or $indiv_deat_year{$fam_husb{$fam_id}}) { $husb_years = '(';
 if ( $indiv_birt_date{$fam_husb{$fam_id}} =~ /^\D.*$/ ) { $husb_years .= '~'; } if ( !$indiv_birt_year{$fam_husb{$fam_id}} ) { $husb_years .= '????'; } else { $husb_years .= $indiv_birt_year{$fam_husb{$fam_id}}; } $husb_years .= '-'; if ( $indiv_deat_date{$fam_husb{$fam_id}} =~ /^\D.*$/ ) { $husb_years .= '~'; } if ( !$indiv_deat_year{$fam_husb{$fam_id}} ) { $husb_years .= '????'; } else { $husb_years .= $indiv_deat_year{$fam_husb{$fam_id}}; } $husb_years .= ')'; }
 else { $husb_years = ''; } if ($extra_vars{"#wife_years"} and $indiv_birt_year{$fam_wife{$fam_id}} or $indiv_deat_year{$fam_wife{$fam_id}}) { $wife_years = '('; if ( $indiv_birt_date{$fam_wife{$fam_id}} =~ /^\D.*$/ ) { $wife_years .= '~'; } if ( !$indiv_birt_year{$fam_wife{$fam_id}} ) { $wife_years .= '????'; } else { $wife_years .= $indiv_birt_year{$fam_wife{$fam_id}}; } $wife_years .= '-';
 if ( $indiv_deat_date{$fam_wife{$fam_id}} =~ /^\D.*$/ ) { $wife_years .= '~'; } if ( !$indiv_deat_year{$fam_wife{$fam_id}} ) { $wife_years .= '????'; } else { $wife_years .= $indiv_deat_year{$fam_wife{$fam_id}}; } $wife_years .= ')'; } else { $wife_years = ''; } for ( $i = $item_beg ; $i < $item_beg + $item_cnt ; $i++ ) { if (!$vartemp[$i]) { &cond_print("$template[$i]"); next; } $line = $template[$i];
 if ($sosa) {  $ind_dabo = '';  if (!$indiv_famc{$fam_husb{$fam_id}}) {  $line =~ s/#hclass/dabo/g;  if ($branch_id{$indiv_branch{$fam_husb{$fam_id}}}) { $branch_link = "javascript:descLoad('".$branch_id{$indiv_branch{$fam_husb{$fam_id}}}."','')";  $ind_dabo = '<br><a class=main1 href="'.$branch_link."\"><img src=\"$pics_path/i_d.gif\" align=\"middle\" border=0>".'</a>'; } }
 $line =~ s/#husb_sosa/\<a name\=\'$sosa\'\>$sosa\<\/a>$ind_dabo/g;  } else { $line =~ s/#husb_sosa//ge; } $line =~ s/<!#husb_ref>/$husb_ref/g; $line =~ s/#husb_name/$indiv_name{$fam_husb{$fam_id}}/g; $marr_date = Ged_date($fam_marr_date{$fam_id}); $line =~ s/#marr_date/$marr_date/g;
 if ($full_place) { $line =~ s/#marr_place/$fam_marr_plac{$fam_id}/g; } else { $line =~ s/#marr_place/$fam_marr_city{$fam_id}/g; } if ($sosaw) {  $ind_dabo = '';  if (!$indiv_famc{$fam_wife{$fam_id}}) {  $line =~ s/#wclass/dabo/g;  if ($branch_id{$indiv_branch{$fam_wife{$fam_id}}}) { $branch_link = "javascript:descLoad('".$branch_id{$indiv_branch{$fam_wife{$fam_id}}}."','')";  $ind_dabo = '<br><a class=main1 href="'.$branch_link."\"><img src=\"$pics_path/i_d.gif\" align=\"middle\" border=0>".'</a>'; } }
 $line =~ s/#wife_sosa/\<a name\=\'$sosaw\'\>$sosaw\<\/a>$ind_dabo/g;  } else { $line =~ s/#wife_sosa//ge; } $line =~ s/<!#wife_ref>/$wife_ref/g; $line =~ s/#wife_name/$indiv_name{$fam_wife{$fam_id}}/g; if ($extra_vars{"#husb_occu"}) { $line =~ s/#husb_occu/$husb_occu/g; } if ($extra_vars{"#wife_occu"}) { $line =~ s/#wife_occu/$wife_occu/g; } if ($extra_vars{"#husb_years"}) { $line =~ s/#husb_years/$husb_years/g; } if ($extra_vars{"#wife_years"}) { $line =~ s/#wife_years/$wife_years/g; } &cond_print("$line"); }
 $free_cnt--; } $sosaload{$sosa_max} = "$file_id.html"; &cond_out("$lang_dir/$file_id.html"); &print_skip($lang_dir,$next_file,$first_file); $out_opt = 1; &get_template("geni-tpl",'',('/#gen_cnt/$gen_old/g','/#sosa_cnt/$num_sosas/g')); &log_print('l_cre_gen_x');
 &open_new("$lang_dir/$genx_name.html"); for ( $i = $page_beg ; $i < $page_beg + $page_cnt ; $i++ ) { if (!$vartemp[$i]) { &cond_print("$template[$i]"); next; } $line = $template[$i]; $line =~ s/#root_name/$root_name/g; &cond_print("$line"); } for ( $j = 1 ; $j < $gen_curr  ; $j++ ) { for ( $i = $block_beg ; $i < $block_beg + $block_cnt ; $i++ ) { &cond_print("$template[$i]"); }
 $next_ref = ""; if ( $sosa_cnt[$j] > $genf_size) { $next_page = $gen_ref[$j];  $next_ref = "<sup>"; $next_link = 1; for ( $k = $sosa_cnt[$j] ; $k > $genf_size ; $k-=$genf_size ) { $next_page++;
 $next_ref .=  "<a class=index href=\"$next_page.html\" target=Main>$next_link</a>"; $next_link++; } $next_ref .= "</sup>"; } for ( $i = $item_beg ; $i < $item_beg + $item_cnt ; $i++ ) { if (!$vartemp[$i]) { &cond_print("$template[$i]"); next; } $line = $template[$i]; $line =~ s/#gen_ref/$gen_ref[$j]/g; $line =~ s/#gen_id/$j/g;
 $line =~ s/<!#gen_count>/\($gen_count[$j]\) $next_ref/g; &cond_print("$line"); } for ( $i = $fblock_beg ; $i < $fblock_beg + $fblock_cnt ; $i++ ) { &cond_print("$template[$i]"); } } &cond_out("$lang_dir/$genx_name.html"); } if ( $dsc_opt  ) { $nbrow_span++; $out_opt = 1; &get_template("dsc-tpl",'','');
 &log_print('l_cre_dsc' ); $free_cnt   = 0; $file_id    = $dscf_name; $first_file = $dscf_name; $prev_file  = $dscf_name; $next_file  = $file_id; $curr_file  = ''; $file_span  = 0; $num_sosas  = 0; $new_file   = 1; $new_block = 1; $first_block = 1; $branch_old = 0;
 $branch_curr = 1; $num_lines = 0; $old_ref = '-'; foreach $ind ( sort by_dabo keys %indiv_dabo ) { $branch_curr   = $indiv_branch{$ind}; if (!$branch_id{$indiv_branch{$ind}}) { next; } if ($desc_names[$branch_nnum{$indiv_branch{$ind}}] ne $old_ref) { $new_ref = $desc_names[$branch_nnum{$indiv_branch{$ind}}]; $old_ref = $new_ref; }
 else { $new_ref = ''; } if ( $file_span && ( $free_cnt <= 0 ) ) { $new_file = 1; } if ( $branch_curr ne $branch_old ) { $new_block =1; $file_span = 0; if ($branch_nnum{$branch_curr} ne $branch_nnum{$branch_old}) { if ($desc_by_name or !$curr_file) { $new_file = 1;
 $bnum_ref[$branch_nnum{$branch_curr}] = $next_file.'.html'; } else { $bnum_ref[$branch_nnum{$branch_curr}] = $curr_file.'.html'; } } elsif ( $free_cnt <= 0 ) { $new_file = 1;  } elsif ( $branch_count{$branch_curr} > $free_cnt ) { if ( $branch_count{$branch_curr} <= $dscf_size ) { $new_file = 1; } else { $file_span = 1; } } }
 elsif ( $free_cnt <= 0 ) { $new_file = 1; $file_span = 1;  } if ($new_file) { if ($num_lines) { for ( $i = $fblock_beg ; $i < $fblock_beg + $fblock_cnt ; $i++ ) { &cond_print("$template[$i]"); } &cond_out("$lang_dir/$file_id.html"); $prev_file = $file_id; $file_id++; } &open_new("$lang_dir/$file_id.html");
 $curr_file = $next_file; $next_file++; &print_header; $first_block = 1; if ($file_span) { for ( $i = $block_beg ; $i < $block_beg + $block_cnt ; $i++ ) { if (!$vartemp[$i]) { &cond_print("$template[$i]"); next; } $line = $template[$i];
 $line =~ s/#branch_name/$indiv_name{$branch_head{$branch_curr}} - suite/g; $line =~ s/#block_ref//g; if ($print_desc) { $line =~ s/#rupture/$branch_id{$branch_curr}/g; } else { $line =~ s/#rupture/&nbsp;/g; } if (!$branch_orig{$branch_curr}) { $line =~ s/#branch_orig//g; } else { $line =~ s/#branch_orig/($branch_orig{$branch_curr})/g; } $branch_old = $branch_curr;
 if (defined $descload{$branch_id{$branch_curr}}) { $descload{$branch_id{$branch_curr}} .= ''.$last_ref; } else { $descload{$branch_id{$branch_curr}} = $last_ref; } &cond_print("$line"); } $first_block = 0; } $free_cnt = $dscf_size; $new_file = 0; } if ( $new_block ) {	 $branch_old = $branch_curr; if (!$first_block) { for ( $i = $fblock_beg ; $i < $fblock_beg + $fblock_cnt ; $i++ ) { &cond_print("$template[$i]"); } }
 $branch_ref{$branch_curr} = $file_id; for ( $i = $block_beg ; $i < $block_beg + $block_cnt ; $i++ ) { if (!$vartemp[$i]) { &cond_print("$template[$i]"); next; } $line = $template[$i]; if ($indiv_sosa{$branch_head{$branch_curr}}) { $line =~ s/#iclass/sosa/g; } $line =~ s/#branch_name/$indiv_name{$branch_head{$branch_curr}}/g; $line =~ s/#block_ref/$new_ref/g;
 if (!$branch_orig{$branch_curr}) { $line =~ s/#branch_orig//g; } else { $line =~ s/#branch_orig/($branch_orig{$branch_curr})/g; } if ($print_desc and !$desc_sosas_only) { $line =~ s/#rupture/<a name="$branch_id{$branch_curr}">$branch_id{$branch_curr}<\/a>/g; } elsif ( $print_sosa and $indi_root and $indiv_sosa{$branch_head{$branch_curr}} and ($sosa_gen{$indiv_sosa{$branch_head{$branch_curr}}} <= $gen_number) ) { @ind_sosa = split(',',$indiv_sosa{$branch_head{$branch_curr}}); $branch_link = "javascript:sosaLoad('".$ind_sosa[0]."')";
 $v = "<a class=main1 href=\"".$branch_link."\">$ind_sosa[0]<\/a>";  $line =~ s/#rupture/<a name="$branch_id{$branch_curr}"><\/a>$v/g;  } elsif ($desc_sosas_only) { $line =~ s/#rupture/<a name="$branch_id{$branch_curr}">$branch_id{$branch_curr}<\/a>/g; } else { $line =~ s/#rupture/<a name="$branch_id{$branch_curr}">&nbsp;<\/a>/g; } &cond_print("$line"); }
 $first_block = 0; $new_block = 0; } if ($indiv_file{$ind}) { $ind_filename =  $indiv_file{$ind} . '.html#' . $ind; } else {						   $ind_filename =  $indiv_file{$indiv_rel{$ind}} . '.html#' . $indiv_rel{$ind}; } $desc_ref = $indiv_branch{$ind}.$indiv_dabo{$ind}; if (length($indiv_dabo{$ind}) == 1) { $branch_link = '0'; } else { $branch_link = substr($indiv_dabo{$ind},0,length($indiv_dabo{$ind})-1); } $branch_link = "javascript:descLoad('".$branch_id{$indiv_branch{$ind}}."','".$indiv_branch{$ind}.$branch_link."')";  $dabo_old = $indiv_dabo{$ind}; $ind_dabo = '<a class=main1 href="' . $branch_link . "\">".$indiv_dabo{$ind}.'</a>'; if ($indiv_famn{$ind}) { $ind_dabo .= '<sup>'.$indiv_famn{$ind}.'</sup>'; } $ind_id = $indiv_branch{$ind}.$indiv_dabo{$ind}; $old_marr_year = '';
 @fams = sort { marr_year($a) <=> marr_year($b) } split ( //, $indiv_fams{$ind} ); $fams_len = @fams; $branch_link = ''; if ( ($indiv_sex{$ind} eq 'M') and ($fams_len == 1) ) { if (!$indiv_branch{$fam_wife{$fams[0]}} or !$branch_id{$indiv_branch{$fam_wife{$fams[0]}}}) { $ind_marrs = 'x '.$indiv_surname{$fam_wife{$fams[0]}}.' '.$indiv_forname{$fam_wife{$fams[0]}}; }
 else {  $indirref{$ind_id} = $file_id;  $link_id = $indiv_branch{$fam_wife{$fams[0]}}.$indiv_dabo{$fam_wife{$fams[0]}}; if ($indirref{$link_id}) { $branch_link = $indirref{$link_id}.".html#".$link_id; }                    else { $branch_link = "javascript:descLoad('".$branch_id{$indiv_branch{$fam_wife{$fams[0]}}}."','".$link_id."')"; } $ind_marrs = 'x <a href="'.$branch_link.'">'.$indiv_surname{$fam_wife{$fams[0]}}.' '.$indiv_forname{$fam_wife{$fams[0]}}.'</a>'; } }
 elsif ( $fams_len == 1 ) { if (!$indiv_branch{$fam_husb{$fams[0]}} or !$branch_id{$indiv_branch{$fam_husb{$fams[0]}}}) { $ind_marrs = 'x '.$indiv_surname{$fam_husb{$fams[0]}}.' '.$indiv_forname{$fam_husb{$fams[0]}}; } else {  $indirref{$ind_id} = $file_id;  $link_id = $indiv_branch{$fam_husb{$fams[0]}}.$indiv_dabo{$fam_husb{$fams[0]}}; if ($indirref{$link_id}) { $branch_link = $indirref{$link_id}.".html#".$link_id; }                    else { $branch_link = "javascript:descLoad('".$branch_id{$indiv_branch{$fam_husb{$fams[0]}}}."','".$link_id."')"; } $ind_marrs = 'x <a href="'.$branch_link.'">'.$indiv_surname{$fam_husb{$fams[0]}}.' '.$indiv_forname{$fam_husb{$fams[0]}}.'</a>'; } }
 elsif ( ($indiv_sex{$ind} eq 'M') and ($fams_len > 1 )) { $ind_marrs = ""; for ( $fams_num = 0 ; $fams_num < $fams_len ; $fams_num++ ) { if ($fams_num) { $fnum = $fams_num+1; $fnum = '<sup>'.$fnum.'</sup>'; } else { $fnum = ''; } if (!$indiv_branch{$fam_wife{$fams[$fams_num]}} or !$branch_id{$indiv_branch{$fam_wife{$fams[$fams_num]}}} ) { $ind_marrs .= 'x'.$fnum.' '.$indiv_surname{$fam_wife{$fams[$fams_num]}}.' '; }
 else { $indirref{$ind_id} = $file_id;  $link_id = $indiv_branch{$fam_wife{$fams[$fams_num]}}.$indiv_dabo{$fam_wife{$fams[$fams_num]}}; if ($indirref{$link_id}) { $branch_link = $indirref{$link_id}.".html#".$link_id; }                    else { $branch_link = "javascript:descLoad('".$branch_id{$indiv_branch{$fam_wife{$fams[$fams_num]}}}."','".$link_id."')"; } $ind_marrs .= 'x'.$fnum.' <a href="'.$branch_link.'">'.$indiv_surname{$fam_wife{$fams[$fams_num]}}.'</a> '; } } }
 elsif ( $fams_len > 1 ) { $ind_marrs = ""; for ( $fams_num = 0 ; $fams_num < $fams_len ; $fams_num++ ) { if ($fams_num) { $fnum = $fams_num+1; $fnum = '<sup>'.$fnum.'</sup>'; } else { $fnum = ''; } if (!$indiv_branch{$fam_husb{$fams[$fams_num]}} or !$branch_id{$indiv_branch{$fam_husb{$fams[$fams_num]}}} ) { $ind_marrs .= 'x'.$fnum.' '.$indiv_surname{$fam_husb{$fams[$fams_num]}}.' '; }
 else { $indirref{$ind_id} = $file_id;  $link_id = $indiv_branch{$fam_husb{$fams[$fams_num]}}.$indiv_dabo{$fam_husb{$fams[$fams_num]}}; if ($indirref{$link_id}) { $branch_link = $indirref{$link_id}.".html#".$link_id; }                    else { $branch_link = "javascript:descLoad('".$branch_id{$indiv_branch{$fam_husb{$fams[$fams_num]}}}."','".$link_id."')"; } $ind_marrs .= 'x'.$fnum.' <a href="'.$branch_link.'">'.$indiv_surname{$fam_husb{$fams[$fams_num]}}.'</a> '; } } }
 else { $ind_marrs = ""; } if (($indiv_has_child{$ind} >0) or $branch_link) { $last_ref = $desc_ref; $desc_ref = '<a name="'.$desc_ref.'"></a>'; } else { $desc_ref = ''; } if ($indiv_dabo{$ind}) { for ( $i = 0 ; $i < length($indiv_dabo{$ind}) ; $i++ ) {  if (($i+1)%5) { $desc_ref .= $desc_indent; } else { $desc_ref .= $desc_indent; } } }
 if ($indiv_birt_year{$ind} or $indiv_deat_year{$ind}) { $ind_dates = '('; if ( $indiv_birt_date{$ind} =~ /^\D.*$/ ) { $ind_dates .= '~'; } if ( !$indiv_birt_year{$ind} ) { $ind_dates .= '????'; } else { $ind_dates .= $indiv_birt_year{$ind}; } $ind_dates .= '-';
 if ( $indiv_deat_date{$ind} =~ /^\D.*$/ ) { $ind_dates .= '~'; } if ( !$indiv_deat_year{$ind} ) { $ind_dates .= '????'; } else { $ind_dates .= $indiv_deat_year{$ind}; } $ind_dates .= ')'; } else { $ind_dates = ''; } for ( $i = $item_beg ; $i < $item_beg + $item_cnt ; $i++ ) { if (!$vartemp[$i]) { &cond_print("$template[$i]"); next; } $line = $template[$i];
 $line =~ s/#desc_ref/$desc_ref/ge; if ($indiv_dabo{$ind} eq '0')	{ $line =~ s/#ind_dabo//ge; } else { $line =~ s/#ind_dabo/$ind_dabo/ge; } $line =~ s/#ind_filename/$ind_filename/ge; $line =~ s/#ind_surname/$indiv_surname{$ind}/ge; if ($indiv_sosa{$ind}) {  $line =~ s/#ind_forname/$indiv_forname{$ind}$sosa_link/g; }
 else { $line =~ s/#ind_forname/$indiv_forname{$ind}/ge;	} $line =~ s/#ind_dates/$ind_dates/ge; $line =~ s/#ind_marrs/$ind_marrs/ge; &cond_print("$line"); } $free_cnt--; $num_lines++; } for ( $i = $fblock_beg ; $i < $fblock_beg + $fblock_cnt ; $i++ ) { &cond_print("$template[$i]"); }
 &cond_out("$lang_dir/$file_id.html"); &print_skip($lang_dir,$next_file,$first_file); $out_opt = 1; &get_template("dsci-tpl",'',('/#branch_cnt/$branch_max/g')); &log_print('l_cre_dsc_x'); &open_new("$lang_dir/$genx_name.html"); $free_cnt   = 0; $file_id    = $dscx_name;
 $first_file = $dscx_name; $prev_file  = $dscx_name; $next_file = $dscx_name; $old_bnum = ''; $old_letter = '-'; if ($desc_by_name) { $dscx_size = 0; } foreach $branch ( sort by_name_year keys %branch_id ) { if (!$branch) { next; } $free_cnt--;			  				
 if ( $branch_nnum{$branch} ne $old_bnum)	{ $new_bnum = $branch_nnum{$branch}; $new_letter = substr( noaccent($desc_names[$new_bnum]), 0, 1 );  if ( $free_cnt <= 0 and ((@desc_select <= $dsc_max_index) or $old_letter eq '-'  or (@desc_select > $dsc_max_index and $new_letter ne $old_letter)) ) { if ( $next_file ne $first_file ) { &cond_out("$lang_dir/$file_id.html");
 $prev_file = $file_id; $file_id++; } &open_new("$lang_dir/$file_id.html"); $next_file++; &print_header; $first_block = 1; $free_cnt = $dscx_size; } if (!$first_block) { for ( $i = $fblock_beg ; $i < $fblock_beg + $fblock_cnt ; $i++ ) { &cond_print("$template[$i]"); } }
 $first_block = 0; $dscx_file[$new_bnum] = "$file_id.html"; for ( $i = $block_beg ; $i < $block_beg + $block_cnt ; $i++ ) { if (!$vartemp[$i]) { &cond_print("$template[$i]"); next; } $line = $template[$i]; $rupture = noaccnosp($desc_names[$new_bnum]);
 $line =~ s/#rupture/$rupture/g; $line =~ s/#branch_name/$desc_names[$new_bnum]/g; &cond_print("$line"); } $old_bnum = $new_bnum; $old_letter = $new_letter; } $next_ref = ""; if ( $branch_count{$branch} > $dscf_size) { $next_page = $branch_ref{$branch};  $next_ref = "<sup>";
 $next_link = 1; for ( $k = $branch_count{$branch} ; $k > $dscf_size ; $k-=$dscf_size ) { $next_page++; $next_ref .=  "<a class=index href=\"$next_page.html\" target=Main>$next_link</a>"; $next_link++; } $next_ref .= "</sup>"; }
 $branch_link = $branch_ref{$branch}.".html#".$branch_id{$branch}; for ( $i = $item_beg ; $i < $item_beg + $item_cnt ; $i++ ) { if (!$vartemp[$i]) { &cond_print("$template[$i]"); next; } $line = $template[$i]; $line =~ s/#branch_ref/$branch_link/g;
 if ($indiv_birt_year{$branch_head{$branch}}) { $line =~ s/#branch_year/$indiv_birt_year{$branch_head{$branch}}/g; } else { $line =~ s/#branch_year/____/g; } if ($indiv_sosa{$branch_head{$branch}}) {  $line =~ s/#branch_id/$indiv_forname{$branch_head{$branch}}$sosa_link/g; } else { $line =~ s/#branch_id/$indiv_forname{$branch_head{$branch}}/g; } if ( $branch_count{$branch} > 1 ) { $line =~ s/<!#branch_count>/\($branch_count{$branch}\) $next_ref/g; }
 else { $line =~ s/<!#branch_count>//g; } &cond_print("$line"); } } &cond_out("$lang_dir/$file_id.html"); &print_skip($lang_dir,$next_file,$first_file); } if ( $ind_opt or $loc_opt or $nam_opt or $gen_number ) { $out_opt = 2; if ( $aut_name =~ /^\s*$/ ) { $aut_name = '';  } $i = 0; $nam_list = "";
 foreach $nam_id ( sort { $name_cnt{$b} <=> $name_cnt{$a} or $a cmp $b } keys %name_cnt ) {	 if ($i) { $nam_list .= ", "; } $nam_list .= $nam_id;	 $i++;  if ($i > 19) { $nam_list .= ",.."; last; }	  }	   $i = 0; $loc_list = ""; if ($place_index) { foreach $loc_id ( sort { $placx_count{$b} <=> $placx_count{$a} or $a cmp $b } keys %placx_count ) {	 if ($i) { $loc_list .= ", "; } $loc_list .= $loc_id;	
 $i++;  if ($i > 9) { $loc_list .= ",.."; last; }	  }	 $loc_list =~ s// /g; } $i = 0; $dsc_list = ""; if ($dsc_opt) { foreach $dsc_id ( sort { $a cmp $b } @desc_select ) {	 if ($i) { $dsc_list .= ", "; } $dsc_list .= $dsc_id;	 $i++; 
 if ($i > 9) { $dsc_list .= ",..";  last; }	  }	   } if ( open( TPL_FILE, ($tmpl_fname="$tpl_dir/indi-tpl.html") ) ) { &log_print('l_proc_tmpl'); $i = 0; $tpl_cond = 0;	 while (<TPL_FILE>) { if (/<!!if\s*(\(\S*?\))/) { eval " if (!$1 or $tpl_cond) { \$tpl_cond++; } ";  } elsif (/^<!!endif/) { if ($tpl_cond) { $tpl_cond--; } }  elsif (!$tpl_cond)  {  $template[$i] = $_; $i++;
 $tpl_cnt = $i; } } } else { die "\nCouldn't open template file $tmpl_fname\n"; } close TPL_FILE ; $names_index = ""; $names_fram2 = ""; $names_fram3 = ""; if ($nam_opt) { foreach $letter ( sort { $a cmp $b } keys %names_alpha ) { $names_index .= "<a href=\"$names_alpha{$letter}\">\u$letter</a> "; $names_fram2 .= "<a class=top href=\"$names_alpha{$letter}\" target=Main>\u$letter</a> "; if ($namx_file{$letter}) { $names_fram3 .= "<a class=top href=\"javascript:"
 . "multiLoad('$namx_file{$letter}#$letter','$names_alpha{$letter}')\" " . "onMouseOver=\"window.status='$letter'; return true\" " . "onMouseOut=\"window.status=''; return true\">\u$letter</a> "; } else { $names_fram3 .= "<a class=top href=\"$names_alpha{$letter}\" target=Main>\u$letter</a> "; } } } $places_index = "";
 $places_fram2 = ""; $places_fram3 = ""; if ($loc_opt) { foreach $letter ( sort { $a cmp $b } keys %places_alpha ) { if ( !$locx_file{$letter} ) { $locx_file{$letter} = "$locx_name.html"; } $places_index .= "<a href=\"$places_alpha{$letter}\">\u$letter</a> "; $places_fram3 .= "<a class=top href=\"javascript:"
 . "multiLoad('$locx_file{$letter}#$letter','$places_alpha{$letter}')\" " . "onMouseOver=\"window.status='$letter'; return true\" " . "onMouseOut=\"window.status=''; return true\">\u$letter</a> "; $places_fram2 .="<a class=top href=\"$places_alpha{$letter}\" target=Main>\u$letter</a> "; } } $gen_index = ""; $gen_fram2 = "";
 if ($gen_number) { for ( $j = 1 ; $j < $gen_curr ; $j++ ) { $gen_index .= "<a href=\"$gen_ref[$j].html#G$j\">" . $j . "</a> "; $gen_fram2 .= "<a class=top href=\"$gen_ref[$j].html#G$j\" target=Main>$j</a> "; } } $desc_index = ""; $desc_fram2 = ""; $desc_fram3 = "";
 if ( $dsc_opt ) { $bnum=0; $old_letter = "-"; foreach $desc_name ( sort { $a cmp $b } @desc_select ) { $bnum = $desc_nnum{$desc_name}-1; $desc_name = noaccnosp($desc_name); if ( !$desc_count[$bnum] ) {  next; } $new_letter = substr( $desc_name, 0, 1 ); 
 if ( $new_letter ne $old_letter ) { $old_letter = $new_letter; $desc_index .= "<a href=\"$bnum_ref[$bnum]#$desc_name\">\u$new_letter</a> "; if (@desc_select > $dsc_max_index) { $desc_fram3 .= "<a class=top href=\"javascript:"
 . "multiLoad('$dscx_file[$bnum]#$desc_name','$bnum_ref[$bnum]#$desc_name')\" " . "onMouseOver=\"window.status='$desc_name'; return true\" " . "onMouseOut=\"window.status=''; return true\">\u$new_letter</a> "; $desc_fram2 .="<a class=top href=\"$bnum_ref[$bnum]\" target=Main>\u$new_letter</a> "; } }
 if (@desc_select <= $dsc_max_index) { $desc_fram3 .= "<a class=top href=\"javascript:" . "multiLoad('$dscx_file[$bnum]#$desc_name','$bnum_ref[$bnum]#$desc_name')\" " . "onMouseOver=\"window.status='$desc_name'; return true\" "
 . "onMouseOut=\"window.status=''; return true\">\u$desc_name</a> "; $desc_fram2 .="<a class=top href=\"$bnum_ref[$bnum]\" target=Main>\u$desc_name</a> "; } } } $file_id = $indx_name; &open_new("$lang_dir/$file_id.html"); &log_print('l_cre_top_x'); if ( $aut_file ) { $aut_file = "$gedcom_id.ged";
 $rc = &bincopy( "$file_ged", "$out_dir/$gedcom_id.ged" ); if ($rc==1) {  &log_text("$out_dir/$gedcom_id.ged copied $rc"); $file_outnum++; } } else { $aut_file = ''; } if ( $aut_opt ) { $aut_opt = "$option_id.opt"; $rc = &bincopy( "$file_opt", "$out_dir/$option_id.opt" );
 if ($rc==1) {  &log_text("$out_dir/$option_id.opt copied $rc"); $file_outnum++; } } if ( $aut_email ) { $aut_mail = '<a href="mailto:'.$aut_email.'" TARGET=_blank TITLE="email">';
 $aut_mail =~ tr/\.\]\#\<\>6574839201zZyYxXwWvVuUtTsSrRqQpPoOnNmMlLkKjJiIhHgGfFeEdDcCbBaA/AaBbCcDdEeFfGgHhIiJjKkLlMmNnOoPpQqRrSsTtUuVvWwXxYyZz1029384756><#]./; $aut_mail = "<SCRIPT LANGUAGE='javascript' TYPE='text/javascript'>Contact('".$aut_mail."',"; $aut_mail .= "'<IMG SRC=\"$pics_path/mail.gif\" BORDER=0> email');</SCRIPT>";  } else { $aut_mail =''; } if ( $counter ) { $compteur = $aut_name."[".$aut_email."]";
 $compteur = url_encode($compteur); $compteur = "<IMG SRC=\"http://jfbaquet.free.fr/compteur.php?name=$compteur&amp;inf=$curr_date$version$num_indivs$gen_lang&amp;init=$counter_init\" BORDER=0 ALT='999999'>"; } else { $compteur =''; } $page_title = text_html($base_title);
 $page_header = text_html($main_header); $page_footer = text_html($main_footer); $page_left = text_html($main_left); $page_right = text_html($main_right); for ( $i = 0 ; $i < $tpl_cnt ; $i++ ) { if ( $template[$i] =~ /<!!FILE\s(\S{1,4}).*>/ ) { $file_suff = $1;
 &cond_out("$lang_dir/$file_id.html"); $file_id = $file_pref . $file_suff; &open_new("$lang_dir/$file_id.html"); } else { $template[$i] =~ s/\.\.\/gedi\.css/\.\.\/$css_style.css/; $template[$i] =~ s/\.\.\/Pics\//$pics_path\//g;
 if (index($template[$i],'#') < 0) { &cond_print("$template[$i]"); next; } if ( $main_header ) { $template[$i] =~ s/#main_header/$page_header/g; }	 if ( $main_footer ) { $template[$i] =~ s/#main_footer/$page_footer/g; }	 if ( $main_left ) { $template[$i] =~ s/#main_left/$page_left/g; }	 if ( $main_right ) { $template[$i] =~ s/#main_right/$page_right/g; }	 if ( $aut_name ) {   $template[$i] =~ s/#aut_pic/$aut_pic/g;
 $template[$i] =~ s/#str_author/$str_author/g; $template[$i] =~ s/#aut_name/$aut_name/g; $template[$i] =~ s/#aut_addr1/$aut_addr1/g; $template[$i] =~ s/#aut_addr2/$aut_addr2/g; $template[$i] =~ s/#aut_addr3/$aut_addr3/g; $template[$i] =~ s/#aut_addr4/$aut_addr4/g;
 $template[$i] =~ s/#aut_ctry/$aut_ctry/g; $template[$i] =~ s/#aut_phone/$aut_phone/g; $template[$i] =~ s/#aut_email/$aut_mail/g; $template[$i] =~ s/#aut_file/$aut_file/g; $template[$i] =~ s/#aut_opt/$aut_opt/g; } else { $template[$i] =~ s/#str_author//g; $template[$i] =~ s/#base_title//g;
 $template[$i] =~ s/#aut_name//g; $template[$i] =~ s/#aut_addr1//g; $template[$i] =~ s/#aut_addr2//g; $template[$i] =~ s/#aut_addr3//g; $template[$i] =~ s/#aut_addr4//g; $template[$i] =~ s/#aut_ctry//g; $template[$i] =~ s/#aut_phone//g; $template[$i] =~ s/#aut_email//g;
 $template[$i] =~ s/#aut_file//g; $template[$i] =~ s/#aut_opt//g; } $template[$i] =~ s/#base_title/$page_title/g; $template[$i] =~ s/#nbrow_span/$nbrow_span/g; $template[$i] =~ s/#banner_pic/$banner_pic/g; $template[$i] =~ s/#banner_url/$banner_url/g; $template[$i] =~ s/#flags/$flags/g;
 $template[$i] =~ s/#compteur/$compteur/g; $template[$i] =~ s/#version/V$version/g; $template[$i] =~ s/#website/$website/g; $template[$i] =~ s/#curr_date/$jour\/$mois\/$an/g; $template[$i] =~ s/#curr_hour/$hmin/g; $template[$i] =~ s/#namx_file/$namx_name\.html/g;
 $template[$i] =~ s/#locx_file/$locx_name\.html/g; $template[$i] =~ s/#genx_file/$genx_name\.html/g; $template[$i] =~ s/#dscx_file/$dscx_name\.html/g; $template[$i] =~ s/#csv_file/\.\.\/$flif_name\.csv/g; $template[$i] =~ s/#mli_file/\.\.\/$mlif_name\.csv/g; $template[$i] =~ s/#ind_cnt/$ind_cnt/g;
 $template[$i] =~ s/#loc_cnt/$num_places/g; $template[$i] =~ s/#fam_cnt/$num_names/g; $template[$i] =~ s/#nam_index/$names_index/g; $template[$i] =~ s/#loc_index/$places_index/g; $template[$i] =~ s/#gen_index/$gen_index/g; $template[$i] =~ s/#dsc_index/$desc_index/g;
 $template[$i] =~ s/#nam_fram2/$names_fram2/g; $template[$i] =~ s/#loc_fram2/$places_fram2/g; $template[$i] =~ s/#gen_fram2/$gen_fram2/g; $template[$i] =~ s/#dsc_fram2/$desc_fram2/g; $template[$i] =~ s/#nam_fram3/$names_fram3/g; $template[$i] =~ s/#loc_fram3/$places_fram3/g;
 $template[$i] =~ s/#gen_fram3/$gen_fram2/g; $template[$i] =~ s/#dsc_fram3/$desc_fram3/g; if ($php) { $template[$i] =~ s/#prefs000/uz.php?p=#prefs000/g; } $template[$i] =~ s/#pref/$file_pref/g; $template[$i] =~ s/#main_page/$main_page/g; $template[$i] =~ s/#main_img/$main_img/g;
 $template[$i] =~ s/#home_page/$home_page/g; $template[$i] =~ s/#home_img/$home_img/g; $template[$i] =~ s/#root_name/$root_name/g; $template[$i] =~ s/#nam_list/$nam_list/g; $template[$i] =~ s/#loc_list/$loc_list/g; $template[$i] =~ s/#dsc_list/$dsc_list/g; &cond_print("$template[$i]"); } }
 &cond_out("$lang_dir/$file_id.html"); $last_file  = "index"; $first_file = $indx_name;	 &print_skip($lang_dir,$last_file,$first_file); } print "\n"; $jvs_file =  $lang_dir.'/'.$file_pref.'_load.jvs'; $out_opt = 2; &open_new("$jvs_file"); &cond_print("function descLoad(branch,dabo) {\n"); &cond_print(" var link = dabo + \"                              \";\n"); 
 &cond_print(" link = link.substr(0,30);\n"); &cond_print(" var v = '';\n"); &cond_print(" switch(branch) {\n"); foreach $branch ( sort { $a cmp $b } keys %branch_id ) { if ($branch) { $branch_ref{$branch} =~ /^.*?d(\d+)$/; $v = $1; if (!defined $descload{$branch_id{$branch}}) { &cond_print(" case '$branch_id{$branch}':v='$v';break;\n"); }
 else { &cond_print(" case '$branch_id{$branch}':v='$v';\n"); foreach $k ( split(//,$descload{$branch_id{$branch}}) ) { $k .= '                              '; $k = substr($k,0,30); &cond_print("  if (link > \"$k\") { v++; }\n"); } &cond_print(" break;\n"); } } }
 &cond_print(" default : return;\n");   &cond_print(" }\n");   &cond_print(" v = '0000' + String(v); v = v.substr(v.length-3,3);\n"); if ($php) { $v = 'uz.php?p='; } else {    $v = ''; } &cond_print(" if (dabo == '') { window.location.href = '".$v.$file_pref."d' + v + '.html#' + branch; }\n"); &cond_print(" else { window.location.href = '".$v.$file_pref."d' + v + '.html#' + link; }\n"); &cond_print(" return;\n");
 &cond_print("}\n"); if ($gen_opt) { &cond_print("function sosaLoad(sosanum) {\n"); foreach $sosanum ( sort { $a <=> $b } keys %sosaload ) { if ($sosanum) { if ($php) { &cond_print(" if (sosanum <= $sosanum) { window.location.href = 'uz.php?p=$sosaload{$sosanum}#' + sosanum; return; }\n"); } else { &cond_print(" if (sosanum <= $sosanum) { window.location.href = '$sosaload{$sosanum}#' + sosanum; return; }\n"); } } }
 &cond_print("}\n"); } &cond_out("$jvs_file"); } if ($fli_opt and ((!-e "$out_dir/$flif_name.csv") or !$ind_opt or $file_outnum)) { &log_print('l_cre_flash'); unless ( open( OUT_FILE, ">$out_dir/$flif_name.csv" ) ) { die "\nCouldn't open output file $out_dir/$flif_name.csv\n"; }
 if ($fli_geneanet) { &geneanet_init( $gnetdef_country, $gnetdef_region, $gnetdef_subregion ); if ($fli_sosa_only) { $fli_type = "A"; } else { $fli_type = "L"; } $fli_fq_only = 1; } else { $noacc_place = ""; $placecodes{$noacc_place} = "$delimiter$delimiter"; } $fli_cnt = 0;
 foreach $indiv_id ( sort by_name_place keys %ind_k ) { $new_name  = $indiv_surname{$indiv_id}; $new_place = $indiv_place{$indiv_id}; $new_city = $indiv_city{$indiv_id}; if ( ( $new_name ne $fli_name ) or ( $new_place ne $fli_place ) ) { if ($fli_cnt) { if ( $fli_deb eq "9999" ) { $fli_deb = ""; } if ( $fli_fin eq "0" )    { $fli_fin = $fli_deb; } if ( !$fli_sosa_only or ( $fli_sosa_only && $fli_ref eq "sosa" ) ) { if ( !$fli_fq_only or ( $fli_fq_only && $fli_name && $fli_place ) )	{ if ($fli_geneanet) { $noacc_place = noaccent($fli_place);
 $noacc_place = lc($noacc_place); if ( !defined( $placecodes{$noacc_place} ) ) { ($subregion,$region,$country) =	&place2code($noacc_place); $placecodes{$noacc_place} = "$subregion$delimiter$region$delimiter$country"; } } if (!$place_format) { print OUT_FILE "$fli_name;$fli_ref;$fli_deb;$fli_fin;$fli_cnt;$fli_city;$placecodes{$noacc_place};$fli_type\n"; }
 else { print OUT_FILE "$fli_name;$fli_ref;$fli_deb;$fli_fin;$fli_cnt;$fli_place;$placecodes{$noacc_place};$fli_type\n"; } } } } $fli_name  = $new_name; $fli_place = $new_place; $fli_city = $new_city; $fli_ref   = ""; $fli_deb   = "9999"; $fli_fin   = "0";
 $fli_cnt   = 0; } if ( $indiv_sosa{$indiv_id} ) { $fli_ref = "sosa"; } if ( ( $indiv_birt_year{$indiv_id} ) and ( $indiv_birt_year{$indiv_id} lt $fli_deb ) ) { $fli_deb = $indiv_birt_year{$indiv_id}; }
 if ( ( $indiv_deat_year{$indiv_id} ) and ( $indiv_deat_year{$indiv_id} gt $fli_fin ) ) { $fli_fin = $indiv_deat_year{$indiv_id}; } if ( ( $indiv_deat_year{$indiv_id} ) and ( $indiv_deat_year{$indiv_id} lt $fli_deb ) ) { $fli_deb = $indiv_deat_year{$indiv_id}; }
 if ( ( $indiv_birt_year{$indiv_id} ) and ( $indiv_birt_year{$indiv_id} gt $fli_fin ) ) { $fli_fin = $indiv_birt_year{$indiv_id}; } $fli_cnt++; } close OUT_FILE ; &log_text("$out_dir/$flif_name.csv created"); $file_outnum++; } if ($mli_opt and ((!-e "$out_dir/$mlif_name.csv") or !$ind_opt or $file_outnum)) { &log_print('l_cre_marry');
 unless ( open( OUT_FILE, ">$out_dir/$mlif_name.csv" ) ) { die "\nCouldn't open output file $out_dir/$mlif_name.csv\n"; } if ($mli_jftools) { &geneanet_init( $gnetdef_country, $gnetdef_region, $gnetdef_subregion ); } else { $noacc_place = ""; $placecodes{$noacc_place} = ""; } foreach $fam_id ( keys %fam_k ) { $mli_place = "";
 $mli_city = ""; $mli_year  = ""; $mli_date  = ""; $mli_type  = ""; $mli_husb  = $fam_husb{$fam_id}; $mli_wife  = $fam_wife{$fam_id}; $mli_husb_sound  = SoundEx($indiv_surname{$mli_husb},4,$lang_id); $mli_wife_sound  = SoundEx($indiv_surname{$mli_wife},4,$lang_id); if ( $fam_marr_plac{$fam_id} ) { $mli_place = $fam_marr_plac{$fam_id}; $mli_city = $fam_marr_city{$fam_id}; $mli_type  = "x"; }
 if ( $fam_marr_date{$fam_id} ) { $mli_date = $fam_marr_date{$fam_id}; $mli_year = $fam_marr_year{$fam_id}; } elsif ( $fam_child{$fam_id} ) { @child = split ( //, $fam_chil{ $fams[$fams_num] } ); if ( $indiv_birt_year{ $child[0] } ) { $mli_year -= $indiv_birt_year{ $child[0] }; $mli_date = "circa " . $mli_year;
 if ( $indiv_birt_plac{ $child[0] } ) { $mli_place = $indiv_birt_plac{ $child[0] } . "?"; $mli_city = $indiv_birt_city{ $child[0] } . "?"; $mli_type  = "?"; } } } elsif ( !$mli_year ) { if ( $indiv_birt_year{$mli_wife} ) { $mli_year = $indiv_birt_year{$mli_wife} + 30; $mli_date = "circa " . $mli_year;
 if ( ( !$mli_type ) && $indiv_birt_plac{$mli_wife} ) { $mli_place = $indiv_birt_plac{$mli_wife} . "?"; $mli_city = $indiv_birt_city{$mli_wife} . "?"; $mli_type  = "?"; } } elsif ( $indiv_birt_year{$mli_husb} ) { $mli_year = $indiv_birt_year{$mli_husb} + 30; $mli_date = "circa " . $mli_year;
 if ( ( !$mli_type ) && $indiv_birt_plac{$mli_husb} ) { $mli_place = $indiv_birt_plac{$mli_husb} . "?"; $mli_city = $indiv_birt_city{$mli_husb} . "?"; $mli_type  = "?"; } } } if ( !$mli_sosa_only or ( $indiv_sosa{$mli_husb} or $indiv_sosa{$mli_wife} ) ) { if ( !$mli_fq_only or ( $mli_fq_only && $mli_date && ( $mli_type eq "x" ) ) ) { if ($mli_jftools) { $noacc_place = noaccent($mli_place);
 $noacc_place = lc($noacc_place); if ( !defined( $placecodes{$noacc_place} ) ) { ( $subregion, $region, $country ) = &place2code($noacc_place); $placecodes{$noacc_place} = "$subregion$delimiter$region$delimiter$country"; } } if (!$place_format) { print OUT_FILE "$gedcom_id;$curr_date;$fam_id;$mli_year;$mli_type;$mli_husb_sound;$indiv_surname{$mli_husb};$indiv_forname{$mli_husb};$indiv_famc{$mli_husb};$mli_wife_sound;$indiv_surname{$mli_wife};$indiv_forname{$mli_wife};$indiv_famc{$mli_wife};$mli_date;$mli_city;$placecodes{$noacc_place}\n"; }
 else { print OUT_FILE "$gedcom_id;$curr_date;$fam_id;$mli_year;$mli_type;$mli_husb_sound;$indiv_surname{$mli_husb};$indiv_forname{$mli_husb};$indiv_famc{$mli_husb};$mli_wife_sound;$indiv_surname{$mli_wife};$indiv_forname{$mli_wife};$indiv_famc{$mli_wife};$mli_date;$mli_place;$placecodes{$noacc_place}\n"; } } } }
 close OUT_FILE ; &log_text("$out_dir/$mlif_name.csv created"); $file_outnum++; } if ( $ind_opt or $loc_opt or $nam_opt or $gen_number or $dsc_opt or $asc_opt  ) { &print_index($tablang[0],$indx_name); } &mess_print('m_fil_creat'); &print_result;
 &log_text("-------------------- Gedail ended $num_indivs individuals, $num_families families\n\n$file_outnum files created or updated in $out_dir"); close LOG_FILE ; $duration = time()-$start_time; return($file_outnum,$duration,$out_dir); } sub by_dabo { $desc_names[$branch_nnum{$indiv_branch{$a}}] cmp $desc_names[$branch_nnum{$indiv_branch{$b}}] or $branch_nnum{$indiv_branch{$a}} cmp $branch_nnum{$indiv_branch{$b}} or nameyear($indiv_birt_year{$branch_head{$indiv_branch{$a}}}) <=> nameyear($indiv_birt_year{$branch_head{$indiv_branch{$b}}}) or $branch_id{$indiv_branch{$a}} cmp $branch_id{$indiv_branch{$b}} or $indiv_dabo{$a} cmp $indiv_dabo{$b} }
 sub by_name_year { $desc_names[$branch_nnum{$a}] cmp $desc_names[$branch_nnum{$b}] or $branch_nnum{$a} cmp $branch_nnum{$b} or nameyear($indiv_birt_year{$branch_head{$a}}) <=> nameyear($indiv_birt_year{$branch_head{$b}}) or $branch_id{$a} cmp $branch_id{$b} } sub nameyear { my $year = $_[0];
 if (!$year) { $year = 9999; } return $year; } sub marr_year { my $year = $fam_marr_year{$_[0]}; if (!$year) { $year = $old_marr_year; } else { $old_marr_year = $year; } return $year; } sub birt_year { my $year = $indiv_birt_year{$_[0]}; if (!$year) { $year = $old_birt_year; } else { $old_birt_year = $year; } return $year; }
 sub by_gen_sosa { $sosa_gen{$a} <=> $sosa_gen{$b} or sosa_long($a) cmp sosa_long($b) } sub by_name_place { $indiv_surname{$a} cmp $indiv_surname{$b} or $indiv_place{$a} cmp $indiv_place{$b} or nameyear($indiv_birt_year{$a}) cmp nameyear($indiv_birt_year{$b}) or nameyear($indiv_deat_year{$a}) cmp nameyear($indiv_deat_year{$b}) or $a cmp $b; }
 sub by_place { placeconv( $indiv_placx{$a} ) cmp placeconv( $indiv_placx{$b} ) or placeconv( $indiv_place{$a} ) cmp placeconv( $indiv_place{$b} ) or noaccent( $indiv_surname{$a} ) cmp noaccent( $indiv_surname{$b} ) or $indiv_forname{$a} cmp $indiv_forname{$b} or $indiv_birt_year{$a} cmp $indiv_birt_year{$b} or $a cmp $b; }
 sub by_surname { nameconv( $indiv_surname{$a} ) cmp nameconv( $indiv_surname{$b} ) or $indiv_forname{$a} cmp $indiv_forname{$b} or $indiv_birt_year{$a} cmp $indiv_birt_year{$b} or $a cmp $b; } sub note_add { my @parms   = @_; my $notes = $parms[0]; my $pref = $parms[1];
 my $text = $parms[2]; if ( ($print_notes) and $text !~ /^$priv_note.*$/) { if ($pref eq 'CONT') {  if ( $notes =~ /^(.*?)<\/LI>$/ ) { $notes = $1."<br>".$text."</LI>"; } else { $notes = $notes."<br>".$text; } } elsif ($pref eq 'CONC') {  if ( $notes =~ /^(.*?)<\/LI>$/ ) { $notes = $1.$text."</LI>"; }
 else { $notes = $notes.$text; } } else { if ($pref) { $pref = "<b>$pref :</b> " } if ( !$notes ) { $notes = $pref.$text } else  {  if ($notes !~ /<LI>/) {  $notes = "<LI>$notes</LI><LI>".$pref.$text."</LI>";  } else { $notes .= "<LI>".$pref.$text."</LI>"; } } } } return $notes; }
 sub event_add { my @parms   = @_; my $events = $parms[0]; my $tag = $parms[1]; my $text = $parms[2]; $text =~ tr//  /; if ($tag eq 'TYPE') {  $events =~ s/^(.*?)(.*?)(.*?)(.*)$/$1$2$text$4/;  } elsif ($tag eq 'DATE') { my $event_year = Ged_year($text);
 $events =~ s/^(.*?)(.*?)(.*?)(.*?)(.*)$/$event_year$2$3$text$5/;  } elsif ($tag eq 'PLAC') { my ($plac,$city,$dummy) = Ged_place($text); $events =~ s/^(.*?)(.*?)(.*?)(.*?)(.*?)(.*)$/$1$2$3$4$plac$6/;  } elsif ($tag eq 'ASSO') { $events =~ s/^(.*?)(.*?)(.*?)(.*?)(.*?)(.*?)(.*)$/$1$2$3$4$5$6$text$7/;  }
 elsif (($tag =~ /(NOTE|CONC|CONT)/) and !$print_notes_2 and $level == 2) { } elsif ($tag eq 'NOTE' and ($text !~ /^($priv_note|@).*$/)) {				 $events =~ s/^(.*?)(.*?)(.*?)(.*?)(.*?)(.*?)(.*?)(.*?)(.*)?$/$1$2$3$4$5$6$7$text$9/;  }				   elsif ($tag eq 'CONT') { $events =~ s/^(.*?)(.*?)(.*?)(.*?)(.*?)(.*?)(.*?)(.*?)(.*)?$/$1$2$3$4$5$6$7$8<br>$text$9/;  }
 elsif ($tag eq 'CONC') { $events =~ s/^(.*?)(.*?)(.*?)(.*?)(.*?)(.*?)(.*?)(.*?)(.*)?$/$1$2$3$4$5$6$7$8$text$9/;  } elsif ($tag eq '_CONT') { $events =~ s/^(.*?)(.*?)(.*?)(.*?)(.*?)(.*?)(.*?)(.*?)(.*)?$/$1$2$3$4$5$6$7<br>$text$8$9/;  }
 elsif ($tag eq '_CONC') { $events =~ s/^(.*?)(.*?)(.*?)(.*?)(.*?)(.*?)(.*?)(.*?)(.*)?$/$1$2$3$4$5$6$7$text$8$9/;  }	 else { if ($events) { $events = ' '.$tag.''.$text.''.$events; } else { $events = ' '.$tag.''.$text.'' } } return $events;  }
 sub events_clear { my @events_list = split(//,$_[0]); my $items = ''; foreach my $item ( sort { lc($a) cmp lc($b) } @events_list  ) { if ($item =~ /^.*?(BIRT|DEAT|MARR).*?.*?Y?$/) { $item = ''; } if ($items) { if ($item) { $items .= ''.$item; } } else { $items = $item; } }
 return $items; } sub events_edit { my $list = $_[0]; my $class = $_[1]; my @events_list = split(//,$list); my $events = ''; my $events_count = @events_list; if ($events_count == 0 ) { return ''; } elsif ($events_count == 1) {  my ($year,$tag,$type,$date,$plac,$asso,$text,$note) = split(//,$events_list[0]); 
 if ($date) { $date = Ged_date($date); $events = "<b>".$date."</b> ";  } $events .= $str_events{$tag}; if ($type) { $events .= "[".$type."]"; } $events .= " ".$text; if ($plac) { $events .= " <i>(".$plac.")</i>"; } if ($note) { $events .= " [<i>".$note."</i>]"; } if ($asso and $str_events{'ASSO'}) { $i = ''; while ($asso =~ /@(.+?)@/) {  $temoin = "<a class=\"main1\" href=\"$indiv_file{$1}.html#$1\">$indiv_name{$1}</a>"; $asso =~ s/@(.+?)@/$i$temoin/; 
 $i = ', '; } $events .= "<b>&nbsp;&nbsp;".$str_events{'ASSO'}." : </b>".$asso;  } $events .= "<br>"; } else { $events = "<UL class=$class>"; my $event_old = '';	my $event_new = ''; foreach $event ( @events_list ) { $event_new = majuscule($event);
 if ($event_new != '$event_old') { $event_old = $event_new; $events .= "<LI>"; my ($year,$tag,$type,$date,$plac,$asso,$text,$note) = split(//,$event);  if ($date) {  $date = Ged_date($date); $events .= "<b>".$date."</b> ";  } $events .= $str_events{$tag}; if ($type) {  if ($str_event_type{$type}) { $events .= "[".$str_event_type{$type}."]"; } else { $events .= "[".$type."]"; } }
 $events .= " ".$text; if ($plac) { $events .= " <i>(".$plac.")</i>"; } if ($note) { $events .= " <i>".$note."</i>"; } if ($asso and $str_events{'ASSO'}) { $i = ''; while ($asso =~ /@(.+?)@/) { $temoin = "<a class=\"main1\" href=\"$indiv_file{$1}.html#$1\">$indiv_name{$1}</a>"; $asso =~ s/@(.+?)@/$i$temoin/; 
 $i = ', '; } $events .= "<b>&nbsp;&nbsp;".$str_events{'ASSO'}." : </b>".$asso;  } $events .= "</LI>"; } } $events .= "</UL>"; } return $events; } sub indinum { my @parms   = @_; my $indinum = $parms[0]; $indinum =~ s/^I?(\d+)I?$/$1/; return $indinum; }
 sub noaccent { my @parms = @_; for (@parms) { tr//aaceeeeiiouuaAACEEEEIIOUUA/ } return wantarray ? @parms : $parms[0]; } sub placeconv { my @parms = @_; for (@parms) {  tr//aaceeeeiiouuaAACEEEEIIOUUA/; s/^\s*$/zzzz/gi; }
 return wantarray ? @parms : $parms[0]; } sub noaccnosp { my @parms = @_; for (@parms) { tr//aaceeeeiiouuaAACEEEEIIOUUA/; s/\s//g; s/\'//g; } return wantarray ? @parms : $parms[0]; } sub nameconv { my @parms = @_; for (@parms) { tr//aaceeeeiiouuaAACEEEEIIOUUA/;
 s/\s//g; s/\'//g; s/^\W(.*)$/ZZZ$1/g; } return wantarray ? @parms : $parms[0]; } sub majuscule { my @parms = @_; for (@parms) { tr/a-z;/A-Zܟ /; } return wantarray ? @parms : $parms[0]; } sub nompropre { my $parm = shift;
 $parm =~ tr/A-Zܟ/a-z/; my @words = split ( /\s/, $parm ); my $result = ''; my $i = 0; while ($words[$i]) {			 $result .= " ".ucfirst($words[$i]); $i++; } return $result; } sub html { my @parms = @_; for (@parms) { s//&Agrave;/g; s//&Acirc;/g; s//&Auml;/g; s//&Ccedil;/g;
 s//&Egrave;/g; s//&Eacute;/g; s//&Ecirc;/g; s//&Euml;/g; s//&Igrave;/g; s//&Icirc;/g; s//&Iuml;/g; s//&Ograve;/g; s//&Ocirc;/g; s//&Ouml;/g; s//&Ugrave;/g; s//&Ucirc;/g; s//&Uuml;/g; s//&Yuml;/g; s//&AElig;/g; s//&agrave;/g; s//&acirc;/g; s//&auml;/g;
 s//&ccedil;/g; s//&egrave;/g; s//&eacute;/g; s//&ecirc;/g; s//&euml;/g; s//&igrave;/g; s//&icirc;/g; s//&iuml;/g; s//&ograve;/g; s//&ocirc;/g; s//&ouml;/g; s//&ugrave;/g; s//&ucirc;/g; s//&uuml;/g; s//&yuml;/g; s//&aelig;/g; s//&ntilde;/g;
 s//&Ntilde;/g; s//&deg;/g; } return wantarray ? @parms : $parms[0]; } sub text_html { my $text = shift; $text = html($text); $text =~ s/\s\s/ &nbsp;/g; $text =~ s/\\n/<br>/g; $text =~ s/\\'/&quote;/g; $text =~ s/<!--$lang_id\///g; $text =~ s/\/$lang_id-->//g;
 $text =~ s/\s(http:\/\/\S*)\s/ <A HREF='$1' TARGET=_blank>$1<\/A> /gi; return $text; } sub get_template { my ($templa,$extra,@repla) = @_; @extra_vars = ''; if ( open( TPL_FILE, ($tmpl_fname="$tpl_dir/$templa.html" ) ) ) { &log_print('l_proc_tmpl'); $i          = 0; $page_beg   = 1; $block_beg  = 0; $item_beg   = 0;
 $fblock_beg = 0; $fpage_beg  = 0; $tpl_cond = 0;	 while (<TPL_FILE>) {  if (/<!!if\s*(\(\S*?\))/) {  eval " if (!$1) { \$tpl_cond++; } "; }   elsif (/^<!!endif/)  {  if ($tpl_cond) { $tpl_cond--; } } elsif ($tpl_cond)  { } elsif (/^<!!page/) { $page_beg = 0; $j = 0; } elsif (/^<!!block/)  { $page_cnt  = $j; $block_beg  = $i; $j = 0; } elsif (/^<!!item/)   { $block_cnt = $j; $item_beg   = $i; $j = 0; } elsif (/^<!!eblock/) { $item_cnt  = $j; $fblock_beg = $i; $j = 0; } elsif (/^<!!epage/)  { $fblock_cnt = $j; $fpage_beg  = $i; $j = 0; } elsif ( $page_beg == 0 ) { $template[$i] = $_; $j++; $i++; } if ($extra) { if ($_ =~ /($extra)/) { $extra_vars{$1} = 1; }  } }
 $fpage_cnt = $j; if ( ( $page_beg != 0 ) or ( $block_beg == 0 ) or ( $item_beg == 0 ) or ( $fblock_beg == 0 ) or ( $fpage_beg == 0 ) ) { die "\nIncorrect template file $tpl_dir/$templa.html\n"; } $tpl_cnt = $page_cnt + $block_cnt + $item_cnt + $fblock_cnt + $fpage_cnt; }
 else { die "\nCouldn't open template file $tpl_dir/$templa.html\n"; } close TPL_FILE ; for ( $i = 0 ; $i < $tpl_cnt ; $i++ ) { $template[$i] =~ s/#version/V$version/g; $template[$i] =~ s/#website/$website/g; $template[$i] =~ s/#curr_date/$jour\/$mois\/$an/g; $template[$i] =~ s/#curr_hour/$hmin/g;
 $template[$i] =~ s/#main_page/$main_page/g; $template[$i] =~ s/#main_img/$main_img/g; $template[$i] =~ s/#home_page/$home_page/g; $template[$i] =~ s/#home_img/$home_img/g; $template[$i] =~ s/#pref/$file_pref/g;		 $template[$i] =~ s/#namx_file/$namx_name\.html/g;
 $template[$i] =~ s/#locx_file/$locx_name\.html/g; $template[$i] =~ s/#genx_file/$genx_name\.html/g; $template[$i] =~ s/#dscx_file/$dscx_name\.html/g; $template[$i] =~ s/\.\.\/Pics\//$pics_path\//g;	 $template[$i] =~ s/\.\.\/gedi\.css/\.\.\/$css_style.css/; 
 foreach $sed (@repla) { eval("\$template[$i] =~ s$sed;"); } $vartemp[$i] = index($template[$i],'#')+1; } } sub print_header { my $i=0; $file_curr =~ /^.*?([^\/\\]*)\.html$/; $curr_file = $1; for ( $i = $page_beg ; $i < $page_beg + $page_cnt ; $i++ ) { if (!$vartemp[$i]) { &cond_print("$template[$i]"); next; } my $line = $template[$i];
 $line =~ s/#prev_file/$prev_file\.html/g; $line =~ s/#next_file/$next_file\.html/g; $line =~ s/#curr_file/$curr_file\.html/g; &cond_print("$line"); } } sub print_skip { my @parms = @_; $out_opt = 2; &open_new("$parms[0]/$parms[1].html");  if ($parms[1] ne $parms[2]) {	   		 &cond_print("<HTML><HEAD><script language=\"javascript\" TYPE=\"text/javascript\">\n");
 &cond_print("<!-- \n"); &cond_print("function skip(page) { document.location.href=page; }\n"); &cond_print("//-->\n"); if ($php) {	&cond_print("</script></HEAD><BODY onload=\"skip('uz.php?p=$parms[2].html')\";></BODY></HTML>"); } else { &cond_print("</script></HEAD><BODY onload=\"skip('$parms[2].html')\";></BODY></HTML>"); } }
 else { 	&cond_print("<HTML><HEAD></HEAD><BODY>THIS PAGE IS EMPTY</BODY></HTML>"); } &cond_out("$parms[0]/$parms[1].html"); return; } sub print_index { my @parms = @_; my $skip_url = $parms[1].'.html'; if ($php) {	$skip_url = $parms[0].'/'.'uz.php?p='.$skip_url; } else { $skip_url = $parms[0].'/'.$skip_url; } unless ( open( OUT_FILE, ">$out_dir/$parms[1].html" ) ) { die "\nCouldn't open output file $out_dir/$parms[1].html\n"; }
 print OUT_FILE "<HTML><HEAD><script language=\"javascript\" TYPE=\"text/javascript\">\n<!-- \n"; print OUT_FILE "function skip(page) { document.location.href=page; }\n//-->\n</script></HEAD><BODY onload=\"skip('$skip_url')\";></BODY></HTML>"; close OUT_FILE ; $out_opt = 2; &open_new("$out_dir/index.html");
 &cond_print("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\" \"http://www.w3.org/TR/html4/loose.dtd\">\n");	    &cond_print("<HTML><HEAD>\n");																	    &cond_print("<meta http-equiv=\"Content-Type\" content=\"text/html; charset=iso-8859-1\">\n");	    &cond_print("<meta name=\"GENERATOR\" content=\"Gedail V$version\">\n");							   
 &cond_print("<meta name=\"AUTHOR\" content=\"J-F BAQUET\">\n");						    &cond_print("<meta name=\"KEYWORDS\" content=\"Gnalogie,Genealogy,$nam_list Gedail\">\n");			   &cond_print("<meta http-equiv=\"Refresh\" content=\"1; URL=$skip_url\">\n");			    &cond_print("<LINK REL=\"SHORTCUT ICON\" HREF=\"favicon.ico\">\n"); 
 &cond_print("<TITLE>Visitez le site de l'auteur http://$website</TITLE></HEAD>\n");  &cond_print("<BODY BACKGROUND=\"$pics_path/Baqbg.jpg\"><br>&nbsp;<br>\n");	 &cond_print("<P ALIGN=\"CENTER\"><FONT size=\"+1\">Gnalogie&nbsp;$nam_list,...</FONT></P>\n");	  &cond_print("<br>&nbsp;<br><FONT size=\"+2\"><CENTER>&nbsp;\n");											
 &cond_print("<P>Site g&eacute;n&eacute;r&eacute; par Gedail ( Ged Automatized Information Linkage )</P>\n");  &cond_print("<P>r&eacute;alis&eacute; by JEAN-FRAN&Ccedil;OIS BAQUET</P>\n"); &cond_print("<P><a href=\"http://$website/outils\">http://$website</a></P>\n"); &cond_print("<P><a href=\"$skip_url\">next ....</a></P>\n"); 
 &cond_print("</FONT></CENTER></BODY></HTML>\n"); &cond_out("$out_dir/index.html"); return; } sub open_new {                                                   $file_curr = shift; $file_num = 0;                                               $file_out = 0; $file_rownum = 0; 
 if ($gzip and $file_curr =~ /^.*?$file_pref[^\/\\]*\.html$/) { $fz = $file_curr.'.gz'; $old_file = IO::Zlib->new($fz, "rb"); if (!defined $old_file) { $file_out = 1; } } else { unless (open(OLD_FILE, "$file_curr")) {             $file_out = 1; } } } sub cond_print { my $line = shift; my $line_in = '';
 if ($line !~ /^\s*$/) { if ($ind_ok) { $line =~ s/<TD([^>]*)>\s*<\/TD>/<TD$1>&nbsp;<\/TD>/gi; } $line =~ s/<TD><\/a>&nbsp;<\/TD>/<TD>&nbsp;<\/TD>/gi; $line =~ s/<b>\s*<\/b>//gi; $line =~ s/<i>\s*<\/i>//gi; $line =~ s/<SPAN\s*CLASS=[^>]*>\s*?<\/SPAN>//gi;
 $line =~ s/<A NAME=\"\"><\/A>//gi; $line =~ s/<center><\/center>//gi; if ($out_opt) { $file_row[$file_rownum] = html($line); if ($gzip  and $file_curr =~ /^.*?$file_pref[^\/\\]*\.html$/) { $line_in = <$old_file>; } else { $line_in = <OLD_FILE>; } if (!(($line =~ /^\s*<!-- let it be -->.*/) and ($line_in =~ /^\s*<!-- let it be -->.*/))) { if ( (!$file_out) and ( $file_row[$file_rownum] ne $line_in ) ) { $file_out = 2; } }
 $file_rownum++; } } } sub cond_out { my @parms = @_; my $j = 0; my $temp = ''; my $line = ''; if ($out_opt) { if ($out_opt == 1) {  for ( $j = $fpage_beg ; $j < $fpage_beg + $fpage_cnt ; $j++ ) { if (!$vartemp[$j]) { &cond_print("$template[$j]"); next; } $line = $template[$j];
 $line =~ s/#prev_file/$prev_file\.html/g; $line =~ s/#next_file/$next_file\.html/g; &cond_print("$line"); } } if ($gzip and $file_curr =~ /^.*?$file_pref[^\/\\]*\.html$/) { $line = <$old_file>; } else { $line = <OLD_FILE>; } if (($line) && (!$file_out)) {$file_out = 3; } if ($gzip and $file_curr =~ /^.*?$file_pref[^\/\\]*\.html$/) { undef $old_file; } else { close OLD_FILE; }        if ($file_out) { if ($gzip  and $file_curr =~ /^.*?$file_pref[^\/\\]*\.html$/) { my $fz = $parms[0].'.gz'; 
 $out_file = IO::Zlib->new($fz, "wb"); if (!defined $out_file) { die "\nCouldn't open output file $fz\n"; } for ( $j = 0 ; $j < $file_rownum ; $j++ ) { $temp .= $file_row[$j]; } print $out_file $temp; $out_file->close; } else { unless ( open( OUT_FILE, ">$parms[0]" ) ) { die "\nCouldn't open output file $parms[0]\n"; }
 for ( $j = 0 ; $j < $file_rownum ; $j++ ) { print OUT_FILE $file_row[$j]; } close OUT_FILE ; } if ( $file_out == 1 ) { &log_text("$parms[0] created"); } if ( $file_out >= 2 ) { &log_text("$parms[0] updated $file_out"); } $file_outnum++; } } return $file_out; } sub bincopy { my @parms     = @_;
 my $closefrom = 0; my $closeto   = 0; my ( $size, $status, $r, $buf ); my @file_in = stat("$parms[0]"); my @file_out = stat("$parms[1]"); if (($file_in[7] eq $file_out[7]) and ($file_in[9] eq $file_out[9])) { return 2; } open( FROM, "< $parms[0]" ) or goto fail_open1;
 binmode FROM or die "($!,$^E)"; $closefrom = 1; open( TO, "> $parms[1]" ) or goto fail_open2; binmode TO or die "($!,$^E)"; $closeto = 1; $size    = -s FROM; $size    = 1024 if ( $size < 512 ); $Too_Big = 1024 * 1024 * 2; $size    = $Too_Big if ( $size > $Too_Big ); $!       = 0;
 for ( ; ; ) { my ( $r, $w, $t ); defined( $r = sysread( FROM, $buf, $size ) ) or goto fail_inner; last unless $r; for ( $w = 0 ; $w < $r ; $w += $t ) { $t = syswrite( TO, $buf, $r - $w, $w ) or goto fail_inner; } } close TO    || goto fail_open2 if $closeto;
 close FROM  || goto fail_open1 if $closefrom; $rc = utime($file_in[8],$file_in[9],$parms[1]); return 1; fail_inner: if ($closeto) { $status = $!; $! = 0; close TO ; $! = $status unless $!; } fail_open2: if ($closefrom) { $status = $!; $!      = 0; close FROM ; $! = $status unless $!; }
 fail_open1: return 0; } sub place_link_init { my $count = 0; my @placefiles = ''; if ($place_links or ($place_links_files[0] eq '*')) { opendir CURRENTREP, "." or die "No current directory: $!"; @placefiles = grep /^placelink-.*?\.csv$/i, readdir CURRENTREP;
 closedir CURRENTREP; } elsif (@place_links_files) { $i=0; foreach $placefile (@place_links_files) { $placefiles[$i] = 'placelink-'.$placefile.'.csv'; $i++; } } foreach $placefile ( sort { $a cmp $b } @placefiles) { if ( open( SOURCE, $placefile ) ) { &log_text("--- $placefile");
 while (<SOURCE>) { chomp; my ($place,$link) = split (/;/); $place =~ s/^\s*(.*?)\s*$/$1/; $place =~ s/\s{2,}/ /g; $place =~ s/-/ /g; $place = lc($place); $place = noaccent($place); $place_link{$place} = $link; $count++; } close SOURCE ; } }
 &log_text("--- places links : $count"); return $count; } sub geneanet_init { my @parms = @_; if ( !$codes_init ) { require Geocodes; $defaultcountry      = $parms[0]; $defaultregion       = $parms[1]; $defaultsubregion    = $parms[2];; $delimiter           = ";"; $placelenghtmax      = 80;
 opendir CURRENTREP, "." or die "No current directory: $!"; my @listkeycodesfile = grep /^codes-(\w\w\w)\.csv$/i, readdir CURRENTREP; closedir CURRENTREP; foreach $keycodesfile (@listkeycodesfile) { if ( $keycodesfile =~ m/^codes-(\w\w\w)\.csv$/i ) { my $countrycode = $1; $countrycode =~ tr/a-z/A-Z/;
 unless ( open( SOURCE, $keycodesfile ) ) { die "\nCouldn't open geneanet codes file $keycodesfile\n"; } &log_print('l_proc_code'); my @oldplace = ""; while (<SOURCE>) { chomp; my ( $place, $postalcode, $subregion, $region ) = split (/;/); $newplace = $place; if ( $place eq $oldplace[0] ) { $newplace = "$place $postalcode";
 if ( $countrycode eq $defaultcountry ) { if ( $region eq $defaultregion ) { if ( !$defaultsubregion || ( $subregion eq $defaultsubregion ) ) { $newplace = "$place $oldplace[1]"; $Geocodes::regioncode{$countrycode} {$newplace} = [ $oldplace[2], $oldplace[3], $countrycode ]; $newplace = $place; } } } }
 else { @oldplace = ( $place, $postalcode, $subregion, $region ); } $Geocodes::regioncode{$countrycode}{$newplace} = [ $subregion, $region, $countrycode ]; } close SOURCE ; } } } $codes_init++; } sub place2code { my @parms  = @_; my $place1 = $parms[0];
 my ( $place2, $departement, $subregion, $region, $country ) = (); $place1 =~ s/\s\s/ /g; $place1 =~ s/\s,/,/g; $place1 =~ s/,,/,/g; $place1 =~ s/,$//g; $place1 =~ s/^,//g; $place1 =~ s/$delimiter|;/,/g; $place1 =~ s/\s\?//g; $place1 =~ s/(\?|\|)//g; $place1 =~ s/^\s*//g; $place1 =~ s/\s*$//g; $place1 =~ s/,+$//g;
 $place1 =~ s/^of,?\s//i; $place1 =~ s/^probably\s//i; $place1 =~ s/.*(cemetery|cemetary)(.*)/$2/i; $place1 =~ s/^saint\s(.*)/st $1/i; $place1 =~ s/^sainte\s(.*)/ste $1/i; if ($place1 && ( length($place1) > $placelenghtmax
 || $place1 =~ m/[a-z]\s\b1[6-9]\d\d\b|\b\d\d ans\b|avant \d+|\bf\d+|\d\D\d|bataille de|\bdied when|children of|sister of|brother of|mother of|\bdaughter\b|^buried\s|died in|\d years|\bage of|\bdate of|\bage \d+|chart \d+|see notes|\bcem\.$|partage le|civil war|died at|vers 1\d\d\d|suicide|executed|non dclar|unknown|^unk$|lieu inconnu|no children|unmarried|not married|at sea/io || $place1 =~ m/^(ca|av|ap)$/o || $place1 !~ m/[A-Za-z0-9]/o ) ) { $place1 = ''; } $place1 =~ s/^\s*//; $place1 =~ s/\s*$//; if($place1) { $place1 =~ m/.*[,( ]?([A-Z])[-]?[0-9]+/; $place1 .= ",$1";
 $place1 =~ tr/ABCDEFGHIJKLMNOPQRSTUVWXYZ\300\301\302\303\304\305\307\310\311\312\313\314\315\316\317\321\322\323\324\325\326\331\332\333\334\335\340\341\342\343\344\345\347\350\351\352\353\354\355\356\357\361\362\363\364\365\366\371\372\373\374\375\377/abcdefghijklmnopqrstuvwxyzaaaaaaceeeeiiiinooooouuuuyaaaaaaceeeeiiiinooooouuuuyy/; $place1 =~ tr/,()a-z0-9/ /cs; $place1 =~ s/^\s*//; $place1 =~ s/\s*$//; my @mots1=split(/\s*[,()]\s*/,$place1); my @mots1b=@mots1; my $mot1last=pop @mots1b; my @mots2=split(/\s*[,() ]\s*/,$place1); my @mots2b=@mots2; my $mot2last=pop @mots2b; my $mot;
 *localregioncode=$Geocodes::regioncode{$defaultcountry} || {}; my $coderegion=$localregioncode{$mot1last} || $localregioncode{$mot2last}; if($coderegion) { ($subregion,$region,$country)=@$coderegion; } else { my $codecountry=$Geocodes::countrycodes1{$mot1last} || $Geocodes::countrycodes2{$mot1last} || $Geocodes::countrycodes2{$mot2last}; my $code;
 if(($defaultcountry eq 'FRA' && !$codecountry) || $codecountry eq 'FRA') { my $string = $place1; while( ($n) = ($string =~ /([0-9]+)/) ) { $string =~ s/[, (]*[0-9]+[) ,]*//; if(length($n) == 5) { $departement = substr($n,0,2); } elsif(length($n) == 2) { if($string =~ /(paris|marseille|lyon)/i) { $departement = '75' if ($string =~ /paris/i);
 $departement = '13' if ($string =~ /marseille/i); $departement = '69' if ($string =~ /lyon/i); } else { $departement = $n; } } } $departement = "F$departement"; $my_region = $Geocodes::subregion2regionFRA{$departement}; return ($departement, $my_region, 'FRA') if $my_region; } if($codecountry) { *localregioncode2=$Geocodes::regioncode{$codecountry} || {}; while(defined($mot = pop @mots1b)) { $code=$localregioncode2{$mot}; if($code && ($$code[2] eq $codecountry)) { ($subregion,$region,$country)=@$code;
 last; } $code=$localregioncode{$mot}; if($code && ($$code[2] eq $codecountry)) { ($subregion,$region,$country)=@$code; last; } $code=$Geocodes::regioncodes1{$mot}; if($code && ($$code[2] eq $codecountry)) { ($subregion,$region,$country)=@$code; last; }
 $code=$Geocodes::regioncodes2{$mot}; if($code && ($$code[2] eq $codecountry)) { ($subregion,$region,$country)=@$code; last; } $code=$Geocodes::regioncodespe1{$mot} || $Geocodes::regioncodespe2{$mot}; if($code && ($$code[2] eq $codecountry)) { $coderegion=$$code[1]; } } if (!$country) { while(defined($mot = pop @mots2b)) { $code=$localregioncode2{$mot}; if($code && ($$code[2] eq $codecountry)) { ($subregion,$region,$country)=@$code;
 last; } $code=$localregioncode{$mot}; if($code && ($$code[2] eq $codecountry)) { ($subregion,$region,$country)=@$code; last; } $code=$Geocodes::regioncodes2{$mot}; if($code && ($$code[2] eq $codecountry)) { ($subregion,$region,$country)=@$code; last; }
 $code=$Geocodes::regioncodespe2{$mot}; if($code && ($$code[2] eq $codecountry)) { $coderegion=$$code[1]; } } ($subregion,$region,$country) = ('', $coderegion, $codecountry) if (!$country); } } else { my $coderegion = ''; my $codecountry = ''; *localregioncode2={}; while(defined($mot = pop @mots1)) { $code=$localregioncode2{$mot}; if(defined $code) { ($subregion,$region,$country)=@$code;
 last; } $code=$localregioncode{$mot}; if(defined $code) { ($subregion,$region,$country)=@$code; last; } $code=$Geocodes::regioncodes1{$mot}; if(defined $code) { ($subregion,$region,$country)=@$code; last; } $code=$Geocodes::regioncodes2{$mot}; if(defined $code) { ($subregion,$region,$country)=@$code;
 last; } $code=$Geocodes::regioncodespe1{$mot} || $Geocodes::regioncodespe2{$mot}; if($code && ($$code[2] eq $codecountry)) { $coderegion=$$code[1]; $codecountry=$$code[2]; *localregioncode2=$Geocodes::regioncode{$codecountry} || {}; }
 if(!$codecountry && !$coderegion) { $codecountry=$Geocodes::countrycodes1{$mot} || $Geocodes::countrycodes2{$mot} || ''; *localregioncode2=$Geocodes::regioncode{$codecountry} || {}; } } if(!$country) { while(defined($mot = pop @mots2)) { $code=$localregioncode2{$mot}; if(defined $code) { ($subregion,$region,$country)=@$code;
 last; } $code=$localregioncode{$mot}; if(defined $code) { ($subregion,$region,$country)=@$code; last; } $code=$Geocodes::regioncodes2{$mot}; if(defined $code) { ($subregion,$region,$country)=@$code; last; } $code=$Geocodes::regioncodespe2{$mot}; if($code && !$coderegion) { $coderegion=$$code[1];
 $codecountry=$$code[2]; *localregioncode2=$Geocodes::regioncode{$codecountry} || {}; } if(!$codecountry && !$coderegion) { $codecountry=$Geocodes::countrycodes2{$mot} || ''; *localregioncode2=$Geocodes::regioncode{$codecountry} || {}; } }
 ($subregion,$region,$country) = ('', $coderegion, $codecountry) if (!$country); } } } } return ($subregion,$region,$country); } sub process_accents { my @parms = @_; my $charset = shift(@parms); for (@parms) {  s/\=\n//; s/\=([0-9A-Fa-f]{2})/chr(hex($1))/ge; if ( $charset eq 'ANSEL' ) { s/C//g; s/A//g; s/E//g; s/I//g;
 s/O//g; s/U//g; s/Y//g; s/A//g; s/E//g; s/I//g; s/O//g; s/U//g; s/Y//g; s/A//g; s/E//g; s/I//g; s/O//g; s/U//g; s/A//g; s/E//g; s/I//g; s/O//g; s/U//g; s/c//g; s/a//g; s/e//g; s/i//g; s/o//g; s/u//g; s/y//g;
 s/a//g; s/e//g; s/i//g; s/o//g; s/u//g; s/y//g; s/a//g; s/e//g; s/i//g; s/o//g; s/u//g; s/a//g; s/e//g; s/i//g; s/o//g; s/u//g; } elsif ( $charset eq 'UTF-8' ) { s/\xC3\xA0//g; s/\xC3\xA2//g; s/\xC3\xA7//g; s/\xC3\xA8//g;
 s/\xC3\xA9//g; s/\xC3\xAA//g; s/\xC3\xAB//g; s/\xC3\xAE//g; s/\xC3\xAF//g; s/\xC3\xB4//g; s/\xC3\xB9//g; s/\xC3\xBB//g; s/\xC3\xA6//g; s/\xC3\x80//g; s/\xC3\x82//g; s/\xC3\x87//g; s/\xC3\x88//g; s/\xC3\x89//g; s/\xC3\x8A//g; s/\xC3\x8B//g; s/\xC3\x8E//g;
 s/\xC3\x8F//g; s/\xC3\x94//g; s/\xC3\x99//g; s/\xC3\x9B//g; s/\xC3\x86//g; } elsif ($charset eq 'IBMPC') { tr/\200-\207/\307\374\351\342\344\340\345\347/; tr/\210-\217/\353\350\357\354/; tr/\220-\227/\311\364\373\371/; tr/\230-\237/\377\326\334\245??/;
 tr/\240-\247/\341\355\363\372\321/; tr/\250-\257/\277?\253\273/; tr/ص//; } elsif ($charset eq 'MAC') { tr/\200-\217//; tr/\220-\237//; tr/\240-\257/ ߮ /; tr/\260-\277/   /; tr/\300-\317/        /;
 tr/\320-\337/         /; tr/\340-\357/    /; tr/\360-\377/       /; } elsif ($charset ne 'ANSI') { tr/\200-\217/E/; tr/\220-\237//; tr/\240-\245//; tr/\256\257\262\266\277//; } } return wantarray ? @parms : $parms[0]; }
 sub err_print { my $parms = shift; $parms=eval('"'.$text{$parms}.'"'); &log_text($parms); if ( ($msg_lang eq "fr") and ($msg_sys eq "DOS") ) { $parms =~ tr//\x80-\x9A/; } print "$parms"; } sub log_text { my $parms = shift;
 my ($ts,$tm,$th,$dj,$dm,$da,$sj,$aj,$is)=localtime(); printf LOG_FILE ("%2d:%02d:%02d %s\n",$th,$tm,$ts,$parms); } sub log_print { my $parms = shift; $parms=eval('"'.$text{$parms}.'"'); &log_text($parms); if ( !$in_progress ) { &mess_print( "m_working" ); $in_progress++; } print ".."; } sub mess_print { my $parms = shift;
 $parms=eval('"'.$text{$parms}.'"'); if ( ($msg_lang eq "fr") and ($msg_sys eq "DOS") ) { $parms =~ tr//\x80-\x9A/; } print "$parms"; } sub img_size { my ($file) = @_; my ( $x, $y ) = ( 0, 0 ); if ( defined($file) && open( STRM, "<$file" ) ) { binmode(STRM);
 if ( $file =~ /\.jpe?g$/i )  { ( $x, $y ) = &jpegsize( \*STRM ); } elsif ( $file =~ /\.gif$/i ) { ( $x, $y ) = &gifsize( \*STRM ); } elsif ( $file =~ /\.bmp$/i ) { ( $x, $y ) = &bmpsize( \*STRM ); } elsif ( $file =~ /\.png$/i ) { ( $x, $y ) = &pngsize( \*STRM ); } else { print "$file is not gif, xbm, jpg, jpeg or png (or has stupid name)"; } close STRM ; }
 return ( $x, $y ); } sub bmpsize { my ($BMP) = @_; my ( $a, $b ) = (0,0); my $buffer = ''; if ( defined($BMP) && read( $BMP, $buffer, 26 ) == 26 ) { ($a, $b) = unpack("x18VV", $buffer); return ( $a, $b ); } else { return ( 0, 0 ); } } sub gifsize { my ($GIF) = @_;
 my ( $a, $b, $c, $d ) = 0; my $type = ''; if ( defined($GIF) && read( $GIF, $type, 6 ) == 6 && $type =~ /GIF8[7,9]a/ && read( $GIF, $s, 4 ) == 4 ) { ( $a, $b, $c, $d ) = unpack( "C" x 4, $s ); return ( $b << 8 | $a, $d << 8 | $c ); } return ( 0, 0 ); } sub pngsize { my ($PNG)  = @_;
 my ($head) = ""; my ( $a, $b, $c, $d, $e, $f, $g, $h ) = 0; if ( defined($PNG) && read( $PNG, $head, 8 ) == 8 && $head eq "\x89\x50\x4e\x47\x0d\x0a\x1a\x0a" && read( $PNG, $head, 4 ) == 4 && read( $PNG, $head, 4 ) == 4 && $head eq "IHDR" && read( $PNG, $head, 8 ) == 8 ) { ( $a, $b, $c, $d, $e, $f, $g, $h ) = unpack( "C" x 8, $head );
 return ( $a << 24 | $b << 16 | $c << 8 | $d, $e << 24 | $f << 16 | $g << 8 | $h ); } return ( 0, 0 ); } sub jpegsize { my($JPEG) = @_; my($done)=0; my($c1,$c2,$ch,$s,$length, $dummy)=(0,0,0,0,0,0); my($a,$b,$c,$d); if(defined($JPEG)		&& read($JPEG, $c1, 1)	&&
 read($JPEG, $c2, 1)	&& ord($c1) == 0xFF		&& ord($c2) == 0xD8		){ while (ord($ch) != 0xDA && !$done) { while (ord($ch) != 0xFF) { return(0,0) unless read($JPEG, $ch, 1); } while (ord($ch) == 0xFF) { return(0,0) unless read($JPEG, $ch, 1); }
 if ((ord($ch) >= 0xC0) && (ord($ch) <= 0xC3)) { read ($JPEG, $dummy, 3) == 3 || return(0,0); read($JPEG, $s, 4) == 4 || return(0,0); ($a,$b,$c,$d)=unpack("C"x4,$s); return ($c<<8|$d, $a<<8|$b ); } else { read ($JPEG, $s, 2) == 2 || return(0,0); ($c1, $c2) = unpack("C"x2,$s); $length = $c1<<8|$c2;
 last if (!defined($length) || $length < 2); read($JPEG, $dummy, $length-2); } } } return (0,0); } sub scan_ged { my @parms = @_; my $dir = $parms[0]; print "=> $dir "; if (opendir (DIR, "$dir\\")) { my $i = 0; my $j = 0; while ($dir) { my @fich_dirs = grep!/^\.\.?$/,readdir DIR; closedir DIR;
 foreach  $fich_dir (@fich_dirs) { if ($fich_dir =~ /^.*\.ged$/) { $gedfile[$i] = "$dir\\$fich_dir"; $i++; } elsif (-d "$dir\\$fich_dir" && !-l "$dir\\$fich_dir") { push @dirs, "$dir\\$fich_dir"; $j++; if ($j > 50) { print "."; $j = 0; } } } $dir = pop @dirs;
 while ((!opendir(DIR, "$dir")) && $dir) { $dir = pop @dirs; } } undef(@fich_dirs); print "\n"; return $i; } print " error $!\n"; return -1; } sub Ged_place {	 my $placin = shift; my ($placout,$city,$orig) = ('','',''); if ( $placin =~ /^(.*?)\s*\?+$/ ) { $orig = "?"; $placin = $1; } if ( $placin =~ /$place_mask/i ) {	 @items = ("",$1,$2,$3,$4,$5,$6,$7,$8,$9);
 $city = $items[$pxi[2]]; if ($place_out) { my $i = 1; for (my $i=1; $i<10; $i++) {  if ($pxo[$i]) { $items[$pxi[$pxo[$i]]] =~ s/^\s*(.*?)\s*$/$1/; $items[$pxi[$pxo[$i]]] =~ s/,/;/; if ($placout) { $placout .= ', '.$items[$pxi[$pxo[$i]]]; } else { $placout = $items[$pxi[$pxo[$i]]]; } } } }
 else { $placout = $placin; } } else { $placin =~ /^([^;,\(\[\\\/]*)[;,\(\[\\\/]?/; $city = $1; $placout = $placin; } $placout = ucfirst($placout); $placout .= $orig; $city =~ s/^\s*(.*?)\s*$/$1/; $city =~ s/\s{2,}/ /g; $city = ucfirst($city);  $city .= $orig;
 return($placout,$city,$orig); } sub Ged_placx {	 my $place = shift; my $placx = ''; if ( $place =~ /^(.*?)\s*\?$/ ) { $place = $1; } if ( $place =~ /$place_masko/i ) { @items = ("",$1,$2,$3,$4,$5,$6,$7,$8,$9); if ($place_index) { for (my $i=1; $i<10; $i++) {  if ($items[$pxx[$i]]) { $items[$pxx[$i]] =~ s/^\s*(.*?)\s*$/$1/;
 if ($placx) { $placx .= ''.$items[$pxx[$i]]; } else { $placx = $items[$pxx[$i]]; } } } } } else {	  $place =~ /^([^;,\(\[\\\/]*)[;,\(\[\\\/]?/; $placx = $1;  } return($placx); } sub Ged_month { my $da=shift; for (my $m=0;$m<12;$m++) { $da =~ s/^(\s*|.*\s)$GED_mois[$m]\s+/$1$tab_mois[$m] /i; } return($da); }
 sub Ged_date {	 my $dag=shift; my ($gd,$gt)=('',''); if ( $dag =~ /^(@#DJULIAN@\s*)?(.*)$/ ) { $dag = $str_daty{'@#DJULIAN@'}.$2; } if ( $dag =~ /^~?BET\s+(.*?)\sAND\s+(.*?)$/i ) { my $da2=$2; my $gd1 = &Ged_date($1); my $gd2 = &Ged_date($da2);
 $gd = "$str_daty{'BET'} $gd1 $str_daty{'AND'} $gd2"; } elsif ( $dag =~ /^~?FROM\s+(.*?)\sTO\s+(.*?)$/i ) { my $da2=$2; my $gd1 = &Ged_date($1); my $gd2 = &Ged_date($da2); $gd = "$str_daty{'FROM'} $gd1 $str_daty{'TO'} $gd2"; } elsif ( $dag =~ /^~?(ABT|CIR|BEF|AFT|FROM|TO)(\s+.*)(\d{4})$/i ) { $gt = $str_daty{$1} if defined($1); if ($full_date) { $gd = $gt . &Ged_month($2) . $3; } else { $gd = $gt . $3; } }
 elsif ( $dag =~ /^~?(CAL|EST)(\s+.*)(\d{4})$/i ) { $gt = $str_daty{$1} if defined($1); if ($full_date) { $gd = &Ged_month($2) . $3 . " (" . $gt . ")"; } else { $gd = $gt . $3; } } elsif ( $dag =~ /^(.*)(\d{4})$/ ) { if ($full_date) { $gd = &Ged_month($1) . $2 ; } else { $gd = $2; } }
 elsif ( $dag =~ /^(@#DFRENCH R@\s*)?(\d{1,2})\/?\s*([a-zA-Z]+?)\/?\s*(an|AN|An|aN)?\s*([0-9IVX]*)$/ ) { my $dr = $2." ".$3." ".$4.$5; my $jr = $2; my $mr = $3; my $ar = $5; if ($mr =~ /^(vend|vnd).*?$/i) { $mr = "Vend" } elsif ($mr =~ /^bru.*?/i) { $mr = "Brum"; } elsif ($mr =~ /^fri.*?/i) { $mr = "Frim"; } elsif ($mr =~ /^niv.*?/i) { $mr = "Niv"; } elsif ($mr =~ /^plu.*?/i) { $mr = "Pluv"; } elsif ($mr =~ /^(vent|vnt).*?/i) { $mr = "Vent"; } elsif ($mr =~ /^ger.*?/i) { $mr = "Germ"; } elsif ($mr =~ /^flo.*?/i) { $mr = "Flor"; } elsif ($mr =~ /^pra.*?/i) { $mr = "Prai"; } elsif ($mr =~ /^mes.*?/i) { $mr = "Mess"; } elsif ($mr =~ /^the.*?/i) { $mr = "Ther"; } elsif ($mr =~ /^fru.*?/i) { $mr = "Fruc"; } elsif ($mr =~ /^(comp|cmp|sans).*?/i) { $mr = "Comp"; } else { $gd = $dr; } if (!$gd) { if ($ar =~ /^\d{1,2}$/) { $ar = substr("0".$ar,-2) } elsif ($ar eq "I") { $ar = "01"; }  elsif ($ar eq "II") { $ar = "02"; }  elsif ($ar eq "III") { $ar = "03"; }  elsif ($ar eq "IV") { $ar = "04"; }  elsif ($ar eq "V") { $ar = "05"; }  elsif ($ar eq "VI") { $ar = "06"; }  elsif ($ar eq "VII") { $ar = "07"; }  elsif ($ar eq "VIII") { $ar = "08"; }  elsif ($ar eq "IX") { $ar = "09"; }  elsif ($ar eq "X") { $ar = "10"; }  elsif ($ar eq "XI") { $ar = "11"; }  elsif ($ar eq "XII") { $ar = "12"; }  elsif ($ar eq "XIII") { $ar = "13"; }  elsif ($ar eq "XIV") { $ar = "14"; }  else { $gd = $dr; } }
 if (!$gd) { $ag = 1791+$ar; if ( $mr eq "Vend") { @lim = (9,22,22,22,23,22,22,22,23,23,23,23,24,23,23); } elsif ( $mr eq "Brum") { @lim = (10,22,22,22,23,22,22,22,23,23,23,23,24,23,23);  } elsif ( $mr eq "Frim") { @lim = (11,21,21,21,22,21,21,21,22,22,22,22,23,22,22); } elsif ( $mr eq "Niv") { @lim = (12,21,21,21,22,21,21,21,22,22,22,22,23,22,22); if ($jr gt 32-$lim[$ar]) { $ag++ }	} else { $ag++; 
 if ( $mr eq "Pluv") { @lim = (1,20,20,20,21,20,20,20,21,21,21,21,22,21,21); } elsif ( $mr eq "Vent") { @lim = (2,19,19,19,20,19,19,19,20,20,20,20,21,20,20); } elsif ( $mr eq "Germ") { @lim = (3,21,21,21,21,21,21,21,22,22,22,22,22,22,22); } elsif ( $mr eq "Flor") { @lim = (4,20,20,20,20,20,20,20,21,21,21,21,21,21,21); } elsif ( $mr eq "Prai") { @lim = (5,20,20,20,20,20,20,20,21,21,21,21,21,21,21); } elsif ( $mr eq "Mess") { @lim = (6,19,19,19,19,19,19,19,20,20,20,20,20,20,20); } elsif ( $mr eq "Ther") { @lim = (7,19,19,19,19,19,19,19,20,20,20,20,20,20,20); } elsif ( $mr eq "Fruc") { @lim = (8,18,18,18,18,18,18,18,19,19,19,19,19,19,19); } elsif ( $mr eq "Comp") { @lim = (9,17,17,17,17,17,17,17,18,18,18,18,18,18,18); } }
 $jg = $jr+$lim[$ar]-1; $mg = $lim[0]; my @mlen = (31,28,31,30,31,30,31,31,30,31,30,31); my $mlim = $mlen[$mg-1]; if (($mg == 2) and ($ar == 4 or $ar == 12)) { $mlim++; } if ($jg > $mlim) {	$jg = $jg-$mlim; $mg++; } if ($mg == 13) { $mg = 1; } $jg = substr("0".$jg,-2);
 if ($conv_date) { $gd = $jg." ".$tab_mois[$mg]." ".$ag; } else { $gd = $dr } if (!$full_date) { $gd = $ag; } } } else { $gd = $dag; } return($gd); } sub Ged_year {	 my $dag=shift; my $gy = ''; if ( $dag =~ /^BET\s+(.*)\sAND\s+(.*)$/i ) { my $da2 = $2; my $gy1 = &Ged_year($1); my $gy2 = &Ged_year($da2);
 $gy = int( ($gy1+$gy2) / 2 ); } elsif ( $dag =~ /^FROM\s+(.*)\sTO\s+(.*)$/i ) { my $da2 = $2; my $gy1 = &Ged_year($1); my $gy2 = &Ged_year($da2); $gy = int( ($gy1+$gy2) / 2 ); } elsif ( $dag =~ /^(.*)(\d{4})$/ ) { $gy = $2; } elsif ( $dag =~ /(\d{4})/ ) { $gy = $1; }
 elsif ( $dag =~ /^(@#DFRENCH R@\s*)?(\d{1,2})\/?\s*([a-zA-Z]+?)\/?\s*(an|AN|An|aN)?\s*([0-9IVX]*)$/ ) { my $mr = $3; my $ar = $5; if ($ar =~ /^\d{1,2}$/) { $ar = substr("0".$ar,-2) } elsif ($ar eq "I") { $ar = "01"; }  elsif ($ar eq "II") { $ar = "02"; }  elsif ($ar eq "III") { $ar = "03"; }  elsif ($ar eq "IV") { $ar = "04"; }  elsif ($ar eq "V") { $ar = "05"; }  elsif ($ar eq "VI") { $ar = "06"; }  elsif ($ar eq "VII") { $ar = "07"; }  elsif ($ar eq "VIII") { $ar = "08"; }  elsif ($ar eq "IX") { $ar = "09"; }  elsif ($ar eq "X") { $ar = "10"; }  elsif ($ar eq "XI") { $ar = "11"; }  elsif ($ar eq "XII") { $ar = "12"; }  elsif ($ar eq "XIII") { $ar = "13"; }  elsif ($ar eq "XIV") { $ar = "14"; }  $gy = 1791+$ar;
 if ( $mr !~ /^(vend|vnd|bru|fri|niv).*?/i) { $gy++; } } return($gy); } sub print_result { $i_save = 0; if (open( SAV_FILE, "gedisave.txt" )) { while (chomp($file_in = <SAV_FILE>) and ($i_save < $save_max)) { if ($file_in ne $file_ged) { $file_save[$i_save] = $file_in; $i_save++;} } close SAV_FILE ; }
 unless (open(SAV_FILE, ">gedisave.txt")) { die "\nCouldn't open output file gedisave.txt\n"; } $file_ged =~ s/\//\\/g; print SAV_FILE "$file_ged\n"; for ($j=0;$j<$i_save;$j++) { print SAV_FILE "$file_save[$j]\n"; } close SAV_FILE ; $file_ged =~ s/\\/\//g; $file_opt =~ s/\\/\//g;
 $out_dir =~ s/\\/\//g; unless (open(OUT_FILE, ">result.html")) { die "\nCouldn't open output file result.html\n"; } print OUT_FILE "<!doctype html public \"-//w3c//dtd html 4.0 transitional//en\">\n"; print OUT_FILE "<HTML><HEAD>\n";
 print OUT_FILE "   <meta http-equiv=\"Content-Type\" content=\"text/html; charset=iso-8859-1\">\n"; print OUT_FILE "   <meta name=\"GENERATOR\" content=\"Gedail V$version\">\n"; print OUT_FILE "   <meta name=\"AUTHOR\"	content=\"J-F BAQUET\">\n"; print OUT_FILE "   <TITLE>".html(eval('"'.$text{'h_last_exec'}.'"'))."</TITLE>"; print OUT_FILE "</HEAD>\n<BODY BACKGROUND=\"Pics/Baqbg.jpg\"><CENTER>";
 print OUT_FILE "<FONT size=+2>&nbsp\n<P>".html(eval('"'.$text{'h_last_exec'}.'"'))." ($curr_date - $hmin)</P></FONT>"; print OUT_FILE "</CENTER>\n<BLOCKQUOTE><FONT size=+2><P>".html(eval('"'.$text{'h_ged_file'}.'"'))." </FONT>"; print OUT_FILE "<a href=\"file://$file_ged\">$file_ged</a></P>\n<FONT size=+2><P>";
 print OUT_FILE html(eval('"'.$text{'h_opt_file'}.'"'))." </FONT>"; if ($file_opt) { print OUT_FILE "<a href=\"file://$file_opt\">$file_opt</a></P>\n"; } else { print OUT_FILE html(eval('"'.$text{'h_def_opt'}.'"'))."</P>\n"; } if ($ind_opt || $loc_opt || $nam_opt || $gen_number) {	 print OUT_FILE "<FONT size=+2><P>".html(eval('"'.$text{'h_web_creat'}.'"'))." </FONT>"; print OUT_FILE "<a href=\"file://$out_dir/index.html\">$out_dir</a></P>\n"; }
 if ($fli_opt) {	 print OUT_FILE "<FONT size=+2><P>".html(eval('"'.$text{'h_fli_opt'}.'"'))." </FONT>"; print OUT_FILE "<a href=\"file://$out_dir/$flif_name.csv\">$out_dir/$flif_name.csv</a></P>\n"; } if ($mli_opt) {	 print OUT_FILE "<FONT size=+2><P>".html(eval('"'.$text{'h_mli_opt'}.'"'))." </FONT>"; print OUT_FILE " </FONT><a href=\"file://$out_dir/$mlif_name.csv\">$out_dir/$mlif_name.csv</a></P>\n"; }
 print OUT_FILE "</BLOCKQUOTE></BODY></HTML>"; close OUT_FILE ; } sub sosa_long { my $sosa=shift; my $out = substr("00000000000000000000",0,20-length($sosa)).$sosa;  return($out); } sub sosa_x2 { my $sosa=shift; my $out = 2 * $sosa; if ($out =~ /e/) {  $sosa_str1 = 2*substr($sosa,length($sosa)-9,9);
 $sosa_str2 = 2*substr($sosa,0,length($sosa)-9); if ($sosa_str1 >= 1000000000) {	$sosa_str1 -= 1000000000; $sosa_str2++; } $out = $sosa_str2.$sosa_str1; }	 return($out); } sub sosa_inc { my $sosa=shift; my $out = $sosa+1; if ($out =~ /e/) {  $sosa_str1 = substr($sosa,length($sosa)-9,9)+1;
 $sosa_str2 = substr($sosa,0,length($sosa)-9); if ($sosa_str1 >= 1000000000) {	$sosa_str1 -= 1000000000; $sosa_str2++; } $out = $sosa_str2.$sosa_str1; }	 return($out); } sub img_prop { my $photo_file = shift; my ( $i_width, $i_height ) = &img_size("$out_dir/$photos_path/$photo_file"); my $i_path = "";
 if ( $i_width * $i_height ) {	 $i_path = '../'.$photos_path.'/'.$photo_file; if ( $i_width > $photo_max_width ) { $i_height = int( ( $i_height * $photo_max_width ) / $i_width ); $i_width  = $photo_max_width; } if ( $i_height > $photo_max_height ) { $i_width  = int( ( $i_width * $photo_max_height ) / $i_height );
 $i_height = $photo_max_height; } } else {  &err_print('e_inva_pict'); } return ( $i_width, $i_height, $i_path ); } sub image_in { my $i_id = shift; my $i_file = shift; if ($obj_type{$i_id} eq 'web') { if ($i_file =~ /^(http|ftp)\:.*$/i) { $obj_path{$i_id}  = $i_file; }
 elsif ($i_file =~ /^(.*)\\([^\\]*?)\.(\w*)$/) { $i_name = $2; $i_type = $3; $pages_dir = "$out_dir/$pages_path"; if ( !opendir PAGES, "$pages_dir" ) { mkdir( $pages_dir, 0755 ); } if ( -e "$i_file" ) { $rc = &bincopy( "$i_file", "$pages_dir/$i_name.$i_type" );
 if ($rc==1) { &log_text("$i_file copied $rc"); $file_outnum++; } $obj_path{$i_id}  = '../'.$pages_path.'/'.$i_name.'.'.$i_type; } } } elsif ($i_file =~ /^(.*(\\|\/))([^\\\/]*?)\.(\w*)$/) { $i_dir = $1; $i_name = $3; $i_type = $4; $i_name1 = $i_name; if ($i_name1 !~ /^[!@#$%^&()_\-{}`\sa-zA-Z0-9]*$/i) {  $i_name1 =~ tr/\*\+\|\=\?\[\]\.\;\:\'\"\<\>/AAAAAACEEEEIIIINOOOOOUUUUYaaaaaaceeeeiiiinooooouuuuyy______________/; }
 if ( -e "$out_dir/$photos_path/gi_$i_name1.jpg" ) { $photo_file = "gi_$i_name1.jpg"; if ( -e "$out_dir/$photos_path/tn_$i_name1.jpg" ) { $obj_thumb{$i_id} = "../$photos_path/tn_$i_name1.jpg"; } } else { $photo_file = "$i_name.$i_type"; if (!$i_dir) { $i_file = $ged_dir.$i_file; } if ( !-e "$out_dir/$photos_path/$photo_file" ) { $rc = &bincopy( "$i_file", "$out_dir/$photos_path/$photo_file" );
 if ($rc==1) { &log_text("$i_file copied $rc"); $file_outnum++; } } } ( $obj_width{$i_id}, $obj_height{$i_id}, $obj_path{$i_id} ) = &img_prop("$photo_file"); } return; } sub url_encode { my($string) = $_[0]; $string =~ s/%/%25/g; $string =~ s/\s/%20/g; $string =~ s/!/%21/g; $string =~ s/\?/%22/g; $string =~ s/#/%23/g;
 $string =~ s/\$/%24/g; $string =~ s/&/%26/g; $string =~ s/\'/%27/g; $string =~ s/\(/%28/g; $string =~ s/\)/%29/g; $string =~ s/\*/%2A/g; $string =~ s/\+/%2B/g; $string =~ s/,/%2C/g; $string =~ s/\-/%2D/g; $string =~ s/\./%2E/g; $string =~ s/\//%2F/g; $string =~ s/:/%3A/g;
 $string =~ s/;/%3B/g; $string =~ s/</%3C/g; $string =~ s/=/%3D/g; $string =~ s/>/%3E/g; $string =~ s/\?/%3F/g; $string =~ s/@/%40/g; $string =~ s/\[/%5B/g; $string =~ s/\\/%5C/g; $string =~ s/\]/%5D/g; $string =~ s/\^/%5E/g; $string =~ s/_/%5F/g; $string =~ s/`/%60/g;
 $string =~ s/\{/%7B/g; $string =~ s/\|/%7C/g; $string =~ s/\}/%7D/g; $string =~ s/~/%7E/g; $string =~ s//%A0/g; $string =~ s//%A1/g; $string =~ s//%A2/g; $string =~ s//%A3/g; $string =~ s//%A4/g; $string =~ s//%A5/g; $string =~ s//%A6/g; $string =~ s//%A7/g;
 $string =~ s//%A8/g; $string =~ s//%A9/g; $string =~ s//%AA/g; $string =~ s//%AB/g; $string =~ s//%AC/g; $string =~ s//%AD/g; $string =~ s//%AE/g; $string =~ s//%AF/g; $string =~ s//%B0/g; $string =~ s//%B1/g; $string =~ s//%B2/g; $string =~ s//%B3/g;
 $string =~ s//%B4/g; $string =~ s//%B5/g; $string =~ s//%B6/g; $string =~ s//%B7/g; $string =~ s//%B8/g; $string =~ s//%B9/g; $string =~ s//%BA/g; $string =~ s//%BB/g; $string =~ s//%BC/g; $string =~ s//%BD/g; $string =~ s//%BE/g; $string =~ s//%BF/g;
 $string =~ s//%C0/g; $string =~ s//%C1/g; $string =~ s//%C2/g; $string =~ s//%C3/g; $string =~ s//%C4/g; $string =~ s//%C5/g; $string =~ s//%C6/g; $string =~ s//%C7/g; $string =~ s//%C8/g; $string =~ s//%C9/g; $string =~ s//%CA/g; $string =~ s//%CB/g;
 $string =~ s//%CC/g; $string =~ s//%CD/g; $string =~ s//%CE/g; $string =~ s//%CF/g; $string =~ s//%D0/g; $string =~ s//%D1/g; $string =~ s//%D2/g; $string =~ s//%D3/g; $string =~ s//%D4/g; $string =~ s//%D5/g; $string =~ s//%D6/g; $string =~ s//%D7/g;
 $string =~ s//%D8/g; $string =~ s//%D9/g; $string =~ s//%DA/g; $string =~ s//%DB/g; $string =~ s//%DC/g; $string =~ s//%DD/g; $string =~ s//%DE/g; $string =~ s//%DF/g; $string =~ s//%E0/g; $string =~ s//%E1/g; $string =~ s//%E2/g; $string =~ s//%E3/g;
 $string =~ s//%E4/g; $string =~ s//%E5/g; $string =~ s//%E6/g; $string =~ s//%E7/g; $string =~ s//%E8/g; $string =~ s//%E9/g; $string =~ s//%EA/g; $string =~ s//%EB/g; $string =~ s//%EC/g; $string =~ s//%ED/g; $string =~ s//%EE/g; $string =~ s//%EF/g;
 $string =~ s//%F0/g; $string =~ s//%F1/g; $string =~ s//%F2/g; $string =~ s//%F3/g; $string =~ s//%F4/g; $string =~ s//%F5/g; $string =~ s//%F6/g; $string =~ s//%F7/g; $string =~ s//%F8/g; $string =~ s//%F9/g; $string =~ s//%FA/g; $string =~ s//%FB/g;
 $string =~ s//%FC/g; $string =~ s//%FD/g; $string =~ s//%FE/g; $string =~ s//%FF/g; return($string); } sub SoundEx   { my($WordStr, $LengthOption, $LangOption) = @_; my($CurChar, $LastChar, $WSLen, $FirstLetter); if($LengthOption > 10) { $LengthOption = 10; } elsif(!$LengthOption or ($LengthOption < 4)) { $LengthOption = 4; } if(!$WordStr) { return(""); } $WordStr =~ tr//AAAAAACEEEEIIIINOOOOOUUUUYaaaaaaceeeeiiiinooooouuuuyy/;
 $WordStr = uc($WordStr); $WordStr =~ s/[^A-Z]//g; $FirstLetter = substr($WordStr,0,1); if( $LangOption eq 'fr') { $WordStr =~ tr/AEIOUYHWBPCKQDTLMNRGJXZSFV/00000000112223345567788899/; } else { $WordStr =~ tr/AEIOUYHWBPFVCSGJKQXZDTLMNR/00000000111122222222334556/; } $WSLen = length($WordStr);
 $LastChar = substr($WordStr, 0, 1); for($i = 1; $i < $WSLen;$i++) { $CurChar = substr($WordStr,$i,1); if($CurChar eq $LastChar) { substr($WordStr,$i,1," "); } else { $LastChar = $CurChar; } } $WordStr = substr($WordStr,1); $WordStr =~ s/(0|\s)//g;
 $WordStr = substr($FirstLetter.$WordStr."0000000000",0,$LengthOption); return($WordStr); } sub set_place_mask   { if ( $place_format =~ /^.*?(\d)(?:.*?(\d))?(?:.*?(\d))?(?:.*?(\d))?(?:.*?(\d))?(?:.*?(\d))?(?:.*?(\d))?(?:.*?(\d))?(?:.*?(\d))?/ ) { $px[$1] = $pxi[$1] = 1; $px[$2] = $pxi[$2] = 2; $px[$3] = $pxi[$3] = 3;
 $px[$4] = $pxi[$4] = 4; $px[$5] = $pxi[$5] = 5; $px[$6] = $pxi[$6] = 6; $px[$7] = $pxi[$7] = 7; $px[$8] = $pxi[$8] = 8; $px[$9] = $pxi[$9] = 9; $px[0] = $pxi[0] = 0; $pxo[0] = 0;  $pxo[1] = $1; $pxo[2] = $2; $pxo[3] = $3;  $pxo[4] = $4;  $pxo[5] = $5;  $pxo[6] = $6; 
 $pxo[7] = $7;  $pxo[8] = $8;  $pxo[9] = $9;  $place_mask = $place_format;  $place_mask =~ s/\s/\\s/g; $place_mask =~ s/\(/\\\(/g; $place_mask =~ s/\)/\\\)/g; $place_mask =~ s/\\\[/\\/g; $place_mask =~ s/\\\]/\\/g; $place_mask =~ s/\[/\(\?\:/g;
 $place_mask =~ s/\]/\)\?/g; $place_mask =~ s/\\/\\[/g; $place_mask =~ s/\\/\\]/g; $place_mask =~ s/\d/(.*?)/g; $place_mask = '^\s*'.$place_mask.'\s*$'; }									  else {  $px[2] = $pxi[2] = 1;  $pxo[1] = 2; $place_mask = '^([^;,\(\[]*)[;,\(\[]?.*?$';     } 
 $place_masko = $place_mask; if ($index_city) { $place_index = "2"; } if ($place_format and ($place_out =~ /^.*?([1-9])(?:.*?(\d))?(?:.*?(\d))?(?:.*?(\d))?(?:.*?(\d))?(?:.*?(\d))?(?:.*?(\d))?(?:.*?(\d))?(?:.*?(\d))?/ )) { $px[0] = $pxo[0] = 0; $px[1] = $pxo[1] = $1;
 $px[2] = $pxo[2] = $2; $px[3] = $pxo[3] = $3;  $px[4] = $pxo[4] = $4;  $px[5] = $pxo[5] = $5;  $px[6] = $pxo[6] = $6;  $px[7] = $pxo[7] = $7;  $px[8] = $pxo[8] = $8;  $px[9] = $pxo[9] = $9; $place_masko = '^(.*?)'; for (my $i=2; $i<10; $i++) { if ($pxo[$i]) { $place_masko .= ',\\s(.*?)'; } }
 $place_masko .= '$'; }				 if ( $place_index =~ /^.*?([1-9])(?:.*?(\d))?(?:.*?(\d))?(?:.*?(\d))?(?:.*?(\d))?(?:.*?(\d))?(?:.*?(\d))?(?:.*?(\d))?(?:.*?(\d))?/ ) { $pxx[0] = 0; $pxx[1] = $1; $pxx[2] = $2; $pxx[3] = $3;  $pxx[4] = $4;  $pxx[5] = $5;  $pxx[6] = $6; 
 $pxx[7] = $7;  $pxx[8] = $8;  $pxx[9] = $9;  my $i = 1; while ($pxx [$i]) {  my $j = 1; while ($pxo[$j]) { if ($pxo[$j] == $pxx[$i] ) { $pxx[$i] = $j;  last;} $j++; } $i++; } }									  else { $pxx[1] = 2; } return; } sub print_repo   { $jvs_file =  $out_dir.'/'.$sour_path.'/'.$file_pref.'_repo.jvs';
 $out_opt = 2; &open_new("$jvs_file"); &cond_print("function repoaffi(repoid) {\n");  &cond_print(" switch(repoid) {\n"); foreach $repo_id ( keys %repo ) { $repo{$repo_id} =~ s/\'/&#39;/g; $repo{$repo_id} =~ s/\"/&#34;/g; my ($repo_name,$repo_addr,$repo_phon,$repo_note) = split(//,$repo{$repo_id}); $repo_name =~ s/(http:\/\/\S*)/<A HREF='$1' TARGET=_blank>$1<\/A>/gi;
 if ($repo_phon) { $repo_phon = "<img src=\"$pics_path/phone.gif\" align=\"middle\" border=0> ".$repo_phon; } &cond_print(" case '$repo_id' : repo(\'$repo_name\',\'$repo_addr\',\'$repo_phon\',\'$repo_note\');break;\n");  } &cond_print(" default : return;\n");  &cond_print(" }\n"); &cond_print("}\n"); &cond_out("$jvs_file");  return; } sub print_source   { if (!$sour_exist) { if ( $tpl_dir =~ /^([A-Za-z]\d{3})(-[A-Za-z]{2})?$/ ) { $tpl_dir = $1.'-'.$tablang[0];		 }
 $/ = $save_nl; &get_template("sour-tpl",'',''); $/ =$ged_file_nl; $sour_dir = "$out_dir/$sour_path"; if ( !opendir SOURCES, "$sour_dir" ) { mkdir( $sour_dir, 0755 ); } if ( $php and (-e "uz.php" ))  { $rc = &bincopy( "uz.php", "$sour_dir/uz.php" );
 if ($rc==1) { &log_text("sour_dir/uz.php copied"); $file_outnum++; } } $sour_count = 0; $file_id     = $sour_name; $first_file   = $sour_name;  $prev_file  = $first_file; $next_file   = $file_id; $sour_exist = 1; $out_opt = 1; } $sour_file{$sour_id} = $file_id; if ($sour_count and ($sour_count+$sour_lines >= $sour_size)) {  &cond_out("$sour_dir/$file_id.html");
 $prev_file = $file_id; $file_id++; $sour_count = 0;		 } if (!$sour_count and $sour_lines) {  &open_new("$sour_dir/$file_id.html"); $next_file++; for ( $i = $page_beg ; $i < $page_beg + $page_cnt ; $i++ ) { &cond_print("$template[$i]"); } }
 for ( $i = $block_beg ; $i < $block_beg + $block_cnt ; $i++ ) { &cond_print("$template[$i]"); } for ( $i = $item_beg ; $i < $item_beg + $item_cnt ; $i++ ) { if (!$vartemp[$i]) { &cond_print("$template[$i]"); next; } $line = $template[$i]; $line =~ s/#sour_id/$sour_id/g; $line =~ s/#sour_refn/$sour_item{'REFN'}/g; $line =~ s/#sour_abbr/$sour_item{'ABBR'}/g; $line =~ s/#sour_auth/$sour_item{'AUTH'}/g;  $sour_titl = ' '.$sour_item{'TITL'};  $sour_titl =~ s/(http:\/\/\S*)/<A HREF='$1' TARGET=_blank><img src="$pics_path\/web.gif" STYLE="vertical-align:middle" border=0><\/A> $1/gi; 
 $sour_titl =~ s/\s(?:[a-z]:.*?)?([a-z0-9_-~]*)\.(jpg|gif|png)/ $1.$2 <A HREF='$1.$2' TARGET=_blank><img src="$pics_path\/photo.gif" STYLE="vertical-align:middle" border=0><\/A>/gi; $sour_titl =~ s/\s(?:[a-z]:.*?)?([a-z0-9_-~]*)\.(html?)/ $1.$2 <A HREF='$1.$2' TARGET=_blank><img src="$pics_path\/oeil.gif" STYLE="vertical-align:middle" border=0><\/A>/gi; $line =~ s/#sour_titl/$sour_titl/g; if ($sour_item{'PUBL'}) { $sour_publ = $sour_item{'PUBL'}; $sour_publ =~ s/(http:\/\/\S*)/<A HREF='$1' TARGET=_blank>$1<\/A>/gi;
 $line =~ s/#sour_publ/\($sour_publ\)/g;  }  else  { $line =~ s/#sour_publ//g; } if ($sour_item{'REPO'}) {  $sour_repo = $sour_item{'REPO'}; if ($sour_repo =~ /^@(.*?)@(.*)$/) { $sour_repo = "<A HREF=\"javascript:repoaffi('$1');\">$1 <img src=\"$pics_path/shelf.gif\" align=\"middle\" border=0><\/A> $2"; } else { $sour_repo =~ s/(http:\/\/\S*)/<A HREF='$1' TARGET=_blank>$1<\/A>/gi; } $line =~ s/#sour_repo/$sour_repo/g;  } 
 else  { $line =~ s/#sour_repo//g; } $line =~ s/#sour_caln/$sour_item{'CALN'}/g; $line =~ s/#sour_text/$sour_item{'TEXT'}/g; $line =~ s/#sour_note/$sour_item{'NOTE'}/g;  &cond_print("$line");  } for ( $i = $fblock_beg ; $i < $fblock_beg + $fblock_cnt ; $i++ ) { &cond_print("$template[$i]"); }
 $sour_count += $sour_lines; $sour_lines = 0; %sour_item = ''; return; } 1;
